@@ -1,16 +1,27 @@
 /*====================================================================*
  -  Copyright (C) 2001 Leptonica.  All rights reserved.
- -  This software is distributed in the hope that it will be
- -  useful, but with NO WARRANTY OF ANY KIND.
- -  No author or distributor accepts responsibility to anyone for the
- -  consequences of using this software, or for whether it serves any
- -  particular purpose or works at all, unless he or she says so in
- -  writing.  Everyone is granted permission to copy, modify and
- -  redistribute this source code, for commercial or non-commercial
- -  purposes, with the following restrictions: (1) the origin of this
- -  source code must not be misrepresented; (2) modified versions must
- -  be plainly marked as such; and (3) this notice may not be removed
- -  or altered from any source or modified source distribution.
+ -
+ -  Redistribution and use in source and binary forms, with or without
+ -  modification, are permitted provided that the following conditions
+ -  are met:
+ -  1. Redistributions of source code must retain the above copyright
+ -     notice, this list of conditions and the following disclaimer.
+ -  2. Redistributions in binary form must reproduce the above
+ -     copyright notice, this list of conditions and the following
+ -     disclaimer in the documentation and/or other materials
+ -     provided with the distribution.
+ -
+ -  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ -  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ -  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ -  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL ANY
+ -  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+ -  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+ -  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+ -  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
+ -  OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ -  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ -  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *====================================================================*/
 
 /*
@@ -29,7 +40,6 @@
 
 static void MakePtas(l_int32 i, l_int32 npts, PTA **pptas, PTA **pptad);
 
-
 static const l_int32  x1[] =  {  300,   300,   300,    95,   32 };
 static const l_int32  y1[] =  { 1200,  1200,  1250,  2821,  934 };
 static const l_int32  x2[] =  { 1200,  1200,  1125,  1432,  487 };
@@ -39,7 +49,6 @@ static const l_int32  y3[] =  {  200,   200,   200,   657,   67 };
 static const l_int32  x4[] =  { 1200,  1200,  1125,  1432,  487 };
 static const l_int32  y4[] =  {  200,   200,   200,   242,   84 };
 
-
 static const l_int32  xp1[] = {  500,   300,   350,   117,   32 };
 static const l_int32  yp1[] = { 1700,  1400,  1100,  2629,  934 };
 static const l_int32  xp2[] = {  850,   400,  1100,  1664,  487 };
@@ -49,16 +58,14 @@ static const l_int32  yp3[] = {  300,   300,   400,   490,   83 };
 static const l_int32  xp4[] = {  850,   1000, 1100,  1664,  487 };
 static const l_int32  yp4[] = {  350,    350,  400,   532,  114 };
 
-
-
-main(int    argc,
-     char **argv)
+int main(int    argc,
+         char **argv)
 {
-PIX   *pix1, *pix2, *pixs1, *pixs2, *pixs3, *pixb1, *pixb2, *pixb3, *pixbs;
-PIX   *pixg2, *pixg3, *pixr2, *pixr3;
-PIX         *pixc1, *pixc2, *pixc3, *pixcs1, *pixcs2, *pixcs3;
-PIX         *pixd, *pixt1, *pixt2, *pixt3;
-PTA         *ptas1, *ptas2, *ptas3, *ptad1, *ptad2, *ptad3;
+PIX          *pixs2, *pixs3, *pixb1, *pixb2, *pixb3;
+PIX          *pixr2, *pixr3;
+PIX          *pixc1, *pixc2, *pixc3, *pixcs1, *pixcs2, *pixcs3;
+PIX          *pixd, *pixt1, *pixt2, *pixt3;
+PTA          *ptas1, *ptas2, *ptas3, *ptad1, *ptad2, *ptad3;
 L_REGPARAMS  *rp;
 
     if (regTestSetup(argc, argv, &rp))
@@ -83,22 +90,6 @@ L_REGPARAMS  *rp;
     pixDestroy(&pixs3);
     pixDestroy(&pixb1);
     pixDestroy(&pixb2);
-    pixDestroy(&pixb3);
-
-    pixd = pixCreate(900, 400, 32);
-    pixSetAll(pixd);
-    pixs3 = pixScaleGammaXform(pixc3, 2.0, 0.4, 0.4, 0.7);
-    pixs2 = pixScaleGammaXform(pixc2, 2.0, 0.5, 0.5, 0.3);
-    pixb1 = pixBlendWithGrayMask(pixd, pixs3, NULL, 100, 100);
-    pixb2 = pixBlendWithGrayMask(pixb1, pixs2, NULL, 300, 130);
-    pixb3 = pixBlendWithGrayMask(pixb2, pixs3, NULL, 600, 160);
-    regTestWritePixAndCheck(rp, pixb3, IFF_PNG);  /* 1 */
-    pixDisplayWithTitle(pixb3, 900, 100, NULL, rp->display);
-    pixDestroy(&pixd);
-    pixDestroy(&pixs3);
-    pixDestroy(&pixs2);
-    pixDestroy(&pixb1);
-    pixDestroy(&pixb2);
     pixDestroy(&pixb3);
 
         /* Test alpha blend rotation */
@@ -108,7 +99,7 @@ L_REGPARAMS  *rp;
     pixr2 = pixRotateWithAlpha(pixc2, +0.3, NULL, 1.0);
     pixb3 = pixBlendWithGrayMask(pixd, pixr3, NULL, 100, 100);
     pixb2 = pixBlendWithGrayMask(pixb3, pixr2, NULL, 400, 100);
-    regTestWritePixAndCheck(rp, pixb2, IFF_PNG);  /* 2 */
+    regTestWritePixAndCheck(rp, pixb2, IFF_PNG);  /* 1 */
     pixDisplayWithTitle(pixb2, 500, 100, NULL, rp->display);
     pixDestroy(&pixd);
     pixDestroy(&pixr3);
@@ -116,20 +107,6 @@ L_REGPARAMS  *rp;
     pixDestroy(&pixb3);
     pixDestroy(&pixb2);
 
-    pixd = pixCreate(1200, 800, 32);
-    pixSetAll(pixd);
-    pixr2 = pixRotateGammaXform(pixc2, 2.0, +0.3, 1.0);
-    pixr3 = pixRotateGammaXform(pixc3, 2.0, -0.3, 1.0);
-    pixb3 = pixBlendWithGrayMask(pixd, pixr3, NULL, 100, 100);
-    pixb2 = pixBlendWithGrayMask(pixb3, pixr2, NULL, 400, 100);
-    regTestWritePixAndCheck(rp, pixb2, IFF_PNG);  /* 3 */
-    pixDisplayWithTitle(pixb2, 500, 100, NULL, rp->display);
-    pixDestroy(&pixd);
-    pixDestroy(&pixr2);
-    pixDestroy(&pixr3);
-    pixDestroy(&pixb2);
-    pixDestroy(&pixb3);
-
     pixcs1 = pixScale(pixc1, 0.35, 0.35);
     pixcs2 = pixScale(pixc2, 0.55, 0.55);
     pixcs3 = pixScale(pixc3, 0.65, 0.65);
@@ -146,34 +123,7 @@ L_REGPARAMS  *rp;
     pixb1 = pixBlendWithGrayMask(pixd, pixt1, NULL, -250, 20);
     pixb2 = pixBlendWithGrayMask(pixb1, pixt2, NULL, -150, -250);
     pixb3 = pixBlendWithGrayMask(pixb2, pixt3, NULL, -100, 220);
-    regTestWritePixAndCheck(rp, pixb3, IFF_PNG);  /* 4 */
-    pixDisplayWithTitle(pixb3, 100, 100, NULL, rp->display);
-    pixDestroy(&pixd);
-    pixDestroy(&pixb1);
-    pixDestroy(&pixb2);
-    pixDestroy(&pixb3);
-    pixDestroy(&pixt1);
-    pixDestroy(&pixt2);
-    pixDestroy(&pixt3);
-    ptaDestroy(&ptas1);
-    ptaDestroy(&ptas2);
-    ptaDestroy(&ptas3);
-    ptaDestroy(&ptad1);
-    ptaDestroy(&ptad2);
-    ptaDestroy(&ptad3);
-
-    pixd = pixCreate(800, 900, 32);
-    pixSetAll(pixd);
-    MakePtas(2, 3, &ptas1, &ptad1);
-    MakePtas(4, 3, &ptas2, &ptad2);
-    MakePtas(3, 3, &ptas3, &ptad3);
-    pixt1 = pixAffinePtaGammaXform(pixcs1, 2.0, ptad1, ptas1, 1.0, 300);
-    pixt2 = pixAffinePtaGammaXform(pixcs2, 2.0, ptad2, ptas2, 0.8, 400);
-    pixt3 = pixAffinePtaGammaXform(pixcs3, 2.0, ptad3, ptas3, 0.7, 300);
-    pixb1 = pixBlendWithGrayMask(pixd, pixt1, NULL, -250, 20);
-    pixb2 = pixBlendWithGrayMask(pixb1, pixt2, NULL, -150, -250);
-    pixb3 = pixBlendWithGrayMask(pixb2, pixt3, NULL, -100, 220);
-    regTestWritePixAndCheck(rp, pixb3, IFF_PNG);  /* 5 */
+    regTestWritePixAndCheck(rp, pixb3, IFF_PNG);  /* 2 */
     pixDisplayWithTitle(pixb3, 100, 100, NULL, rp->display);
     pixDestroy(&pixd);
     pixDestroy(&pixb1);
@@ -201,34 +151,7 @@ L_REGPARAMS  *rp;
     pixb1 = pixBlendWithGrayMask(pixd, pixt1, NULL, -150, 20);
     pixb2 = pixBlendWithGrayMask(pixb1, pixt2, NULL, -50, -250);
     pixb3 = pixBlendWithGrayMask(pixb2, pixt3, NULL, -100, 220);
-    regTestWritePixAndCheck(rp, pixb3, IFF_PNG);  /* 6 */
-    pixDisplayWithTitle(pixb3, 300, 100, NULL, rp->display);
-    pixDestroy(&pixd);
-    pixDestroy(&pixb1);
-    pixDestroy(&pixb2);
-    pixDestroy(&pixb3);
-    pixDestroy(&pixt1);
-    pixDestroy(&pixt2);
-    pixDestroy(&pixt3);
-    ptaDestroy(&ptas1);
-    ptaDestroy(&ptas2);
-    ptaDestroy(&ptas3);
-    ptaDestroy(&ptad1);
-    ptaDestroy(&ptad2);
-    ptaDestroy(&ptad3);
-
-    pixd = pixCreate(900, 900, 32);
-    pixSetAll(pixd);
-    MakePtas(2, 4, &ptas1, &ptad1);
-    MakePtas(4, 4, &ptas2, &ptad2);
-    MakePtas(3, 4, &ptas3, &ptad3);
-    pixt1 = pixProjectivePtaGammaXform(pixcs1, 2.0, ptad1, ptas1, 1.0, 300);
-    pixt2 = pixProjectivePtaGammaXform(pixcs2, 2.0, ptad2, ptas2, 0.8, 400);
-    pixt3 = pixProjectivePtaGammaXform(pixcs3, 2.0, ptad3, ptas3, 0.7, 400);
-    pixb1 = pixBlendWithGrayMask(pixd, pixt1, NULL, -150, 20);
-    pixb2 = pixBlendWithGrayMask(pixb1, pixt2, NULL, -50, -250);
-    pixb3 = pixBlendWithGrayMask(pixb2, pixt3, NULL, -100, 220);
-    regTestWritePixAndCheck(rp, pixb3, IFF_PNG);  /* 7 */
+    regTestWritePixAndCheck(rp, pixb3, IFF_PNG);  /* 3 */
     pixDisplayWithTitle(pixb3, 300, 100, NULL, rp->display);
     pixDestroy(&pixd);
     pixDestroy(&pixb1);
@@ -256,34 +179,7 @@ L_REGPARAMS  *rp;
     pixb1 = pixBlendWithGrayMask(pixd, pixt1, NULL, -150, 20);
     pixb2 = pixBlendWithGrayMask(pixb1, pixt2, NULL, -50, -250);
     pixb3 = pixBlendWithGrayMask(pixb2, pixt3, NULL, -100, 220);
-    regTestWritePixAndCheck(rp, pixb3, IFF_PNG);  /* 8 */
-    pixDisplayWithTitle(pixb3, 500, 100, NULL, rp->display);
-    pixDestroy(&pixd);
-    pixDestroy(&pixb1);
-    pixDestroy(&pixb2);
-    pixDestroy(&pixb3);
-    pixDestroy(&pixt1);
-    pixDestroy(&pixt2);
-    pixDestroy(&pixt3);
-    ptaDestroy(&ptas1);
-    ptaDestroy(&ptas2);
-    ptaDestroy(&ptas3);
-    ptaDestroy(&ptad1);
-    ptaDestroy(&ptad2);
-    ptaDestroy(&ptad3);
-
-    pixd = pixCreate(900, 900, 32);
-    pixSetAll(pixd);
-    MakePtas(2, 4, &ptas1, &ptad1);
-    MakePtas(4, 4, &ptas2, &ptad2);
-    MakePtas(3, 4, &ptas3, &ptad3);
-    pixt1 = pixBilinearPtaGammaXform(pixcs1, 2.0, ptad1, ptas1, 1.0, 300);
-    pixt2 = pixBilinearPtaGammaXform(pixcs2, 2.0, ptad2, ptas2, 0.8, 400);
-    pixt3 = pixBilinearPtaGammaXform(pixcs3, 2.0, ptad3, ptas3, 0.7, 400);
-    pixb1 = pixBlendWithGrayMask(pixd, pixt1, NULL, -150, 20);
-    pixb2 = pixBlendWithGrayMask(pixb1, pixt2, NULL, -50, -250);
-    pixb3 = pixBlendWithGrayMask(pixb2, pixt3, NULL, -100, 220);
-    regTestWritePixAndCheck(rp, pixb3, IFF_PNG);  /* 9 */
+    regTestWritePixAndCheck(rp, pixb3, IFF_PNG);  /* 4 */
     pixDisplayWithTitle(pixb3, 500, 100, NULL, rp->display);
     pixDestroy(&pixd);
     pixDestroy(&pixb1);
@@ -305,7 +201,6 @@ L_REGPARAMS  *rp;
     pixDestroy(&pixcs1);
     pixDestroy(&pixcs2);
     pixDestroy(&pixcs3);
-
     return regTestCleanup(rp);
 }
 
