@@ -32,24 +32,17 @@
  *   peaks in HS.
  */
 
-#include <math.h>
-#ifndef  _WIN32
-#include <unistd.h>
-#else
-#include <windows.h>   /* for Sleep() */
-#endif  /* _WIN32 */
 #include "allheaders.h"
 
-
 int main(int    argc,
          char **argv)
 {
 l_int32       i, j, x, y, rval, gval, bval;
-l_uint32      pixel;
+l_uint32      pixel1, pixel2;
 l_float32     frval, fgval, fbval;
 NUMA         *nahue, *nasat, *napk;
-PIX          *pixs, *pixhsv, *pixh, *pixg, *pixf, *pixd;
-PIX          *pixr, *pixt1, *pixt2, *pixt3;
+PIX          *pixs, *pixhsv, *pixh, *pixg, *pixf, *pixd, *pixr;
+PIX          *pix1, *pix2, *pix3;
 PIXA         *pixa, *pixapk;
 PTA          *ptapk;
 L_REGPARAMS  *rp;
@@ -64,17 +57,17 @@ L_REGPARAMS  *rp;
         for (j = 0; j < 650; j++) {
             gval = 255 - j / 30;
             bval = 70 + j / 30;
-            composeRGBPixel(rval, gval, bval, &pixel);
-            pixSetPixel(pixs, j, i, pixel);
+            composeRGBPixel(rval, gval, bval, &pixel1);
+            pixSetPixel(pixs, j, i, pixel1);
         }
     }
 
         /* Place an image inside the frame and convert to HSV */
-    pixt1 = pixRead("1555.003.jpg");
-    pixt2 = pixScale(pixt1, 0.5, 0.5);
-    pixRasterop(pixs, 100, 100, 2000, 2000, PIX_SRC, pixt2, 0, 0);
-    pixDestroy(&pixt1);
-    pixDestroy(&pixt2);
+    pix1 = pixRead("1555.003.jpg");
+    pix2 = pixScale(pix1, 0.5, 0.5);
+    pixRasterop(pixs, 100, 100, 2000, 2000, PIX_SRC, pix2, 0, 0);
+    pixDestroy(&pix1);
+    pixDestroy(&pix2);
     pixDisplayWithTitle(pixs, 400, 0, "Input image", rp->display);
     pixa = pixaCreate(0);
     pixhsv = pixConvertRGBToHSV(NULL, pixs);
@@ -91,26 +84,16 @@ L_REGPARAMS  *rp;
     pixaAddPix(pixa, pixf, L_INSERT);
     gplotSimple1(nahue, GPLOT_PNG, "/tmp/lept/regout/junkhue",
                  "Histogram of hue values");
-#ifndef  _WIN32
-    sleep(1);
-#else
-    Sleep(1000);
-#endif  /* _WIN32 */
-    pixt3 = pixRead("/tmp/lept/regout/junkhue.png");
-    regTestWritePixAndCheck(rp, pixt3, IFF_PNG);  /* 1 */
-    pixDisplayWithTitle(pixt3, 100, 300, "Histo of hue", rp->display);
-    pixaAddPix(pixa, pixt3, L_INSERT);
+    pix3 = pixRead("/tmp/lept/regout/junkhue.png");
+    regTestWritePixAndCheck(rp, pix3, IFF_PNG);  /* 1 */
+    pixDisplayWithTitle(pix3, 100, 300, "Histo of hue", rp->display);
+    pixaAddPix(pixa, pix3, L_INSERT);
     gplotSimple1(nasat, GPLOT_PNG, "/tmp/lept/regout/junksat",
                  "Histogram of saturation values");
-#ifndef  _WIN32
-    sleep(1);
-#else
-    Sleep(1000);
-#endif  /* _WIN32 */
-    pixt3 = pixRead("/tmp/lept/regout/junksat.png");
-    regTestWritePixAndCheck(rp, pixt3, IFF_PNG);  /* 2 */
-    pixDisplayWithTitle(pixt3, 100, 800, "Histo of saturation", rp->display);
-    pixaAddPix(pixa, pixt3, L_INSERT);
+    pix3 = pixRead("/tmp/lept/regout/junksat.png");
+    regTestWritePixAndCheck(rp, pix3, IFF_PNG);  /* 2 */
+    pixDisplayWithTitle(pix3, 100, 800, "Histo of saturation", rp->display);
+    pixaAddPix(pixa, pix3, L_INSERT);
     pixd = pixaDisplayTiledAndScaled(pixa, 32, 270, 7, 0, 30, 3);
     regTestWritePixAndCheck(rp, pixd, IFF_PNG);  /* 3 */
     pixDisplayWithTitle(pixd, 0, 400, "Hue and Saturation Mosaic", rp->display);
@@ -136,22 +119,24 @@ L_REGPARAMS  *rp;
     pixr = pixScaleBySampling(pixs, 0.4, 0.4);
     for (i = 0; i < 6; i++) {
         ptaGetIPt(ptapk, i, &x, &y);
-        pixt1 = pixMakeRangeMaskHS(pixr, y, 20, x, 20, L_INCLUDE_REGION);
-        pixaAddPix(pixa, pixt1, L_INSERT);
-        pixGetAverageMaskedRGB(pixr, pixt1, 0, 0, 1, L_MEAN_ABSVAL,
+        pix1 = pixMakeRangeMaskHS(pixr, y, 20, x, 20, L_INCLUDE_REGION);
+        pixaAddPix(pixa, pix1, L_INSERT);
+        pixGetAverageMaskedRGB(pixr, pix1, 0, 0, 1, L_MEAN_ABSVAL,
                                &frval, &fgval, &fbval);
         composeRGBPixel((l_int32)frval, (l_int32)fgval, (l_int32)fbval,
-                        &pixel);
-        pixt2 = pixCreateTemplate(pixr);
-        pixSetAll(pixt2);
-        pixPaintThroughMask(pixt2, pixt1, 0, 0, pixel);
-        pixaAddPix(pixa, pixt2, L_INSERT);
-        pixt3 = pixCreateTemplate(pixr);
-        pixSetAllArbitrary(pixt3, pixel);
-        pixaAddPix(pixa, pixt3, L_INSERT);
+                        &pixel1);
+        pixGetPixelAverage(pixr, pix1, 0, 0, 1, &pixel2);
+        regTestCompareValues(rp, pixel1 >> 8, pixel2 >> 8, 0.0);  /* 5 - 10 */
+        pix2 = pixCreateTemplate(pixr);
+        pixSetAll(pix2);
+        pixPaintThroughMask(pix2, pix1, 0, 0, pixel1);
+        pixaAddPix(pixa, pix2, L_INSERT);
+        pix3 = pixCreateTemplate(pixr);
+        pixSetAllArbitrary(pix3, pixel1);
+        pixaAddPix(pixa, pix3, L_INSERT);
     }
     pixd = pixaDisplayTiledAndScaled(pixa, 32, 225, 3, 0, 30, 3);
-    regTestWritePixAndCheck(rp, pixd, IFF_PNG);  /* 5 */
+    regTestWritePixAndCheck(rp, pixd, IFF_PNG);  /* 8 */
     pixDisplayWithTitle(pixd, 600, 0, "Masks over peaks", rp->display);
     pixDestroy(&pixs);
     pixDestroy(&pixr);
