@@ -80,7 +80,7 @@ PIX        *pixs, *pixn, *pixg, *pixb, *pix2, *pix3, *pix4, *pix5, *pix6;
     pixDisplay(pix2, 200, 100);
 
         /* Reverse direction: get the word boxes for the dewarped pix ... */
-    pixGetWordBoxesInTextlines(pix2, 1, 5, 5, 500, 100, &boxa1, NULL);
+    pixGetWordBoxesInTextlines(pix2, 5, 5, 500, 100, &boxa1, NULL);
     pix3 = pixConvertTo32(pix2);
     pixRenderBoxaArb(pix3, boxa1, 2, 255, 0, 0);
     pixDisplay(pix3, 400, 100);
@@ -99,7 +99,7 @@ PIX        *pixs, *pixn, *pixg, *pixb, *pix2, *pix3, *pix4, *pix5, *pix6;
     pixDisplay(pix4, 600, 100);
 
         /* Forward direction: get the word boxes for the input pix ... */
-    pixGetWordBoxesInTextlines(pixb, 1, 5, 5, 500, 100, &boxa3, NULL);
+    pixGetWordBoxesInTextlines(pixb, 5, 5, 500, 100, &boxa3, NULL);
     pix5 = pixConvertTo32(pixb);
     pixRenderBoxaArb(pix5, boxa3, 2, 255, 0, 0);
     pixDisplay(pix5, 800, 100);
