@@ -57,15 +57,15 @@
 #include <math.h>
 #include "allheaders.h"
 
-static void test_gif(const char *fname, PIXA *pixa, L_REGPARAMS *rp);
-static l_int32 test_mem_gif(const char *fname, l_int32 index);
-
-
     /* Needed for HAVE_LIBGIF and or HAVE_LIBUNGIF */
 #ifdef HAVE_CONFIG_H
 #include <config_auto.h>
 #endif /* HAVE_CONFIG_H */
 
+#if HAVE_LIBGIF || HAVE_LIBUNGIF
+#include "gif_lib.h"
+#endif  /* HAVE_LIBGIF || HAVE_LIBUNGIF */
+
 #define   FILE_1BPP     "feyn.tif"
 #define   FILE_2BPP     "weasel2.4g.png"
 #define   FILE_4BPP     "weasel4.16c.png"
@@ -75,9 +75,13 @@ static l_int32 test_mem_gif(const char *fname, l_int32 index);
 #define   FILE_16BPP    "test16.tif"
 #define   FILE_32BPP    "marge.jpg"
 
+static void test_gif(const char *fname, PIXA *pixa, L_REGPARAMS *rp);
+static l_int32 test_mem_gif(const char *fname, l_int32 index);
+
 int main(int    argc,
          char **argv)
 {
+char          buf[64];
 l_int32       success;
 PIX          *pix;
 PIXA         *pixa;
@@ -88,12 +92,23 @@ L_REGPARAMS  *rp;
             "libgif or libungif are required for gifio_reg\n"
             "See environ.h: #define HAVE_LIBGIF or HAVE_LIBUNGIF 1\n"
             "See prog/Makefile: link in -lgif or -lungif\n\n");
-    return 0;
+    return 1;
 #endif  /* abort */
 
     if (regTestSetup(argc, argv, &rp))
         return 1;
 
+        /* 5.1+ and not 5.1.2 */
+    snprintf(buf, sizeof(buf), "%s_reg", rp->testname);
+#if (GIFLIB_MAJOR < 5 || (GIFLIB_MAJOR == 5 && GIFLIB_MINOR == 0))
+    L_ERROR("Require giflib-5.1 or later.\n", buf);
+    return 1;
+#endif  /* < 5.1 */
+#if GIFLIB_MAJOR == 5 && GIFLIB_MINOR == 1 && GIFLIB_RELEASE == 2  /* 5.1.2 */
+    L_ERROR("Can't use giflib-5.1.2; suggest 5.1.3 or later.\n", buf);
+    return 1;
+#endif  /* 5.1.2 */
+
         /* Set up for display output */
     pixa = (rp->display) ? pixaCreate(0) : NULL;
 
