@@ -32,8 +32,6 @@
  *         dirin:  directory of input pages
  *         size: size of SE used for dilation
  *         rank: min pixel fraction required in both directions in match
- *         rootname: used for naming the two output files (templates
- *                   and c.c. data)
  *
  * Notes:
  *     (1) All components larger than a default size are not saved.
@@ -42,6 +40,9 @@
  *         gives good accuracy without too manyclasses, is:
  *               size = 2  (2 x 2 structuring element)
  *               rank = 0.97
+ *     (3) The two output files (for templates and c.c. data)
+ *         are written with the rootname
+ *               /tmp/lept/jb/result
  */
 
 #include "allheaders.h"
@@ -62,14 +63,15 @@
     /* for display output of all instances, sorted by class */
 #define   X_SPACING                10
 #define   Y_SPACING                15
-#define   MAX_OUTPUT_WIDTH        400
+#define   MAX_OUTPUT_WIDTH         400
 
+static const char  rootname[] = "/tmp/lept/jb/result";
 
 int main(int    argc,
          char **argv)
 {
 char         filename[BUF_SIZE];
-char        *dirin, *rootname, *fname;
+char        *dirin, *fname;
 l_int32      i, size, firstpage, npages, nfiles;
 l_float32    rank;
 JBDATA      *data;
@@ -79,25 +81,26 @@ PIX         *pix, *pixt;
 PIXA        *pixa, *pixadb;
 static char  mainName[] = "jbrankhaus";
 
-    if (argc != 5 && argc != 7)
+    if (argc != 4 && argc != 6)
         return ERROR_INT(
-             " Syntax: jbrankhaus dirin size rank rootname [firstpage, npages]",
+             " Syntax: jbrankhaus dirin size rank [firstpage, npages]",
              mainName, 1);
 
     dirin = argv[1];
     size = atoi(argv[2]);
     rank = atof(argv[3]);
-    rootname = argv[4];
 
-    if (argc == 5) {
+    if (argc == 4) {
         firstpage = 0;
         npages = 0;
     }
     else {
-        firstpage = atoi(argv[5]);
-        npages = atoi(argv[6]);
+        firstpage = atoi(argv[4]);
+        npages = atoi(argv[5]);
     }
 
+    lept_mkdir("lept/jb");
+
 #if 0
 
     /*--------------------------------------------------------------*/
@@ -136,27 +139,33 @@ static char  mainName[] = "jbrankhaus";
                 npages, nfiles);
     for (i = 0; i < npages; i++) {
         pix = pixaGetPix(pixa, i, L_CLONE);
-        snprintf(filename, BUF_SIZE, "%s.%04d", rootname, i);
+        snprintf(filename, BUF_SIZE, "%s.%03d", rootname, i);
         fprintf(stderr, "filename: %s\n", filename);
         pixWrite(filename, pix, IFF_PNG);
         pixDestroy(&pix);
     }
 
 #if  DISPLAY_DIFFERENCE
+    {
+    char *fname;
+    PIX  *pix1, *pix2;
     fname = sarrayGetString(safiles, 0, L_NOCOPY);
-    pixt = pixRead(fname);
-    pix = pixaGetPix(pixa, 0, L_CLONE);
-    pixXor(pixt, pixt, pix);
-    pixWrite("junk_output_diff", pixt, IFF_PNG);
-    pixDestroy(&pix);
-    pixDestroy(&pixt);
+    pix1 = pixRead(fname);
+    pix2 = pixaGetPix(pixa, 0, L_CLONE);
+    pixXor(pix1, pix1, pix2);
+    pixWrite("/tmp/lept/jb/output_diff.png", pix1, IFF_PNG);
+    pixDestroy(&pix1);
+    pixDestroy(&pix2);
+    }
 #endif  /* DISPLAY_DIFFERENCE */
 
 #if  DEBUG_TEST_DATA_IO
-{ JBDATA  *newdata;
-  PIX     *newpix;
-  PIXA    *newpixa;
-  l_int32  same, iofail;
+    {
+    JBDATA  *newdata;
+    PIX     *newpix;
+    PIXA    *newpixa;
+    l_int32  same, iofail;
+
         /* Read the data back in and render the pages */
     newdata = jbDataRead(rootname);
     newpixa = jbDataRender(newdata, FALSE);
@@ -179,7 +188,7 @@ static char  mainName[] = "jbrankhaus";
         fprintf(stderr, "read/write for jbdata succeeds\n");
     jbDataDestroy(&newdata);
     pixaDestroy(&newpixa);
-}
+    }
 #endif  /* DEBUG_TEST_DATA_IO */
 
 #if  RENDER_DEBUG
@@ -201,7 +210,7 @@ static char  mainName[] = "jbrankhaus";
         /* display all instances, organized by template */
     pix = pixaaDisplayByPixa(classer->pixaa,
                              X_SPACING, Y_SPACING, MAX_OUTPUT_WIDTH);
-    pixWrite("output_instances", pix, IFF_PNG);
+    pixWrite("/tmp/lept/jb/output_instances", pix, IFF_PNG);
     pixDestroy(&pix);
 #endif  /* DISPLAY_ALL_INSTANCES */
 
@@ -216,4 +225,3 @@ static char  mainName[] = "jbrankhaus";
 
     return 0;
 }
-
