@@ -29,11 +29,6 @@
  */
 
 #include <string.h>
-#ifndef  _WIN32
-#include <unistd.h>
-#else
-#include <windows.h>   /* for Sleep() */
-#endif  /* _WIN32 */
 #include "allheaders.h"
 
 static const char  *kdatastr = " 20.3    50   80  50   20 "
@@ -157,7 +152,7 @@ L_REGPARAMS  *rp;
     pixWrite("/tmp/lept/regout/ker5.png", pixt, IFF_PNG);
     regTestCheckFile(rp, "/tmp/lept/regout/ker5.png");  /* 8 */
     if (rp->display)
-        pixCompareGray(pixd, pixt, L_COMPARE_ABS_DIFF, GPLOT_X11, NULL,
+        pixCompareGray(pixd, pixt, L_COMPARE_ABS_DIFF, GPLOT_PNG, NULL,
                        NULL, NULL, NULL);
     pixt2 = pixBlockconvTiled(pixg, 5, 5, 3, 6);
     pixSaveTiled(pixt2, pixa, 1.0, 0, 20, 0);
@@ -216,11 +211,6 @@ L_REGPARAMS  *rp;
 
     pixCompareGray(pixt, pixt2, L_COMPARE_ABS_DIFF, GPLOT_PNG, NULL,
                    &avediff, &rmsdiff, NULL);
-#ifndef  _WIN32
-    sleep(1);  /* give gnuplot time to write out the file */
-#else
-    Sleep(1000);
-#endif  /* _WIN32 */
     pixp = pixRead("/tmp/lept/comp/compare_gray0.png");
     pixSaveTiled(pixp, pixa, 1.0, 0, 20, 0);
     pixWrite("/tmp/lept/regout/conv3.png", pixp, IFF_PNG);
