@@ -28,13 +28,17 @@
  *  maptest.c
  *
  *  Tests map function for RGB (uint32) keys and count (int32) values.
+ *  The underlying rbtree takes 64 bit keys and values, so it also works
+ *  transparently with 32 bit keys and values.
  *
  *  We take a colormapped image and use the map to accumulate a
  *  histogram of the colors, using the 32-bit rgb value as the key.
- *  The value stores the number of pixels with that color that we have seen.
- *  Also test the iterator on the map.
+ *  The value is the number of pixels with that color that we have seen.
  *
- *  We also build an inverse colormap table using a map.
+ *  Also:
+ *   * test the forward and backward iterators on the map
+ *   * build an inverse colormap table using a map.
+ *   * test RGB histogram and counting functions in pix4.c
  */
 
 #include "allheaders.h"
@@ -42,6 +46,7 @@
 static L_AMAP *BuildMapHistogram(PIX *pix, l_int32 factor, l_int32 print);
 static void DisplayMapHistogram(L_AMAP *m, PIXCMAP *cmap,
                                 const char *rootname);
+static void DisplayMapRGBHistogram(L_AMAP *m, const char *rootname);
 static void TestMapIterator1(L_AMAP *m, l_int32  print);
 static void TestMapIterator2(L_AMAP *m, l_int32  print);
 static void TestMapIterator3(L_AMAP *m, l_int32  print);
@@ -104,7 +109,7 @@ RB_TYPE   *pval;
         /* Build a histogram the old-fashioned way */
     na = pixGetCmapHistogram(pix, 1);
     numaWrite("/tmp/lept/map/map2.na", na);
-    gplotSimple1(na, GPLOT_PNG, "/tmp/lept/map/map1", NULL);
+    gplotSimple1(na, GPLOT_PNG, "/tmp/lept/map/map3", NULL);
     numaDestroy(&na);
 
         /* Build a separate map from (rgb) --> colormap index ... */
@@ -121,12 +126,20 @@ RB_TYPE   *pval;
         pixcmapGetColor32(cmap, i, &val32);
         key.utype = val32;
         pval = l_amapFind(m, key);
-        if (i != pval->itype)
+        if (pval && (i != pval->itype))
             fprintf(stderr, "i = %d != val = %llx\n", i, pval->itype);
     }
     l_amapDestroy(&m);
-
     pixDestroy(&pix);
+
+        /* Build and display a real RGB histogram */
+    pix = pixRead("wyom.jpg");
+    m = pixGetColorAmapHistogram(pix, 1);
+    DisplayMapRGBHistogram(m, "/tmp/lept/map/map4");
+    fprintf(stderr, " Using pixCountRGBColors: %d\n", pixCountRGBColors(pix));
+    l_amapDestroy(&m);
+    pixDestroy(&pix);
+
     return 0;
 }
 
@@ -206,6 +219,45 @@ RB_TYPE  *pval;
     return;
 }
 
+static void
+DisplayMapRGBHistogram(L_AMAP      *m,
+                       const char  *rootname)
+{
+char          buf[128];
+l_int32       ncolors, npix, ival, maxn, maxn2;
+l_uint32      val32, maxcolor;
+L_AMAP_NODE  *n;
+NUMA         *na;
+
+    fprintf(stderr, "\n --------------- Display RGB histogram ------------\n");
+    na = numaCreate(0);
+    ncolors = npix = 0;
+    maxn = 0;
+    maxcolor = 0;
+    n = l_amapGetFirst(m);
+    while (n) {
+        ncolors++;
+        ival = n->value.itype;
+        if (ival > maxn) {
+            maxn = ival;
+            maxcolor = n->key.utype;
+        }
+        numaAddNumber(na, ival);
+        npix += ival;
+        n = l_amapGetNext(n);
+    }
+    fprintf(stderr, " Num colors = %d, Num pixels = %d\n", ncolors, npix);
+    fprintf(stderr, " Color %x has count %d\n", maxcolor, maxn);
+    maxn2 = amapGetCountForColor(m, maxcolor);
+    if (maxn != maxn2)
+        fprintf(stderr, " Error: maxn2 = %d; not equal to %d\n", maxn, maxn2);
+    gplotSimple1(na, GPLOT_PNG, rootname, NULL);
+    snprintf(buf, sizeof(buf), "%s.png", rootname);
+    l_fileDisplay(buf, 1400, 0, 1.0);
+    numaDestroy(&na);
+    return;
+}
+
 static void
 TestMapIterator1(L_AMAP  *m,
                  l_int32  print)  /* forward iterator; fixed tree */
