@@ -35,26 +35,24 @@
 l_int32 main(int    argc,
              char **argv)
 {
-char      *boxatxt;
-l_int32    i;
-BOXA      *boxa1, *boxa2, *boxa3;
-BOXAA     *baa, *baa1;
-NUMAA     *naa1;
-PIX       *pixdb, *pix1, *pix2, *pix3, *pix4;
-PIXA      *pixa1, *pixa2, *pixa3, *pixat;
-L_RECOG   *recog;
-L_RECOGA  *recoga;
-SARRAY    *sa1;
+char     *boxatxt;
+l_int32   i;
+BOXA     *boxa1, *boxa2, *boxa3;
+BOXAA    *baa, *baa1;
+NUMAA    *naa1;
+PIX      *pixdb, *pix1, *pix2, *pix3, *pix4;
+PIXA     *pixa1, *pixa2, *pixa3, *pixat;
+L_RECOG  *recog;
+SARRAY   *sa1;
 
     /* ----- Example identifying samples using training data ----- */
-#if 1
 
-    lept_mkdir("lept");
+    lept_mkdir("lept/recog");
 
         /* Read the training data */
     pixat = pixaRead("recog/sets/train06.pa");
-    recog = recogCreateFromPixa(pixat, 0, 0, L_USE_ALL, 128, 1);
-    recoga = recogaCreateFromRecog(recog);
+    recog = recogCreateFromPixa(pixat, 0, 0, 0, 128, 1);
+    recogAverageSamples(&recog, 0);  /* required for splitting characters */
     pix1 = pixaDisplayTiledWithText(pixat, 1500, 1.0, 10, 1, 8, 0xff000000);
     pixDisplay(pix1, 0, 0);
     pixDestroy(&pix1);
@@ -63,6 +61,7 @@ SARRAY    *sa1;
         /* Read the data from all samples */
     pix1 = pixRead("recog/sets/samples06.png");
     boxatxt = pixGetText(pix1);
+    fprintf(stderr, "%s\n", boxatxt);
     boxa1 = boxaReadMem((l_uint8 *)boxatxt, strlen(boxatxt));
     pixa1 = pixaCreateFromBoxa(pix1, boxa1, NULL);
     pixDestroy(&pix1);  /* destroys boxa1 */
@@ -79,15 +78,14 @@ SARRAY    *sa1;
             /* Show the 2d box data in the sample */
         boxa2 = pixConnComp(pix1, NULL, 8);
         baa = boxaSort2d(boxa2, NULL, 6, 6, 5);
-        pix2 = boxaaDisplay(baa, 3, 1, 0xff000000, 0x00ff0000, 0, 0);
+        pix2 = boxaaDisplay(pix1, baa, 3, 1, 0xff000000, 0x00ff0000, 0, 0);
         pixaAddPix(pixa3, pix2, L_INSERT);
         boxaaDestroy(&baa);
         boxaDestroy(&boxa2);
 
             /* Get the numbers in the sample */
-        recogaIdentifyMultiple(recoga, pix1, 0, 5, 3, 0,
-                               &boxa3, NULL, &pixdb, 0);
-        sa1 = recogaExtractNumbers(recoga, boxa3, 0.7, -1, &baa1, &naa1);
+        recogIdentifyMultiple(recog, pix1, 0, 0, &boxa3, NULL, &pixdb, 0);
+        sa1 = recogExtractNumbers(recog, boxa3, 0.7, -1, &baa1, &naa1);
         sarrayWriteStream(stderr, sa1);
         boxaaWriteStream(stderr, baa1);
         numaaWriteStream(stderr, naa1);
@@ -102,10 +100,10 @@ SARRAY    *sa1;
     }
 
     pix3 = pixaDisplayLinearly(pixa2, L_VERT, 1.0, 0, 20, 1, NULL);
-    pixWrite("/tmp/lept/pix3.png", pix3, IFF_PNG);
+    pixWrite("/tmp/lept/recog/pix3.png", pix3, IFF_PNG);
     pix4 = pixaDisplayTiledInRows(pixa3, 32, 1500, 1.0, 0, 20, 2);
     pixDisplay(pix4, 500, 0);
-    pixWrite("/tmp/lept/pix4.png", pix4, IFF_PNG);
+    pixWrite("/tmp/lept/recog/pix4.png", pix4, IFF_PNG);
     pixaDestroy(&pixa2);
     pixaDestroy(&pixa3);
     pixDestroy(&pix1);
@@ -113,9 +111,7 @@ SARRAY    *sa1;
     pixDestroy(&pix4);
     pixaDestroy(&pixa1);
     boxaDestroy(&boxa1);
-    recogaDestroy(&recoga);
-
-#endif
+    recogDestroy(&recog);
 
     return 0;
 }
