@@ -40,8 +40,7 @@ l_uint32     redval, greenval;
 l_float32    f;
 L_WSHED     *wshed;
 PIX         *pixs, *pixc, *pixd;
-PIX         *pixt0, *pixt1, *pixt2, *pixt3, *pixt4, *pixt5;
-PIX         *pixt6, *pixt7, *pixt8;
+PIX         *pix1, *pix2, *pix3, *pix4, *pix5, *pix6, *pix7, *pix8;
 PIXA        *pixac;
 PTA         *pta;
 static char  mainName[] = "watershedtest";
@@ -71,43 +70,43 @@ static char  mainName[] = "watershedtest";
     pixSaveTiled(pixs, pixac, 1.0, 1, 10, 32);
     pixWrite("/tmp/pattern.png", pixs, IFF_PNG);
     startTimer();
-    pixLocalExtrema(pixs, 0, 0, &pixt1, &pixt2);
+    pixLocalExtrema(pixs, 0, 0, &pix1, &pix2);
     fprintf(stderr, "Time for extrema: %7.3f\n", stopTimer());
-    pixSetOrClearBorder(pixt1, 2, 2, 2, 2, PIX_CLR);
+    pixSetOrClearBorder(pix1, 2, 2, 2, 2, PIX_CLR);
     composeRGBPixel(255, 0, 0, &redval);
     composeRGBPixel(0, 255, 0, &greenval);
     pixc = pixConvertTo32(pixs);
-    pixPaintThroughMask(pixc, pixt2, 0, 0, greenval);
-    pixPaintThroughMask(pixc, pixt1, 0, 0, redval);
+    pixPaintThroughMask(pixc, pix2, 0, 0, greenval);
+    pixPaintThroughMask(pixc, pix1, 0, 0, redval);
     pixSaveTiled(pixc, pixac, 1.0, 0, 10, 32);
     pixWrite("/tmp/pixc.png", pixc, IFF_PNG);
-    pixSaveTiled(pixt1, pixac, 1.0, 0, 10, 32);
-    pixSelectMinInConnComp(pixs, pixt1, &pta, NULL);
+    pixSaveTiled(pix1, pixac, 1.0, 0, 10, 32);
+    pixSelectMinInConnComp(pixs, pix1, &pta, NULL);
 /*    ptaWriteStream(stderr, pta, 1); */
-    pixt3 = pixGenerateFromPta(pta, w, h);
-    pixSaveTiled(pixt3, pixac, 1.0, 1, 10, 32);
+    pix3 = pixGenerateFromPta(pta, w, h);
+    pixSaveTiled(pix3, pixac, 1.0, 1, 10, 32);
 
-    pixt4 = pixConvertTo32(pixs);
-    pixPaintThroughMask(pixt4, pixt3, 0, 0, greenval);
-    pixSaveTiled(pixt4, pixac, 1.0, 0, 10, 32);
-    pixt5 = pixRemoveSeededComponents(NULL, pixt3, pixt1, 8, 2);
-    pixSaveTiled(pixt5, pixac, 1.0, 0, 10, 32);
-    pixZero(pixt5, &empty);
+    pix4 = pixConvertTo32(pixs);
+    pixPaintThroughMask(pix4, pix3, 0, 0, greenval);
+    pixSaveTiled(pix4, pixac, 1.0, 0, 10, 32);
+    pix5 = pixRemoveSeededComponents(NULL, pix3, pix1, 8, 2);
+    pixSaveTiled(pix5, pixac, 1.0, 0, 10, 32);
+    pixZero(pix5, &empty);
     fprintf(stderr, "Is empty?  %d\n", empty);
-    pixDestroy(&pixt4);
-    pixDestroy(&pixt5);
+    pixDestroy(&pix4);
+    pixDestroy(&pix5);
 
-    wshed = wshedCreate(pixs, pixt3, 10, 0);
+    wshed = wshedCreate(pixs, pix3, 10, 0);
     startTimer();
     wshedApply(wshed);
     fprintf(stderr, "Time for wshed: %7.3f\n", stopTimer());
-    pixt6 = pixaDisplayRandomCmap(wshed->pixad, w, h);
-    pixSaveTiled(pixt6, pixac, 1.0, 1, 10, 32);
+    pix6 = pixaDisplayRandomCmap(wshed->pixad, w, h);
+    pixSaveTiled(pix6, pixac, 1.0, 1, 10, 32);
     numaWriteStream(stderr, wshed->nalevels);
-    pixt7 = wshedRenderFill(wshed);
-    pixSaveTiled(pixt7, pixac, 1.0, 0, 10, 32);
-    pixt8 = wshedRenderColors(wshed);
-    pixSaveTiled(pixt8, pixac, 1.0, 0, 10, 32);
+    pix7 = wshedRenderFill(wshed);
+    pixSaveTiled(pix7, pixac, 1.0, 0, 10, 32);
+    pix8 = wshedRenderColors(wshed);
+    pixSaveTiled(pix8, pixac, 1.0, 0, 10, 32);
     wshedDestroy(&wshed);
 
     pixd = pixaDisplay(pixac, 0, 0);
@@ -116,12 +115,12 @@ static char  mainName[] = "watershedtest";
     pixDestroy(&pixd);
     pixaDestroy(&pixac);
 
-    pixDestroy(&pixt1);
-    pixDestroy(&pixt2);
-    pixDestroy(&pixt3);
-    pixDestroy(&pixt6);
-    pixDestroy(&pixt7);
-    pixDestroy(&pixt8);
+    pixDestroy(&pix1);
+    pixDestroy(&pix2);
+    pixDestroy(&pix3);
+    pixDestroy(&pix6);
+    pixDestroy(&pix7);
+    pixDestroy(&pix8);
     pixDestroy(&pixs);
     pixDestroy(&pixc);
     ptaDestroy(&pta);
