@@ -61,7 +61,7 @@ BOXAA       *baa;
 NUMA        *nai;
 NUMAA       *naa;
 SARRAY      *safiles;
-PIX         *pixs, *pixt1, *pixt2, *pixd;
+PIX         *pixs, *pix1, *pix2, *pixd;
 PIXCMAP     *cmap;
 static char  mainName[] = "wordsinorder";
 
@@ -94,24 +94,26 @@ static char  mainName[] = "wordsinorder";
             L_WARNING("image file %d not read\n", mainName, i);
             continue;
         }
-        pixGetWordBoxesInTextlines(pixs, 2, MIN_WORD_WIDTH, MIN_WORD_HEIGHT,
+        pix1 = pixReduceRankBinary2(pixs, 1, NULL);
+        pixGetWordBoxesInTextlines(pix1, MIN_WORD_WIDTH, MIN_WORD_HEIGHT,
                                    MAX_WORD_WIDTH, MAX_WORD_HEIGHT,
                                    &boxa, &nai);
         boxaaAddBoxa(baa, boxa, L_INSERT);
         numaaAddNuma(naa, nai, L_INSERT);
+        pixDestroy(&pix1);
 
 #if  RENDER_PAGES
             /* Show the results on a 2x reduced image, where each
              * word is outlined and the color of the box depends on the
              * computed textline. */
-        pixt1 = pixReduceRankBinary2(pixs, 2, NULL);
-        pixGetDimensions(pixt1, &w, &h, NULL);
+        pix1 = pixReduceRankBinary2(pixs, 2, NULL);
+        pixGetDimensions(pix1, &w, &h, NULL);
         pixd = pixCreate(w, h, 8);
         cmap = pixcmapCreateRandom(8, 1, 1);  /* first color is black */
         pixSetColormap(pixd, cmap);
 
-        pixt2 = pixUnpackBinary(pixt1, 8, 1);
-        pixRasterop(pixd, 0, 0, w, h, PIX_SRC | PIX_DST, pixt2, 0, 0);
+        pix2 = pixUnpackBinary(pix1, 8, 1);
+        pixRasterop(pixd, 0, 0, w, h, PIX_SRC | PIX_DST, pix2, 0, 0);
         ncomp = boxaGetCount(boxa);
         for (j = 0; j < ncomp; j++) {
             box = boxaGetBox(boxa, j, L_CLONE);
@@ -125,8 +127,8 @@ static char  mainName[] = "wordsinorder";
         snprintf(filename, BUF_SIZE, "%s.%05d", rootname, i);
         fprintf(stderr, "filename: %s\n", filename);
         pixWrite(filename, pixd, IFF_PNG);
-        pixDestroy(&pixt1);
-        pixDestroy(&pixt2);
+        pixDestroy(&pix1);
+        pixDestroy(&pix2);
         pixDestroy(&pixs);
         pixDestroy(&pixd);
 #endif  /* RENDER_PAGES */
