@@ -403,10 +403,8 @@ PIX      *pixn, *pixm, *pixd, *pix1, *pix2, *pix3, *pix4;
          * which is stored as a single pixel in pix3. */
     pixGetDimensions(pixs, &w, &h, NULL);
     pixOtsuAdaptiveThreshold(pixs, w, h, 0, 0, scorefract, &pix3, NULL);
-    if (pix3 && pthresh) {
-        pixGetPixel(pix3, 0, 0, &val);
-        *pthresh = val;
-    }
+    pixGetPixel(pix3, 0, 0, &val);
+    if (pthresh) *pthresh = val;
     pixDestroy(&pix3);
 
         /* Threshold the background normalized images differentially,
@@ -450,7 +448,7 @@ PIX      *pixn, *pixm, *pixd, *pix1, *pix2, *pix3, *pix4;
  * <pre>
  * Notes:
  *      (1) The window width and height are 2 * %whsize + 1.  The minimum
- *          value for %whsize is 2; typically it is \>= 7..
+ *          value for %whsize is 2; typically it is >= 7..
  *      (2) For nx == ny == 1, this defaults to pixSauvolaBinarize().
  *      (3) Why a tiled version?
  *          (a) Because the mean value accumulator is a uint32, overflow
@@ -570,7 +568,7 @@ PIXTILING  *pt;
  * <pre>
  * Notes:
  *      (1) The window width and height are 2 * %whsize + 1.  The minimum
- *          value for %whsize is 2; typically it is \>= 7..
+ *          value for %whsize is 2; typically it is >= 7..
  *      (2) The local statistics, measured over the window, are the
  *          average and standard deviation.
  *      (3) The measurements of the mean and standard deviation are
@@ -582,7 +580,7 @@ PIXTILING  *pt;
  *          where:
  *            t = local threshold
  *            m = local mean
- *            k = %factor (\>= 0)   [ typ. 0.35 ]
+ *            k = %factor (>= 0)   [ typ. 0.35 ]
  *            s = local standard deviation, which is maximized at
  *                127.5 when half the samples are 0 and half are 255.
  *      (5) The basic idea of Niblack and Sauvola binarization is that
@@ -659,8 +657,6 @@ PIX     *pixg, *pixsc, *pixm, *pixms, *pixth, *pixd;
         pixDestroy(&pixth);
     if (ppixd)
         *ppixd = pixd;
-    else
-        pixDestroy(&pixd);
     pixDestroy(&pixg);
     pixDestroy(&pixsc);
     return 0;
@@ -683,7 +679,7 @@ PIX     *pixg, *pixsc, *pixm, *pixms, *pixth, *pixd;
  *          where:
  *            t = local threshold
  *            m = local mean
- *            k = %factor (\>= 0)   [ typ. 0.35 ]
+ *            k = %factor (>= 0)   [ typ. 0.35 ]
  *            s = local standard deviation, which is maximized at
  *                127.5 when half the samples are 0 and half are 255.
  *      (2) See pixSauvolaBinarize() for other details.
