@@ -61,6 +61,7 @@
  *           PIX     *pixModifyBrightness()
  *
  *      Color shifting
+ *           PIX     *pixMosaicColorShiftRGB()
  *           PIX     *pixColorShiftRGB()
  *
  *      General multiplicative constant color transform
@@ -128,11 +129,11 @@ static const l_int32  DEFAULT_HISTO_SAMPLES = 100000;
 /*!
  * \brief   pixGammaTRC()
  *
- * \param[in]    pixd [optional] null or equal to pixs
- * \param[in]    pixs 8 or 32 bpp; or 2, 4 or 8 bpp with colormap
- * \param[in]    gamma gamma correction; must be > 0.0
- * \param[in]    minval  input value that gives 0 for output; can be < 0
- * \param[in]    maxval  input value that gives 255 for output; can be > 255
+ * \param[in]    pixd     [optional] null or equal to pixs
+ * \param[in]    pixs     8 or 32 bpp; or 2, 4 or 8 bpp with colormap
+ * \param[in]    gamma    gamma correction; must be > 0.0
+ * \param[in]    minval   input value that gives 0 for output; can be < 0
+ * \param[in]    maxval   input value that gives 255 for output; can be > 255
  * \return  pixd always
  *
  * <pre>
@@ -218,12 +219,12 @@ PIXCMAP  *cmap;
 /*!
  * \brief   pixGammaTRCMasked()
  *
- * \param[in]    pixd [optional] null or equal to pixs
- * \param[in]    pixs 8 or 32 bpp; not colormapped
- * \param[in]    pixm [optional] null or 1 bpp
- * \param[in]    gamma gamma correction; must be > 0.0
- * \param[in]    minval  input value that gives 0 for output; can be < 0
- * \param[in]    maxval  input value that gives 255 for output; can be > 255
+ * \param[in]    pixd      [optional] null or equal to pixs
+ * \param[in]    pixs      8 or 32 bpp; not colormapped
+ * \param[in]    pixm      [optional] null or 1 bpp
+ * \param[in]    gamma     gamma correction; must be > 0.0
+ * \param[in]    minval    input value that gives 0 for output; can be < 0
+ * \param[in]    maxval    input value that gives 255 for output; can be > 255
  * \return  pixd always
  *
  * <pre>
@@ -284,11 +285,11 @@ NUMA    *nag;
 /*!
  * \brief   pixGammaTRCWithAlpha()
  *
- * \param[in]    pixd [optional] null or equal to pixs
- * \param[in]    pixs 32 bpp
- * \param[in]    gamma gamma correction; must be > 0.0
- * \param[in]    minval  input value that gives 0 for output; can be < 0
- * \param[in]    maxval  input value that gives 255 for output; can be > 255
+ * \param[in]    pixd     [optional] null or equal to pixs
+ * \param[in]    pixs     32 bpp
+ * \param[in]    gamma    gamma correction; must be > 0.0
+ * \param[in]    minval   input value that gives 0 for output; can be < 0
+ * \param[in]    maxval   input value that gives 255 for output; can be > 255
  * \return  pixd always
  *
  * <pre>
@@ -402,9 +403,9 @@ NUMA      *na;
 /*!
  * \brief   pixContrastTRC()
  *
- * \param[in]    pixd [optional] null or equal to pixs
- * \param[in]    pixs 8 or 32 bpp; or 2, 4 or 8 bpp with colormap
- * \param[in]    factor  0.0 is no enhancement
+ * \param[in]    pixd     [optional] null or equal to pixs
+ * \param[in]    pixs     8 or 32 bpp; or 2, 4 or 8 bpp with colormap
+ * \param[in]    factor   0.0 is no enhancement
  * \return  pixd always
  *
  * <pre>
@@ -476,10 +477,10 @@ PIXCMAP  *cmap;
 /*!
  * \brief   pixContrastTRCMasked()
  *
- * \param[in]    pixd [optional] null or equal to pixs
- * \param[in]    pixs 8 or 32 bpp; or 2, 4 or 8 bpp with colormap
- * \param[in]    pixm [optional] null or 1 bpp
- * \param[in]    factor  0.0 is no enhancement
+ * \param[in]    pixd     [optional] null or equal to pixs
+ * \param[in]    pixs     8 or 32 bpp; or 2, 4 or 8 bpp with colormap
+ * \param[in]    pixm     [optional] null or 1 bpp
+ * \param[in]    factor   0.0 is no enhancement
  * \return  pixd always
  *
  * <pre>
@@ -536,8 +537,8 @@ NUMA    *nac;
 /*!
  * \brief   numaContrastTRC()
  *
- * \param[in]    factor generally between 0.0 [no enhancement]
- *              and 1.0, but can be larger than 1.0
+ * \param[in]    factor   generally between 0.0 [no enhancement]
+ *                        and 1.0, but can be larger than 1.0
  * \return  na, or NULL on error
  *
  * <pre>
@@ -589,10 +590,10 @@ NUMA      *na;
 /*!
  * \brief   pixEqualizeTRC()
  *
- * \param[in]    pixd [optional] null or equal to pixs
- * \param[in]    pixs 8 bpp gray, 32 bpp rgb, or colormapped
- * \param[in]    fract fraction of equalization movement of pixel values
- * \param[in]    factor subsampling factor; integer >= 1
+ * \param[in]    pixd     [optional] null or equal to pixs
+ * \param[in]    pixs     8 bpp gray, 32 bpp rgb, or colormapped
+ * \param[in]    fract    fraction of equalization movement of pixel values
+ * \param[in]    factor   subsampling factor; integer >= 1
  * \return  pixd, or NULL on error
  *
  * <pre>
@@ -691,9 +692,9 @@ PIXCMAP  *cmap;
 /*!
  * \brief   numaEqualizeTRC()
  *
- * \param[in]    pix 8 bpp, no colormap
- * \param[in]    fract fraction of equalization movement of pixel values
- * \param[in]    factor subsampling factor; integer >= 1
+ * \param[in]    pix     8 bpp, no colormap
+ * \param[in]    fract   fraction of equalization movement of pixel values
+ * \param[in]    factor  subsampling factor; integer >= 1
  * \return  nad, or NULL on error
  *
  * <pre>
@@ -755,9 +756,9 @@ NUMA      *nah, *nasum, *nad;
 /*!
  * \brief   pixTRCMap()
  *
- * \param[in]    pixs 8 grayscale or 32 bpp rgb; not colormapped
- * \param[in]    pixm [optional] 1 bpp mask
- * \param[in]    na mapping array
+ * \param[in]    pixs    8 grayscale or 32 bpp rgb; not colormapped
+ * \param[in]    pixm    [optional] 1 bpp mask
+ * \param[in]    na      mapping array
  * \return  pixd, or NULL on error
  *
  * <pre>
@@ -881,9 +882,9 @@ l_uint32  *data, *datam, *line, *linem;
 /*!
  * \brief   pixUnsharpMasking()
  *
- * \param[in]    pixs all depths except 1 bpp; with or without colormaps
+ * \param[in]    pixs       all depths except 1 bpp; with or without colormaps
  * \param[in]    halfwidth  "half-width" of smoothing filter
- * \param[in]    fract  fraction of edge added back into image
+ * \param[in]    fract      fraction of edge added back into image
  * \return  pixd, or NULL on error
  *
  * <pre>
@@ -950,9 +951,9 @@ PIX     *pixt, *pixd, *pixr, *pixrs, *pixg, *pixgs, *pixb, *pixbs;
 /*!
  * \brief   pixUnsharpMaskingGray()
  *
- * \param[in]    pixs 8 bpp; no colormap
+ * \param[in]    pixs       8 bpp; no colormap
  * \param[in]    halfwidth  "half-width" of smoothing filter
- * \param[in]    fract  fraction of edge added back into image
+ * \param[in]    fract      fraction of edge added back into image
  * \return  pixd, or NULL on error
  *
  * <pre>
@@ -1027,10 +1028,10 @@ PIXACC  *pixacc;
 /*!
  * \brief   pixUnsharpMaskingFast()
  *
- * \param[in]    pixs all depths except 1 bpp; with or without colormaps
+ * \param[in]    pixs       all depths except 1 bpp; with or without colormaps
  * \param[in]    halfwidth  "half-width" of smoothing filter; 1 and 2 only
- * \param[in]    fract  fraction of high frequency added to image
- * \param[in]    direction L_HORIZ, L_VERT, L_BOTH_DIRECTIONS
+ * \param[in]    fract      fraction of high frequency added to image
+ * \param[in]    direction  L_HORIZ, L_VERT, L_BOTH_DIRECTIONS
  * \return  pixd, or NULL on error
  *
  * <pre>
@@ -1120,10 +1121,10 @@ PIX     *pixt, *pixd, *pixr, *pixrs, *pixg, *pixgs, *pixb, *pixbs;
 /*!
  * \brief   pixUnsharpMaskingGrayFast()
  *
- * \param[in]    pixs 8 bpp; no colormap
+ * \param[in]    pixs       8 bpp; no colormap
  * \param[in]    halfwidth  "half-width" of smoothing filter: 1 or 2
- * \param[in]    fract  fraction of high frequency added to image
- * \param[in]    direction L_HORIZ, L_VERT, L_BOTH_DIRECTIONS
+ * \param[in]    fract      fraction of high frequency added to image
+ * \param[in]    direction  L_HORIZ, L_VERT, L_BOTH_DIRECTIONS
  * \return  pixd, or NULL on error
  *
  * <pre>
@@ -1169,10 +1170,10 @@ PIX  *pixd;
 /*!
  * \brief   pixUnsharpMaskingGray1D()
  *
- * \param[in]    pixs 8 bpp; no colormap
- * \param[in]    halfwidth  "half-width" of smoothing filter: 1 or 2
- * \param[in]    fract  fraction of high frequency added to image
- * \param[in]    direction of filtering; use L_HORIZ or L_VERT
+ * \param[in]    pixs        8 bpp; no colormap
+ * \param[in]    halfwidth   "half-width" of smoothing filter: 1 or 2
+ * \param[in]    fract       fraction of high frequency added to image
+ * \param[in]    direction   filtering direction; use L_HORIZ or L_VERT
  * \return  pixd, or NULL on error
  *
  * <pre>
@@ -1304,17 +1305,16 @@ PIX       *pixd;
 /*!
  * \brief   pixUnsharpMaskingGray2D()
  *
- * \param[in]    pixs 8 bpp; no colormap
+ * \param[in]    pixs       8 bpp; no colormap
  * \param[in]    halfwidth  "half-width" of smoothing filter: 1 or 2
- * \param[in]    fract  fraction of high frequency added to image
+ * \param[in]    fract      fraction of high frequency added to image
  * \return  pixd, or NULL on error
  *
  * <pre>
  * Notes:
- *      (1) For halfwidth == 1, we implement the full sharpening filter
- *          directly.  For halfwidth == 2, we implement the the lowpass
- *          filter separably and then compute the sharpening result locally.
- *      (2) Returns a clone if no sharpening is requested.
+ *      (1) This is for %halfwidth == 1, 2.
+ *      (2) The lowpass filter is implemented separably.
+ *      (3) Returns a clone if no sharpening is requested.
  * </pre>
  */
 PIX *
@@ -1323,8 +1323,8 @@ pixUnsharpMaskingGray2D(PIX       *pixs,
                         l_float32  fract)
 {
 l_int32     w, h, d, wpls, wpld, wplf, i, j, ival, sval;
-l_uint32   *datas, *datad, *lines, *lines0, *lines1, *lines2, *lined;
-l_float32   val, a[9];
+l_uint32   *datas, *datad, *lines, *lined;
+l_float32   val, norm;
 l_float32  *dataf, *linef, *linef0, *linef1, *linef2, *linef3, *linef4;
 PIX        *pixd;
 FPIX       *fpix;
@@ -1343,57 +1343,45 @@ FPIX       *fpix;
     if (halfwidth != 1 && halfwidth != 2)
         return (PIX *)ERROR_PTR("halfwidth must be 1 or 2", procName, NULL);
 
-    pixd = pixCopyBorder(NULL, pixs, halfwidth, halfwidth,
-                         halfwidth, halfwidth);
+    if ((pixd = pixCopyBorder(NULL, pixs, halfwidth, halfwidth,
+                              halfwidth, halfwidth)) == NULL)
+        return (PIX *)ERROR_PTR("pixd not made", procName, NULL);
     datad = pixGetData(pixd);
     wpld = pixGetWpl(pixd);
     datas = pixGetData(pixs);
     wpls = pixGetWpl(pixs);
 
-    if (halfwidth == 1) {
-        for (i = 0; i < 9; i++)
-            a[i] = -fract / 9.0;
-        a[4] = 1.0 + fract * 8.0 / 9.0;
-        for (i = 1; i < h - 1; i++) {
-            lines0 = datas + (i - 1) * wpls;
-            lines1 = datas + i * wpls;
-            lines2 = datas + (i + 1) * wpls;
-            lined = datad + i * wpld;
-            for (j = 1; j < w - 1; j++) {
-                val = a[0] * GET_DATA_BYTE(lines0, j - 1) +
-                      a[1] * GET_DATA_BYTE(lines0, j) +
-                      a[2] * GET_DATA_BYTE(lines0, j + 1) +
-                      a[3] * GET_DATA_BYTE(lines1, j - 1) +
-                      a[4] * GET_DATA_BYTE(lines1, j) +
-                      a[5] * GET_DATA_BYTE(lines1, j + 1) +
-                      a[6] * GET_DATA_BYTE(lines2, j - 1) +
-                      a[7] * GET_DATA_BYTE(lines2, j) +
-                      a[8] * GET_DATA_BYTE(lines2, j + 1);
-                ival = (l_int32)(val + 0.5);
-                ival = L_MAX(0, ival);
-                ival = L_MIN(255, ival);
-                SET_DATA_BYTE(lined, j, ival);
-            }
-        }
-
-        return pixd;
+        /* Do the low pass separably.  Store the result of horizontal
+         * smoothing in an intermediate fpix.  */
+    if ((fpix = fpixCreate(w, h)) == NULL) {
+        pixDestroy(&pixd);
+        return (PIX *)ERROR_PTR("fpix not made", procName, NULL);
     }
-
-        /* For halfwidth == 2, do the low pass separably.  Store
-         * the result of horizontal smoothing in an intermediate fpix. */
-    fpix = fpixCreate(w, h);
     dataf = fpixGetData(fpix);
     wplf = fpixGetWpl(fpix);
-    for (i = 2; i < h - 2; i++) {
-        lines = datas + i * wpls;
-        linef = dataf + i * wplf;
-        for (j = 2; j < w - 2; j++) {
-            val = GET_DATA_BYTE(lines, j - 2) +
-                  GET_DATA_BYTE(lines, j - 1) +
-                  GET_DATA_BYTE(lines, j) +
-                  GET_DATA_BYTE(lines, j + 1) +
-                  GET_DATA_BYTE(lines, j + 2);
-            linef[j] = val;
+    if (halfwidth == 1) {
+        for (i = 0; i < h; i++) {
+            lines = datas + i * wpls;
+            linef = dataf + i * wplf;
+            for (j = 1; j < w - 1; j++) {
+                val = GET_DATA_BYTE(lines, j - 1) +
+                      GET_DATA_BYTE(lines, j) +
+                      GET_DATA_BYTE(lines, j + 1);
+                linef[j] = val;
+            }
+        }
+    } else {
+        for (i = 0; i < h; i++) {
+            lines = datas + i * wpls;
+            linef = dataf + i * wplf;
+            for (j = 2; j < w - 2; j++) {
+                val = GET_DATA_BYTE(lines, j - 2) +
+                      GET_DATA_BYTE(lines, j - 1) +
+                      GET_DATA_BYTE(lines, j) +
+                      GET_DATA_BYTE(lines, j + 1) +
+                      GET_DATA_BYTE(lines, j + 2);
+                linef[j] = val;
+            }
         }
     }
 
@@ -1403,24 +1391,44 @@ FPIX       *fpix;
          * be added to I, then the highpass filter value is
          *     H = I - L
          * and the new sharpened value is
-         *     N = I + f * H.
-         */
-    for (i = 2; i < h - 2; i++) {
-        linef0 = dataf + (i - 2) * wplf;
-        linef1 = dataf + (i - 1) * wplf;
-        linef2 = dataf + i * wplf;
-        linef3 = dataf + (i + 1) * wplf;
-        linef4 = dataf + (i + 2) * wplf;
-        lined = datad + i * wpld;
-        lines = datas + i * wpls;
-        for (j = 2; j < w - 2; j++) {
-            val = 0.04 * (linef0[j] + linef1[j] + linef2[j] +
-                          linef3[j] + linef4[j]);  /* L: lowpass filter value */
-            sval = GET_DATA_BYTE(lines, j);   /* I: source pixel */
-            ival = (l_int32)(sval + fract * (sval - val) + 0.5);
-            ival = L_MAX(0, ival);
-            ival = L_MIN(255, ival);
-            SET_DATA_BYTE(lined, j, ival);
+         *     N = I + f * H.                 */
+    if (halfwidth == 1) {
+        for (i = 1; i < h - 1; i++) {
+            linef0 = dataf + (i - 1) * wplf;
+            linef1 = dataf + i * wplf;
+            linef2 = dataf + (i + 1) * wplf;
+            lined = datad + i * wpld;
+            lines = datas + i * wpls;
+            norm = 1.0 / 9.0;
+            for (j = 1; j < w - 1; j++) {
+                val = norm * (linef0[j] + linef1[j] +
+                              linef2[j]);         /* L: lowpass filter value */
+                sval = GET_DATA_BYTE(lines, j);   /* I: source pixel */
+                ival = (l_int32)(sval + fract * (sval - val) + 0.5);
+                ival = L_MAX(0, ival);
+                ival = L_MIN(255, ival);
+                SET_DATA_BYTE(lined, j, ival);
+            }
+        }
+    } else {
+        for (i = 2; i < h - 2; i++) {
+            linef0 = dataf + (i - 2) * wplf;
+            linef1 = dataf + (i - 1) * wplf;
+            linef2 = dataf + i * wplf;
+            linef3 = dataf + (i + 1) * wplf;
+            linef4 = dataf + (i + 2) * wplf;
+            lined = datad + i * wpld;
+            lines = datas + i * wpls;
+            norm = 1.0 / 25.0;
+            for (j = 2; j < w - 2; j++) {
+                val = norm * (linef0[j] + linef1[j] + linef2[j] + linef3[j] +
+                              linef4[j]);  /* L: lowpass filter value */
+                sval = GET_DATA_BYTE(lines, j);   /* I: source pixel */
+                ival = (l_int32)(sval + fract * (sval - val) + 0.5);
+                ival = L_MAX(0, ival);
+                ival = L_MIN(255, ival);
+                SET_DATA_BYTE(lined, j, ival);
+            }
         }
     }
 
@@ -1429,16 +1437,15 @@ FPIX       *fpix;
 }
 
 
-
 /*-----------------------------------------------------------------------*
  *                    Hue and saturation modification                    *
  *-----------------------------------------------------------------------*/
 /*!
  * \brief   pixModifyHue()
  *
- * \param[in]    pixd [optional] can be null or equal to pixs
- * \param[in]    pixs 32 bpp rgb
- * \param[in]    fract between -1.0 and 1.0
+ * \param[in]    pixd      [optional] can be null or equal to pixs
+ * \param[in]    pixs      32 bpp rgb
+ * \param[in]    fract     between -1.0 and 1.0
  * \return  pixd, or NULL on error
  *
  * <pre>
@@ -1510,9 +1517,9 @@ l_uint32  *data, *line;
 /*!
  * \brief   pixModifySaturation()
  *
- * \param[in]    pixd [optional] can be null, existing or equal to pixs
- * \param[in]    pixs 32 bpp rgb
- * \param[in]    fract between -1.0 and 1.0
+ * \param[in]    pixd     [optional] can be null, existing or equal to pixs
+ * \param[in]    pixs     32 bpp rgb
+ * \param[in]    fract    between -1.0 and 1.0
  * \return  pixd, or NULL on error
  *
  * <pre>
@@ -1578,9 +1585,9 @@ l_uint32  *data, *line;
 /*!
  * \brief   pixMeasureSaturation()
  *
- * \param[in]    pixs 32 bpp rgb
- * \param[in]    factor subsampling factor; integer >= 1
- * \param[out]   psat average saturation
+ * \param[in]    pixs     32 bpp rgb
+ * \param[in]    factor   subsampling factor; integer >= 1
+ * \param[out]   psat     average saturation
  * \return  pixd, or NULL on error
  */
 l_int32
@@ -1626,9 +1633,9 @@ l_uint32  *data, *line;
 /*!
  * \brief   pixModifyBrightness()
  *
- * \param[in]    pixd [optional] can be null, existing or equal to pixs
- * \param[in]    pixs 32 bpp rgb
- * \param[in]    fract between -1.0 and 1.0
+ * \param[in]    pixd     [optional] can be null, existing or equal to pixs
+ * \param[in]    pixs     32 bpp rgb
+ * \param[in]    fract    between -1.0 and 1.0
  * \return  pixd, or NULL on error
  *
  * <pre>
@@ -1694,13 +1701,113 @@ l_uint32  *data, *line;
 /*-----------------------------------------------------------------------*
  *                             Color shifting                            *
  *-----------------------------------------------------------------------*/
+/*!
+ * \brief   pixMosaicColorShiftRGB()
+ *
+ * \param[in]    pixs     32 bpp rgb
+ * \param[in]    roff   center offset of red component
+ * \param[in]    goff   center offset of green component
+ * \param[in]    boff   center offset of blue component
+ * \param[in]    delta  increments from center offsets [0.0 - 0.1];
+ *                      use 0.0 to get the default (0.04)
+ * \param[in]    nincr  number of increments in each (positive and negative)
+ *                      direction; use 0 to get the default (2).
+ * \return  pix, or NULL on error
+ *
+ * <pre>
+ * Notes:
+ *      (1) This generates a mosaic view of the effect of shifting the RGB
+ *          components.  See pixColorShiftRGB() for details on the shifting.
+ *      (2) The offsets (%roff, %goff, %boff) set the color center point,
+ *          and the deviations from this are shown separately for deltas
+ *          in r, g and b.  For each component, we show 2 * %nincr + 1
+ *          images.
+ *      (3) Usage: color prints differ from the original due to three factors:
+ *          illumination, calibration of the camera in acquisition,
+ *          and calibration of the printer.  This function can be used
+ *          to iteratively match a color print to the original.  On each
+ *          iteration, the center offsets are set to the best match so
+ *          far, and the @delta increments are typically reduced.
+ * </pre>
+ */
+PIX *
+pixMosaicColorShiftRGB(PIX       *pixs,
+                       l_float32  roff,
+                       l_float32  goff,
+                       l_float32  boff,
+                       l_float32  delta,
+                       l_int32    nincr)
+{
+char       buf[64];
+l_int32    i, j;
+l_float32  del;
+L_BMF     *bmf;
+PIX       *pix1, *pix2, *pix3;
+PIXA      *pixa;
+
+    PROCNAME("pixMosaicColorShiftRGB");
+
+    if (!pixs  || pixGetDepth(pixs) != 32)
+        return (PIX *)ERROR_PTR("pixs undefined or not rgb", procName, NULL);
+    if (roff < -1.0 || roff > 1.0)
+        return (PIX *)ERROR_PTR("roff not in [-1.0, 1.0]", procName, NULL);
+    if (goff < -1.0 || goff > 1.0)
+        return (PIX *)ERROR_PTR("goff not in [-1.0, 1.0]", procName, NULL);
+    if (boff < -1.0 || boff > 1.0)
+        return (PIX *)ERROR_PTR("boff not in [-1.0, 1.0]", procName, NULL);
+    if (delta < 0.0 || delta > 0.1)
+        return (PIX *)ERROR_PTR("delta not in [0.0, 0.1]", procName, NULL);
+    if (delta == 0.0) delta = 0.04;
+    if (nincr < 0 || nincr > 6)
+        return (PIX *)ERROR_PTR("nincr not in [0, 6]", procName, NULL);
+    if (nincr == 0) nincr = 2;
+
+    pixa = pixaCreate(3 * (2 * nincr + 1));
+    bmf = bmfCreate(NULL, 8);
+    pix1 = pixScaleToSize(pixs, 400, 0);
+    for (i = 0, del = - nincr * delta; i < 2 * nincr + 1; i++, del += delta) {
+        pix2 = pixColorShiftRGB(pix1, roff + del, goff, boff);
+        snprintf(buf, sizeof(buf), "%4.2f, %4.2f, %4.2f",
+                 roff + del, goff, boff);
+        pix3 = pixAddSingleTextblock(pix2, bmf, buf, 0xff000000,
+                                     L_ADD_BELOW, 0);
+        pixaAddPix(pixa, pix3, L_INSERT);
+        pixDestroy(&pix2);
+    }
+    for (i = 0, del = - nincr * delta; i < 2 * nincr + 1; i++, del += delta) {
+        pix2 = pixColorShiftRGB(pix1, roff, goff + del, boff);
+        snprintf(buf, sizeof(buf), "%4.2f, %4.2f, %4.2f",
+                 roff, goff + del, boff);
+        pix3 = pixAddSingleTextblock(pix2, bmf, buf, 0xff000000,
+                                     L_ADD_BELOW, 0);
+        pixaAddPix(pixa, pix3, L_INSERT);
+        pixDestroy(&pix2);
+    }
+    for (i = 0, del = - nincr * delta; i < 2 * nincr + 1; i++, del += delta) {
+        pix2 = pixColorShiftRGB(pix1, roff, goff, boff + del);
+        snprintf(buf, sizeof(buf), "%4.2f, %4.2f, %4.2f",
+                 roff, goff, boff + del);
+        pix3 = pixAddSingleTextblock(pix2, bmf, buf, 0xff000000,
+                                     L_ADD_BELOW, 0);
+        pixaAddPix(pixa, pix3, L_INSERT);
+        pixDestroy(&pix2);
+    }
+    pixDestroy(&pix1);
+
+    pix1 = pixaDisplayTiledAndScaled(pixa, 32, 300, 2 * nincr + 1, 0, 30, 2);
+    pixaDestroy(&pixa);
+    bmfDestroy(&bmf);
+    return pix1;
+}
+
+
 /*!
  * \brief   pixColorShiftRGB()
  *
- * \param[in]    pixs 32 bpp rgb
- * \param[in]    rfract fractional shift in red component
- * \param[in]    gfract fractional shift in green component
- * \param[in]    bfract fractional shift in blue component
+ * \param[in]    pixs     32 bpp rgb
+ * \param[in]    rfract   fractional shift in red component
+ * \param[in]    gfract   fractional shift in green component
+ * \param[in]    bfract   fractional shift in blue component
  * \return  pixd, or NULL on error
  *
  * <pre>
@@ -1738,11 +1845,11 @@ PIX       *pixd;
     if (pixGetDepth(pixs) != 32)
         return (PIX *)ERROR_PTR("pixs not 32 bpp", procName, NULL);
     if (rfract < -1.0 || rfract > 1.0)
-        return (PIX *)ERROR_PTR("rfract not in [-1.0,...,1.0]", procName, NULL);
+        return (PIX *)ERROR_PTR("rfract not in [-1.0, 1.0]", procName, NULL);
     if (gfract < -1.0 || gfract > 1.0)
-        return (PIX *)ERROR_PTR("gfract not in [-1.0,...,1.0]", procName, NULL);
+        return (PIX *)ERROR_PTR("gfract not in [-1.0, 1.0]", procName, NULL);
     if (bfract < -1.0 || bfract > 1.0)
-        return (PIX *)ERROR_PTR("bfract not in [-1.0,...,1.0]", procName, NULL);
+        return (PIX *)ERROR_PTR("bfract not in [-1.0, 1.0]", procName, NULL);
     if (rfract == 0.0 && gfract == 0.0 && bfract == 0.0)
         return pixCopy(NULL, pixs);
 
@@ -1796,8 +1903,10 @@ PIX       *pixd;
 /*!
  * \brief   pixMultConstantColor()
  *
- * \param[in]    pixs colormapped or rgb
- * \param[in]    rfact, gfact, bfact multiplicative factors on each component
+ * \param[in]    pixs     colormapped or rgb
+ * \param[in]    rfact    red multiplicative factor
+ * \param[in]    gfact    green multiplicative factor
+ * \param[in]    bfact    blue multiplicative factor
  * \return  pixd colormapped or rgb, with colors scaled, or NULL on error
  *
  * <pre>
@@ -1881,8 +1990,8 @@ PIXCMAP   *cmap;
 /*!
  * \brief   pixMultMatrixColor()
  *
- * \param[in]    pixs colormapped or rgb
- * \param[in]    kel kernel 3x3 matrix of floats
+ * \param[in]    pixs    colormapped or rgb
+ * \param[in]    kel     kernel 3x3 matrix of floats
  * \return  pixd colormapped or rgb, or NULL on error
  *
  * <pre>
@@ -1991,10 +2100,10 @@ PIXCMAP   *cmap;
 /*!
  * \brief   pixHalfEdgeByBandpass()
  *
- * \param[in]    pixs 8 bpp gray or 32 bpp rgb
- * \param[in]    sm1h, sm1v "half-widths" of smoothing filter sm1
- * \param[in]    sm2h, sm2v "half-widths" of smoothing filter sm2;
- *                      require sm2 != sm1
+ * \param[in]    pixs         8 bpp gray or 32 bpp rgb
+ * \param[in]    sm1h, sm1v   "half-widths" of smoothing filter sm1
+ * \param[in]    sm2h, sm2v   "half-widths" of smoothing filter sm2;
+ *                            require sm2 != sm1
  * \return  pixd, or NULL on error
  *
  * <pre>
@@ -2045,18 +2154,25 @@ PIX     *pixg, *pixacc, *pixc1, *pixc2;
         pixg = pixClone(pixs);
 
         /* Make a convolution accumulator and use it twice */
-    if ((pixacc = pixBlockconvAccum(pixg)) == NULL)
+    if ((pixacc = pixBlockconvAccum(pixg)) == NULL) {
+        pixDestroy(&pixg);
         return (PIX *)ERROR_PTR("pixacc not made", procName, NULL);
-    if ((pixc1 = pixBlockconvGray(pixg, pixacc, sm1h, sm1v)) == NULL)
+    }
+    if ((pixc1 = pixBlockconvGray(pixg, pixacc, sm1h, sm1v)) == NULL) {
+        pixDestroy(&pixg);
+        pixDestroy(&pixacc);
         return (PIX *)ERROR_PTR("pixc1 not made", procName, NULL);
-    if ((pixc2 = pixBlockconvGray(pixg, pixacc, sm2h, sm2v)) == NULL)
-        return (PIX *)ERROR_PTR("pixc2 not made", procName, NULL);
+    }
+    pixc2 = pixBlockconvGray(pixg, pixacc, sm2h, sm2v);
+    pixDestroy(&pixg);
     pixDestroy(&pixacc);
+    if (!pixc2) {
+        pixDestroy(&pixc1);
+        return (PIX *)ERROR_PTR("pixc2 not made", procName, NULL);
+    }
 
         /* Compute the half-edge using pixc1 - pixc2.  */
     pixSubtractGray(pixc1, pixc1, pixc2);
-
-    pixDestroy(&pixg);
     pixDestroy(&pixc2);
     return pixc1;
 }
