@@ -274,9 +274,9 @@ l_int32  ret1, ret2;
  *          opening or closing for any of the sels in the input sela.
  *      (2) The fileindex parameter is inserted into the output
  *          filename, as described below.
- *      (3) If filename == NULL, the output file is fmorphgen.\<n\>.c,
- *          where \<n\> is equal to the 'fileindex' parameter.
- *      (4) If filename != NULL, the output file is \<filename\>.\<n\>.c.
+ *      (3) If filename == NULL, the output file is fmorphgen.<n>.c,
+ *          where <n> is equal to the 'fileindex' parameter.
+ *      (4) If filename != NULL, the output file is <filename>.<n>.c.
  * </pre>
  */
 l_int32
@@ -304,16 +304,16 @@ SARRAY  *sa1, *sa2, *sa3;
     if ((nsels = selaGetCount(sela)) == 0)
         return ERROR_INT("no sels in sela", procName, 1);
 
-        /* Make array of sel names */
-    sa1 = selaGetSelnames(sela);
-
         /* Make array of textlines from morphtemplate1.txt */
     if ((filestr = (char *)l_binaryRead(TEMPLATE1, &size)) == NULL)
         return ERROR_INT("filestr not made", procName, 1);
-    if ((sa2 = sarrayCreateLinesFromString(filestr, 1)) == NULL)
-        return ERROR_INT("sa2 not made", procName, 1);
+    sa2 = sarrayCreateLinesFromString(filestr, 1);
     LEPT_FREE(filestr);
-/*    sarrayWriteStream(stderr, sa2); */
+    if (!sa2)
+        return ERROR_INT("sa2 not made", procName, 1);
+
+        /* Make array of sel names */
+    sa1 = selaGetSelnames(sela);
 
         /* Make strings containing function call names */
     sprintf(bigbuf, "PIX *pixMorphDwa_%d(PIX *pixd, PIX *pixs, "
@@ -369,8 +369,7 @@ SARRAY  *sa1, *sa2, *sa3;
     str_low_dtp1 = stringNew(bigbuf);
 
         /* Make the output sa */
-    if ((sa3 = sarrayCreate(0)) == NULL)
-        return ERROR_INT("sa3 not made", procName, 1);
+    sa3 = sarrayCreate(0);
 
         /* Copyright notice and info header */
     sarrayParseRange(sa2, 0, &actstart, &end, &newstart, "--", 0);
@@ -453,8 +452,7 @@ SARRAY  *sa1, *sa2, *sa3;
     sarrayAppendRange(sa3, sa2, actstart, end);
 
         /* Output to file */
-    if ((filestr = sarrayToString(sa3, 1)) == NULL)
-        return ERROR_INT("filestr from sa3 not made", procName, 1);
+    filestr = sarrayToString(sa3, 1);
     nbytes = strlen(filestr);
     if (filename)
         snprintf(bigbuf, L_BUF_SIZE, "%s.%d.c", filename, fileindex);
@@ -516,13 +514,16 @@ SEL     *sel;
         /* Make the array of textlines from morphtemplate2.txt */
     if ((filestr = (char *)l_binaryRead(TEMPLATE2, &size)) == NULL)
         return ERROR_INT("filestr not made", procName, 1);
-    if ((sa1 = sarrayCreateLinesFromString(filestr, 1)) == NULL)
-        return ERROR_INT("sa1 not made", procName, 1);
+    sa1 = sarrayCreateLinesFromString(filestr, 1);
     LEPT_FREE(filestr);
+    if (!sa1)
+        return ERROR_INT("sa1 not made", procName, 1);
 
         /* Make the array of static function names */
-    if ((sa2 = sarrayCreate(2 * nsels)) == NULL)
+    if ((sa2 = sarrayCreate(2 * nsels)) == NULL) {
+        sarrayDestroy(&sa1);
         return ERROR_INT("sa2 not made", procName, 1);
+    }
     for (i = 0; i < nsels; i++) {
         sprintf(bigbuf, "fdilate_%d_%d", fileindex, i);
         sarrayAddString(sa2, bigbuf, L_COPY);
@@ -531,8 +532,7 @@ SEL     *sel;
     }
 
         /* Make the static prototype strings */
-    if ((sa3 = sarrayCreate(2 * nsels)) == NULL)
-        return ERROR_INT("sa3 not made", procName, 1);
+    sa3 = sarrayCreate(2 * nsels);  /* should be ok */
     for (i = 0; i < 2 * nsels; i++) {
         fname = sarrayGetString(sa2, i, L_NOCOPY);
         sprintf(bigbuf, "static void  %s%s", fname, PROTOARGS);
@@ -547,14 +547,9 @@ SEL     *sel;
     str_doc2 = stringNew(bigbuf);
     sprintf(bigbuf, " *             void       ferode_%d_*()", fileindex);
     str_doc3 = stringNew(bigbuf);
-    sprintf(bigbuf, " *  fmorphopgen_low_%d()", fileindex);
-    str_doc4 = stringNew(bigbuf);
-    sprintf(bigbuf, "fmorphopgen_low_%d(l_uint32  *datad,", fileindex);
-    str_def1 = stringNew(bigbuf);
 
         /* Output to this sa */
-    if ((sa4 = sarrayCreate(0)) == NULL)
-        return ERROR_INT("sa4 not made", procName, 1);
+    sa4 = sarrayCreate(0);
 
         /* Copyright notice and info header */
     sarrayParseRange(sa1, 0, &actstart, &end, &newstart, "--", 0);
@@ -571,11 +566,22 @@ SEL     *sel;
 
         /* Insert static protos */
     for (i = 0; i < 2 * nsels; i++) {
-        if ((linestr = sarrayGetString(sa3, i, L_COPY)) == NULL)
+        if ((linestr = sarrayGetString(sa3, i, L_COPY)) == NULL) {
+            sarrayDestroy(&sa1);
+            sarrayDestroy(&sa2);
+            sarrayDestroy(&sa3);
+            sarrayDestroy(&sa4);
             return ERROR_INT("linestr not retrieved", procName, 1);
+        }
         sarrayAddString(sa4, linestr, L_INSERT);
     }
 
+        /* More strings with function names */
+    sprintf(bigbuf, " *  fmorphopgen_low_%d()", fileindex);
+    str_doc4 = stringNew(bigbuf);
+    sprintf(bigbuf, "fmorphopgen_low_%d(l_uint32  *datad,", fileindex);
+    str_def1 = stringNew(bigbuf);
+
         /* Insert function header */
     sarrayParseRange(sa1, newstart, &actstart, &end, &newstart, "--", 0);
     sarrayAppendRange(sa4, sa1, actstart, end);
@@ -619,10 +625,14 @@ SEL     *sel;
         sarrayAppendRange(sa4, sa1, argstart, argend);
 
             /* Declare and define wplsN args, as necessary */
-        if ((sel = selaGetSel(sela, i/2)) == NULL)
+        if ((sel = selaGetSel(sela, i/2)) == NULL) {
+            sarrayDestroy(&sa1);
+            sarrayDestroy(&sa2);
+            sarrayDestroy(&sa3);
+            sarrayDestroy(&sa4);
             return ERROR_INT("sel not returned", procName, 1);
-        if ((sa5 = sarrayMakeWplsCode(sel)) == NULL)
-            return ERROR_INT("sa5 not made", procName, 1);
+        }
+        sa5 = sarrayMakeWplsCode(sel);
         sarrayJoin(sa4, sa5);
         sarrayDestroy(&sa5);
 
@@ -630,8 +640,7 @@ SEL     *sel;
         sarrayAppendRange(sa4, sa1, loopstart, loopend);
 
             /* Insert barrel-op code for *dptr */
-        if ((sa6 = sarrayMakeInnerLoopDWACode(sel, i)) == NULL)
-            return ERROR_INT("sa6 not made", procName, 1);
+        sa6 = sarrayMakeInnerLoopDWACode(sel, i);
         sarrayJoin(sa4, sa6);
         sarrayDestroy(&sa6);
 
@@ -640,8 +649,7 @@ SEL     *sel;
     }
 
         /* Output to file */
-    if ((filestr = sarrayToString(sa4, 1)) == NULL)
-        return ERROR_INT("filestr from sa4 not made", procName, 1);
+    filestr = sarrayToString(sa4, 1);
     nbytes = strlen(filestr);
     if (filename)
         snprintf(bigbuf, L_BUF_SIZE, "%slow.%d.c", filename, fileindex);
@@ -653,7 +661,6 @@ SEL     *sel;
     sarrayDestroy(&sa3);
     sarrayDestroy(&sa4);
     LEPT_FREE(filestr);
-
     return 0;
 }
 
@@ -704,8 +711,7 @@ SARRAY  *sa;
         }
     }
 
-    if ((sa = sarrayCreate(0)) == NULL)
-        return (SARRAY *)ERROR_PTR("sa not made", procName, NULL);
+    sa = sarrayCreate(0);
 
         /* Add declarations */
     if (allvshifts == TRUE) {   /* packs them as well as possible */
@@ -779,8 +785,7 @@ SARRAY  *sa;
         }
     }
 
-    if ((sa = sarrayCreate(0)) == NULL)
-        return (SARRAY *)ERROR_PTR("sa not made", procName, NULL);
+    sa = sarrayCreate(0);
     if (count == 0) {
         L_WARNING("no hits in Sel %d\n", procName, index);
         return sa;  /* no code inside! */
@@ -794,7 +799,7 @@ SARRAY  *sa;
                 if (optype == L_MORPH_DILATE) {
                     dely = sel->cy - i;
                     delx = sel->cx - j;
-                } else if (optype == L_MORPH_ERODE) {
+                } else {  /* optype == L_MORPH_ERODE */
                     dely = i - sel->cy;
                     delx = j - sel->cx;
                 }
