@@ -28,51 +28,44 @@
  * \file gifio.c
  * <pre>
  *
- *    Read gif file
+ *    Reading gif
  *          PIX            *pixReadStreamGif()
- *          static PIX     *gifToPix()
- *          static PIX     *pixUninterlaceGIF()
- *
- *    Write gif file
- *          l_int32         pixWriteStreamGif()
- *          static l_int32  pixToGif()
- *
- *    Read/write gif from/to memory
  *          PIX            *pixReadMemGif()
  *          static l_int32  gifReadFunc()
+ *          static PIX     *gifToPix()
+ *
+ *    Writing gif
+ *          l_int32         pixWriteStreamGif()
  *          l_int32         pixWriteMemGif()
  *          static l_int32  gifWriteFunc()
+ *          static l_int32  pixToGif()
+ *
+ *    Removing interlacing
+ *          static PIX     *pixUninterlaceGIF()
  *
  *    The initial version of this module was generously contribued by
- *    Antony Dovgal.  He can be contacted at:  tony *AT* daylessday.org
+ *    Antony Dovgal.
  *
  *    The functions that read and write from pix to gif-compressed memory,
  *    using gif internal functions DGifOpen() and EGifOpen() that are
  *    available in 5.1 and later, were contributed by Tobias Peirick.
  *
- *    Important version information:
+ *    Version information:
  *
- *    (1) This uses the gif library, version 4.1.6 or later.
- *        Do not use 4.1.4.  It has serious problems handling 1 bpp images.
- *    (2) There are some issues with version 5.0:
- *        ~ valgrind detects uninitialized values used used for writing
- *          and conditionally jumping in EGifPutScreenDesc().  This has
- *          not been fixed as of 1/20/2016.
- *        ~ DGifSlurp() crashes on some images, apparently triggered by
- *          by some GIF extension records.  This has been fixed as of 5.1.
- *    (3) E. Raymond has changed the high-level interface with 5.0
- *        and again with 5.1, and to keep up we have used macros
- *        determined by the major and minor version numbers.  Note that
- *        tiff, jpeg, png and webp have maintained a consistent high-level
- *        interface through all versions; version-dependent code paths
- *        are not required to use them.
+ *    (1) This supports the gif library, version 5.1 or later, for which
+ *        gif read-from-mem and write-to-mem allow these operations
+ *        without writing temporary files.
+ *    (2) There has never been a gif stream interface.  For versions
+ *        before 5.1, it was necessary to use a file descriptor, and to
+ *        generate a file stream from the low-level descriptor.  With the
+ *        memory interface in 5.1 that can be used on all platforms, it
+ *        is no longer necessary to use any API code with file descriptors.
+ *    (3) The public interface changed with 5.0 and with 5.1, and we
+ *        no longer support 4.6.1 and 5.0.
  *    (4) Version 5.1.2 came out on Jan 7, 2016.  Leptonica cannot
  *        successfully read gif files that it writes with this version;
  *        DGifSlurp() gets an internal error from an uninitialized array
- *        and returns failure.  E. Raymond fixed the problem for 5.1.3,
- *        and we disable leptonica with 5.1.2.
- *    (5) For Version 5.1 and later, the gif read-from-mem and write-to-mem
- *        now work without writing temporary files.
+ *        and returns failure.  The problem was fixed in 5.1.3.
  * </pre>
  */
 
@@ -81,12 +74,6 @@
 #endif  /* HAVE_CONFIG_H */
 
 #include <string.h>
-#include <sys/types.h>
-#ifndef _MSC_VER
-#include <unistd.h>
-#else
-#include <io.h>
-#endif  /* _MSC_VER */
 #include "allheaders.h"
 
 /* --------------------------------------------------------------------*/
@@ -95,34 +82,12 @@
 
 #include "gif_lib.h"
 
-    /* GIF supports 4-way interlacing by raster lines.
-     * Leptonica restores interlaced data to normal raster order. */
-static PIX * pixUninterlaceGIF(PIX  *pixs);
-static const l_int32 InterlacedOffset[] = {0, 4, 2, 1};
-static const l_int32 InterlacedJumps[] = {8, 8, 4, 2};
-
     /* Interface that enables low-level GIF support for reading from memory */
 static PIX * gifToPix(GifFileType *gif);
     /* Interface that enables low-level GIF support for writing to memory */
 static l_int32 pixToGif(PIX *pix, GifFileType *gif);
 
-    /* Basic interface changed in 5.0 (!).  We have to do this for
-     * backward compatibililty with 4.1.6. */
-#if GIFLIB_MAJOR < 5
-#define GifMakeMapObject         MakeMapObject
-#define GifFreeMapObject         FreeMapObject
-#define DGifOpenFileHandle(a,b)  DGifOpenFileHandle(a)
-#define EGifOpenFileHandle(a,b)  EGifOpenFileHandle(a)
-#endif  /* before 5.0 */
-
-    /* Basic interface changed again in 5.1 (!) */
-#if GIFLIB_MAJOR < 5 || (GIFLIB_MAJOR == 5 && GIFLIB_MINOR == 0)
-#define DGifCloseFile(a,b)       DGifCloseFile(a)
-#define EGifCloseFile(a,b)       EGifCloseFile(a)
-#endif  /* 4.1.6 or 5.0 */
-
     /*! For in-memory decoding of GIF; 5.1+ */
-#if (GIFLIB_MAJOR == 5 && GIFLIB_MINOR >= 1) || GIFLIB_MAJOR > 5
 typedef struct GifReadBuffer
 {
     size_t            size;    /*!< size of buffer                           */
@@ -136,55 +101,129 @@ static l_int32  gifReadFunc(GifFileType *gif, GifByteType *dest,
     /*! Low-level callback for in-memory encoding */
 static l_int32  gifWriteFunc(GifFileType *gif, const GifByteType *src,
                              l_int32 bytesToWrite);
-#endif  /* 5.1 and beyond */
+
+    /* GIF supports 4-way interlacing by raster lines.
+     * Before 5.0, it was necessary for leptonica to restore interlaced
+     * data to normal raster order when reading to a pix. With 5.0,
+     * the de-interlacing is done by the library read function. */
+static PIX * pixUninterlaceGIF(PIX  *pixs);
+static const l_int32 InterlacedOffset[] = {0, 4, 2, 1};
+static const l_int32 InterlacedJumps[] = {8, 8, 4, 2};
 
 
 /*---------------------------------------------------------------------*
- *                       Reading gif from file                         *
+ *                            Reading gif                              *
  *---------------------------------------------------------------------*/
 /*!
  * \brief   pixReadStreamGif()
  *
- * \param[in]    fp file stream
+ * \param[in]  fp   file stream opened for reading
  * \return  pix, or NULL on error
  */
 PIX *
 pixReadStreamGif(FILE  *fp)
 {
-l_int32          fd;
-GifFileType     *gif;
+l_uint8  *filedata;
+size_t    filesize;
+PIX      *pix;
 
     PROCNAME("pixReadStreamGif");
 
+    if (!fp)
+        return (PIX *)ERROR_PTR("fp not defined", procName, NULL);
+
+        /* Read data into memory from file */
+    rewind(fp);
+    if ((filedata = l_binaryReadStream(fp, &filesize)) == NULL)
+        return (PIX *)ERROR_PTR("filedata not read", procName, NULL);
+
+        /* Uncompress from memory */
+    pix = pixReadMemGif(filedata, filesize);
+    LEPT_FREE(filedata);
+    if (!pix)
+        L_ERROR("failed to read gif from file data\n", procName);
+    return pix;
+}
+
+
+/*!
+ * \brief   pixReadMemGif()
+ *
+ * \param[in]  cdata    const; gif-encoded
+ * \param[in]  size     bytes data
+ * \return  pix, or NULL on error
+ *
+ * <pre>
+ * Notes:
+ *     (1) For libgif version >= 5.1, this uses the DGifOpen() buffer
+ *         interface.  No temp files are required.
+ *     (2) For libgif version < 5.1, it was necessary to write the compressed
+ *         data to file and read it back, and we couldn't use the GNU
+ *         runtime extension fmemopen() because libgif doesn't have a file
+ *         stream interface.
+ * </pre>
+ */
+PIX *
+pixReadMemGif(const l_uint8  *cdata,
+              size_t          size)
+{
+GifFileType   *gif;
+GifReadBuffer  buffer;
+
+    PROCNAME("pixReadMemGif");
+
+        /* 5.1+ and not 5.1.2 */
+#if (GIFLIB_MAJOR < 5 || (GIFLIB_MAJOR == 5 && GIFLIB_MINOR == 0))
+    L_ERROR("Require giflib-5.1 or later\n", procName);
+    return NULL;
+#endif  /* < 5.1 */
 #if GIFLIB_MAJOR == 5 && GIFLIB_MINOR == 1 && GIFLIB_RELEASE == 2  /* 5.1.2 */
-    L_ERROR("Can't use giflib-5.1.2; suggest 5.1.1 or earlier\n", procName);
+    L_ERROR("Can't use giflib-5.1.2; suggest 5.1.3 or later\n", procName);
     return NULL;
 #endif  /* 5.1.2 */
 
-    if ((fd = fileno(fp)) < 0)
-        return (PIX *)ERROR_PTR("invalid file descriptor", procName, NULL);
-#ifdef _WIN32
-    fd = _dup(fd);
-#endif /* _WIN32 */
+    if (!cdata)
+        return (PIX *)ERROR_PTR("cdata not defined", procName, NULL);
 
-#ifndef _MSC_VER
-    lseek(fd, 0, SEEK_SET);
-#else
-    _lseek(fd, 0, SEEK_SET);
-#endif  /* _MSC_VER */
-
-    if ((gif = DGifOpenFileHandle(fd, NULL)) == NULL)
-        return (PIX *)ERROR_PTR("invalid file or file not found",
+    buffer.cdata = cdata;
+    buffer.size = size;
+    buffer.pos = 0;
+    if ((gif = DGifOpen((void*)&buffer, gifReadFunc, NULL)) == NULL)
+        return (PIX *)ERROR_PTR("could not open gif stream from memory",
                                 procName, NULL);
 
     return gifToPix(gif);
 }
 
 
+static l_int32
+gifReadFunc(GifFileType  *gif,
+            GifByteType  *dest,
+            l_int32       bytesToRead)
+{
+GifReadBuffer  *buffer;
+l_int32         bytesRead;
+
+    PROCNAME("gifReadFunc");
+
+    if ((buffer = (GifReadBuffer*)gif->UserData) == NULL)
+        return ERROR_INT("UserData not set", procName, -1);
+
+    if(buffer->pos >= buffer->size)
+        return -1;
+
+    bytesRead = (buffer->pos < buffer->size - bytesToRead)
+              ? bytesToRead : buffer->size - buffer->pos;
+    memcpy(dest, buffer->cdata + buffer->pos, bytesRead);
+    buffer->pos += bytesRead;
+    return bytesRead;
+}
+
+
 /*!
  * \brief   gifToPix()
  *
- * \param[in]    gif  opened gif stream
+ * \param[in]  gif   opened gif stream
  * \return  pix, or NULL on error
  *
  * <pre>
@@ -200,13 +239,11 @@ gifToPix(GifFileType  *gif)
 l_int32          wpl, i, j, w, h, d, cindex, ncolors;
 l_int32          rval, gval, bval;
 l_uint32        *data, *line;
-PIX             *pixd, *pixdi;
+PIX             *pixd;
 PIXCMAP         *cmap;
 ColorMapObject  *gif_cmap;
 SavedImage       si;
-#if (GIFLIB_MAJOR == 5 && GIFLIB_MINOR >= 1) || GIFLIB_MAJOR > 5
 int              giferr;
-#endif  /* 5.1 and beyond */
 
     PROCNAME("gifToPix");
 
@@ -296,57 +333,28 @@ int              giferr;
         }
     }
 
-        /* If the image has been interlaced (for viewing in a browser),
-         * this restores the raster lines to normal order. */
-    if (gif->Image.Interlace) {
-        pixdi = pixUninterlaceGIF(pixd);
-        pixTransferAllData(pixd, &pixdi, 0, 0);
-    }
+    /* Versions before 5.0 required un-interlacing to restore
+     * the raster lines to normal order if the image
+     * had been interlaced (for viewing in a browser):
+         if (gif->Image.Interlace) {
+             PIX *pixdi = pixUninterlaceGIF(pixd);
+             pixTransferAllData(pixd, &pixdi, 0, 0);
+         }
+     * This is no longer required. */
 
     DGifCloseFile(gif, &giferr);
     return pixd;
 }
 
 
-static PIX *
-pixUninterlaceGIF(PIX  *pixs)
-{
-l_int32    w, h, d, wpl, j, k, srow, drow;
-l_uint32  *datas, *datad, *lines, *lined;
-PIX       *pixd;
-
-    PROCNAME("pixUninterlaceGIF");
-
-    if (!pixs)
-        return (PIX *)ERROR_PTR("pixs not defined", procName, NULL);
-
-    pixGetDimensions(pixs, &w, &h, &d);
-    wpl = pixGetWpl(pixs);
-    pixd = pixCreateTemplate(pixs);
-    datas = pixGetData(pixs);
-    datad = pixGetData(pixd);
-    for (k = 0, srow = 0; k < 4; k++) {
-        for (drow = InterlacedOffset[k]; drow < h;
-             drow += InterlacedJumps[k], srow++) {
-            lines = datas + srow * wpl;
-            lined = datad + drow * wpl;
-            for (j = 0; j < w; j++)
-                memcpy(lined, lines, 4 * wpl);
-        }
-    }
-
-    return pixd;
-}
-
-
 /*---------------------------------------------------------------------*
- *                         Writing gif to file                         *
+ *                            Writing gif                              *
  *---------------------------------------------------------------------*/
 /*!
  * \brief   pixWriteStreamGif()
  *
- * \param[in]    fp file stream
- * \param[in]    pix 1, 2, 4, 8, 16 or 32 bpp
+ * \param[in]  fp    file stream opened for writing
+ * \param[in]  pix   1, 2, 4, 8, 16 or 32 bpp
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -354,69 +362,136 @@ PIX       *pixd;
  *      (1) All output gif have colormaps.  If the pix is 32 bpp rgb,
  *          this quantizes the colors and writes out 8 bpp.
  *          If the pix is 16 bpp grayscale, it converts to 8 bpp first.
- *      (2) We can't write to memory using open_memstream() because
- *          the gif functions write through a file descriptor, not a
- *          file stream.
  * </pre>
  */
 l_int32
 pixWriteStreamGif(FILE  *fp,
                   PIX   *pix)
 {
-l_int32      result;
-l_int32      fd;
-GifFileType  *gif;
-#if (GIFLIB_MAJOR == 5 && GIFLIB_MINOR >= 1) || GIFLIB_MAJOR > 5
-int           giferr;
-#endif  /* 5.1 and beyond */
+l_uint8  *filedata;
+size_t    filebytes, nbytes;
 
     PROCNAME("pixWriteStreamGif");
 
-        /* See version information at top of this file */
-#if GIFLIB_MAJOR == 5 && GIFLIB_MINOR == 1 && GIFLIB_RELEASE == 2  /* 5.1.2 */
-    return ERROR_INT("Can't use giflib-5.1.2; suggest 5.1.1 or earlier\n",
-                     procName, 1);
-#endif  /* 5.1.2 */
-
     if (!fp)
         return ERROR_INT("stream not open", procName, 1);
     if (!pix)
         return ERROR_INT("pix not defined", procName, 1);
+
+    pixSetPadBits(pix, 0);
+    if (pixWriteMemGif(&filedata, &filebytes, pix) != 0) {
+        LEPT_FREE(filedata);
+        return ERROR_INT("failure to gif encode pix", procName, 1);
+    }
+
     rewind(fp);
+    nbytes = fwrite(filedata, 1, filebytes, fp);
+    LEPT_FREE(filedata);
+    if (nbytes != filebytes)
+        return ERROR_INT("write error", procName, 1);
+    return 0;
+}
 
-    if ((fd = fileno(fp)) < 0)
-        return ERROR_INT("invalid file descriptor", procName, 1);
-#ifdef _WIN32
-    fd = _dup(fd);
-#endif /* _WIN32 */
 
-        /* Get the gif file handle */
-    if ((gif = EGifOpenFileHandle(fd, NULL)) == NULL) {
+/*!
+ * \brief   pixWriteMemGif()
+ *
+ * \param[out]   pdata data of gif compressed image
+ * \param[out]   psize size of returned data
+ * \param[in]    pix
+ * \return  0 if OK, 1 on error
+ *
+ * <pre>
+ * Notes:
+ *      (1) See comments in pixReadMemGif()
+ * </pre>
+ */
+l_int32
+pixWriteMemGif(l_uint8  **pdata,
+               size_t    *psize,
+               PIX       *pix)
+{
+int           giferr;
+l_int32       result;
+GifFileType  *gif;
+L_BBUFFER    *buffer;
+
+    PROCNAME("pixWriteMemGif");
+
+        /* 5.1+ and not 5.1.2 */
+#if (GIFLIB_MAJOR < 5 || (GIFLIB_MAJOR == 5 && GIFLIB_MINOR == 0))
+    L_ERROR("Require giflib-5.1 or later\n", procName);
+    return 1;
+#endif  /* < 5.1 */
+#if GIFLIB_MAJOR == 5 && GIFLIB_MINOR == 1 && GIFLIB_RELEASE == 2  /* 5.1.2 */
+    L_ERROR("Can't use giflib-5.1.2; suggest 5.1.3 or later\n", procName);
+    return 1;
+#endif  /* 5.1.2 */
+
+    if (!pdata)
+        return ERROR_INT("&data not defined", procName, 1 );
+    *pdata = NULL;
+    if (!psize)
+        return ERROR_INT("&size not defined", procName, 1 );
+    *psize = 0;
+    if (!pix)
+        return ERROR_INT("&pix not defined", procName, 1 );
+
+    if ((buffer = bbufferCreate(NULL, 0)) == NULL)
+        return ERROR_INT("failed to create buffer", procName, 1);
+
+    if ((gif = EGifOpen((void*)buffer, gifWriteFunc, NULL)) == NULL) {
+        bbufferDestroy(&buffer);
         return ERROR_INT("failed to create GIF image handle", procName, 1);
     }
 
-    pixSetPadBits(pix, 0);
     result = pixToGif(pix, gif);
     EGifCloseFile(gif, &giferr);
+
+    if (result == 0) {
+        *pdata = bbufferDestroyAndSaveData(&buffer, psize);
+    } else {
+        bbufferDestroy(&buffer);
+    }
     return result;
 }
 
+
+static l_int32
+gifWriteFunc(GifFileType        *gif,
+             const GifByteType  *src,
+             l_int32             bytesToWrite)
+{
+L_BBUFFER  *buffer;
+
+    PROCNAME("gifWriteFunc");
+
+    if ((buffer = (L_BBUFFER*)gif->UserData) == NULL)
+        return ERROR_INT("UserData not set", procName, -1);
+
+    if(bbufferRead(buffer, (l_uint8*)src, bytesToWrite) == 0)
+        return bytesToWrite;
+    return 0;
+}
+
+
 /*!
  * \brief   pixToGif()
  *
- * \param[in]    pix 1, 2, 4, 8, 16 or 32 bpp
- * \param[in]    gif  opened gif stream
+ * \param[in]  pix    1, 2, 4, 8, 16 or 32 bpp
+ * \param[in]  gif    opened gif stream
  * \return  0 if OK, 1 on error
  *
  * <pre>
  * Notes:
  *      (1) This encodes the pix to the gif stream. The stream is not
- *          closes by this function.
+ *          closed by this function.
  *      (2) It is static to make this function private.
  * </pre>
  */
 static l_int32
-pixToGif(PIX *pix, GifFileType *gif)
+pixToGif(PIX          *pix,
+         GifFileType  *gif)
 {
 char            *text;
 l_int32          wpl, i, j, w, h, d, ncolor, rval, gval, bval;
@@ -426,9 +501,6 @@ PIX             *pixd;
 PIXCMAP         *cmap;
 ColorMapObject  *gif_cmap;
 GifByteType     *gif_line;
-#if (GIFLIB_MAJOR == 5 && GIFLIB_MINOR >= 1) || GIFLIB_MAJOR > 5
-int              giferr;
-#endif  /* 5.1 and beyond */
 
     PROCNAME("pixToGif");
 
@@ -567,195 +639,38 @@ int              giferr;
 
 
 /*---------------------------------------------------------------------*
- *                      Read/write from/to memory                      *
+ *                        Removing interlacing                         *
  *---------------------------------------------------------------------*/
-/*!
- * \brief   pixReadMemGif()
- *
- * \param[in]    data const; gif-encoded
- * \param[in]    size of data
- * \return  pix, or NULL on error
- *
- * <pre>
- * Notes:
- *   * For Giflib version >= 5.1, this uses the DGifOpen() buffer
- *     interface.  No temp files are required.
- *   * For Giflib version < 5.1:
- *       (1) Write the gif compressed data to file and read it back.
- *           Note: we can't use the GNU runtime extension fmemopen()
- *           because libgif doesn't have a file stream interface.
- *       (2) This should be relatively safe from a sophisticated attack,
- *           because we use mkstemp (or its Windows equivalent) to generate
- *           a filename and link the file.  It would be nice to go further
- *           and do this:
- *               l_int32 fd = mkstemp(template);
- *               FILE *fp = fdopen(fd, "w+b");
- *               fwrite(data, 1, size, fp);
- *               rewind(fp);
- *               Pix *pix = pixReadStreamGif(fp);
- *           but this can't be done with gif files because of the way
- *           that libgif handles the file descriptors: fp is in a
- *           bad state after writing.
- * </pre>
- */
-PIX *
-pixReadMemGif(const l_uint8  *cdata,
-              size_t          size)
+static PIX *
+pixUninterlaceGIF(PIX  *pixs)
 {
-#if (GIFLIB_MAJOR == 5 && GIFLIB_MINOR >= 1) || GIFLIB_MAJOR > 5
-GifFileType   *gif;
-GifReadBuffer  buffer;
-#else
-char          *fname;
-PIX           *pix;
-#endif  /* 5.1 and beyond */
+l_int32    w, h, d, wpl, j, k, srow, drow;
+l_uint32  *datas, *datad, *lines, *lined;
+PIX       *pixd;
 
-    PROCNAME("pixReadMemGif");
+    PROCNAME("pixUninterlaceGIF");
 
-    if (!cdata)
-        return (PIX *)ERROR_PTR("cdata not defined", procName, NULL);
+    if (!pixs)
+        return (PIX *)ERROR_PTR("pixs not defined", procName, NULL);
 
-#if (GIFLIB_MAJOR == 5 && GIFLIB_MINOR >= 1) || GIFLIB_MAJOR > 5
-    buffer.cdata = cdata;
-    buffer.size = size;
-    buffer.pos = 0;
-    if ((gif = DGifOpen((void*)&buffer, gifReadFunc, NULL)) == NULL)
-        return (PIX *)ERROR_PTR("could not open gif stream from memory",
-                                procName, NULL);
-
-    return gifToPix(gif);
-#else
-    L_INFO("using a temp file; not reading from memory\n", procName); 
-        /* Write to a temp file */
-    fname = l_makeTempFilename();
-    l_binaryWrite(fname, "w", (l_uint8 *)cdata, size);
-
-        /* Read back from the file */
-    pix = pixRead(fname);
-    lept_rmfile(fname);
-    LEPT_FREE(fname);
-    if (!pix) L_ERROR("pix not read\n", procName);
-    return pix;
-#endif  /* 5.1 and beyond */
-}
-
-
-#if (GIFLIB_MAJOR == 5 && GIFLIB_MINOR >= 1) || GIFLIB_MAJOR > 5
-static l_int32
-gifReadFunc(GifFileType *gif, GifByteType *dest, l_int32 bytesToRead) 
-{
-GifReadBuffer  *buffer;
-l_int32         bytesRead;
-
-    PROCNAME("gifReadFunc");
-
-    if ((buffer = (GifReadBuffer*)gif->UserData) == NULL)
-        return ERROR_INT("UserData not set", procName, -1);
-
-    if(buffer->pos >= buffer->size)
-        return -1;
-
-    bytesRead = (buffer->pos < buffer->size - bytesToRead)
-              ? bytesToRead : buffer->size - buffer->pos;
-    memcpy(dest, buffer->cdata + buffer->pos, bytesRead);
-    buffer->pos += bytesRead;
-    return bytesRead;
-}
-#endif  /* 5.1 and beyond */
-
-
-/*!
- * \brief   pixWriteMemGif()
- *
- * \param[out]   pdata data of gif compressed image
- * \param[out]   psize size of returned data
- * \param[in]    pix
- * \return  0 if OK, 1 on error
- *
- * <pre>
- * Notes:
- *      (1) See comments in pixReadMemGif()
- *      (2) For Giflib version >= 5.1, this uses the EGifOpen() buffer
- *          interface.  No temp files are required.
- * </pre>
- */
-l_int32
-pixWriteMemGif(l_uint8  **pdata,
-               size_t    *psize,
-               PIX       *pix)
-{
-#if (GIFLIB_MAJOR == 5 && GIFLIB_MINOR >= 1) || GIFLIB_MAJOR > 5
-int           giferr;
-l_int32       result;
-GifFileType  *gif;
-L_BBUFFER     *buffer;
-#else
-char         *fname;
-#endif  /* 5.1 and beyond */
-
-    PROCNAME("pixWriteMemGif");
-
-    if (!pdata)
-        return ERROR_INT("&data not defined", procName, 1 );
-    *pdata = NULL;
-    if (!psize)
-        return ERROR_INT("&size not defined", procName, 1 );
-    *psize = 0;
-    if (!pix)
-        return ERROR_INT("&pix not defined", procName, 1 );
-
-#if (GIFLIB_MAJOR == 5 && GIFLIB_MINOR >= 1) || GIFLIB_MAJOR > 5
-    if((buffer = bbufferCreate(NULL, 0)) == NULL) {
-        return ERROR_INT("failed to create buffer", procName, 1);
+    pixGetDimensions(pixs, &w, &h, &d);
+    wpl = pixGetWpl(pixs);
+    pixd = pixCreateTemplate(pixs);
+    datas = pixGetData(pixs);
+    datad = pixGetData(pixd);
+    for (k = 0, srow = 0; k < 4; k++) {
+        for (drow = InterlacedOffset[k]; drow < h;
+             drow += InterlacedJumps[k], srow++) {
+            lines = datas + srow * wpl;
+            lined = datad + drow * wpl;
+            for (j = 0; j < w; j++)
+                memcpy(lined, lines, 4 * wpl);
+        }
     }
 
-    if ((gif = EGifOpen((void*)buffer, gifWriteFunc, NULL)) == NULL) {
-        bbufferDestroy(&buffer);
-        return ERROR_INT("failed to create GIF image handle", procName, 1);
-    }
-
-    result = pixToGif(pix, gif);
-    EGifCloseFile(gif, &giferr);
-
-    if(result == 0) {
-        *pdata = bbufferDestroyAndSaveData(&buffer, psize);
-    } else {
-        bbufferDestroy(&buffer);
-    }
-    return result;
-#else
-    L_INFO("writing to a temp file, not directly to memory\n", procName);
-
-        /* Write to a temp file */
-    fname = l_makeTempFilename();
-    pixWrite(fname, pix, IFF_GIF);
-
-        /* Read back into memory */
-    *pdata = l_binaryRead(fname, psize);
-    lept_rmfile(fname);
-    LEPT_FREE(fname);
-    return 0;
-#endif
+    return pixd;
 }
 
 
-#if (GIFLIB_MAJOR == 5 && GIFLIB_MINOR >= 1) || GIFLIB_MAJOR > 5
-static l_int32
-gifWriteFunc(GifFileType *gif, const GifByteType *src, l_int32 bytesToWrite) 
-{
-L_BBUFFER  *buffer;
-
-    PROCNAME("gifWriteFunc");
-
-    if ((buffer = (L_BBUFFER*)gif->UserData) == NULL)
-        return ERROR_INT("UserData not set", procName, -1);
-
-    if(bbufferRead(buffer, (l_uint8*)src, bytesToWrite) == 0)
-        return bytesToWrite;
-    return 0;
-}
-#endif  /* 5.1 and beyond */
-
-
 /* -----------------------------------------------------------------*/
 #endif    /* HAVE_LIBGIF || HAVE_LIBUNGIF  */
