@@ -107,11 +107,6 @@ const char  *gplotstylenames[] = {"with lines",
                                   "with impulses",
                                   "with linespoints",
                                   "with dots"};
-const char  *gplotfilestyles[] = {"LINES",
-                                  "POINTS",
-                                  "IMPULSES",
-                                  "LINESPOINTS",
-                                  "DOTS"};
 const char  *gplotfileoutputs[] = {"",
                                    "PNG",
                                    "PS",
@@ -146,9 +141,10 @@ gplotCreate(const char  *rootname,
             const char  *xlabel,
             const char  *ylabel)
 {
-char   *newroot;
-char    buf[L_BUF_SIZE];
-GPLOT  *gplot;
+char    *newroot;
+char     buf[L_BUF_SIZE];
+l_int32  badchar;
+GPLOT   *gplot;
 
     PROCNAME("gplotCreate");
 
@@ -157,6 +153,9 @@ GPLOT  *gplot;
     if (outformat != GPLOT_PNG && outformat != GPLOT_PS &&
         outformat != GPLOT_EPS && outformat != GPLOT_LATEX)
         return (GPLOT *)ERROR_PTR("outformat invalid", procName, NULL);
+    stringCheckForChars(rootname, "`;&|><\"?*", &badchar);
+    if (badchar)  /* danger of command injection */
+        return (GPLOT *)ERROR_PTR("invalid rootname", procName, NULL);
 
     if ((gplot = (GPLOT *)LEPT_CALLOC(1, sizeof(GPLOT))) == NULL)
         return (GPLOT *)ERROR_PTR("gplot not made", procName, NULL);
@@ -254,7 +253,8 @@ GPLOT  *gplot;
  *               x = startx + i * delx
  *          These are set with numaSetParameters().  Their default
  *          values are startx = 0.0, delx = 1.0.
- *      (3) If nax is defined, it must be the same size as nay.
+ *      (3) If nax is defined, it must be the same size as nay, and
+ *          must have at least one number.
  *      (4) The 'plottitle' string can have spaces, double
  *          quotes and backquotes, but not single quotes.
  * </pre>
@@ -279,19 +279,20 @@ SARRAY    *sa;
         return ERROR_INT("gplot not defined", procName, 1);
     if (!nay)
         return ERROR_INT("nay not defined", procName, 1);
-    if (plotstyle != GPLOT_LINES && plotstyle != GPLOT_POINTS &&
-        plotstyle != GPLOT_IMPULSES && plotstyle != GPLOT_LINESPOINTS &&
-        plotstyle != GPLOT_DOTS)
+    if (plotstyle < 0 || plotstyle >= NUM_GPLOT_STYLES)
         return ERROR_INT("invalid plotstyle", procName, 1);
 
-    n = numaGetCount(nay);
-    numaGetParameters(nay, &startx, &delx);
-    if (nax) {
-        if (n != numaGetCount(nax))
-            return ERROR_INT("nax and nay sizes differ", procName, 1);
+    if ((n = numaGetCount(nay)) == 0)
+        return ERROR_INT("no points to plot", procName, 1);
+    if (nax && (n != numaGetCount(nax)))
+        return ERROR_INT("nax and nay sizes differ", procName, 1);
+    if (n == 1 && plotstyle == GPLOT_LINES) {
+        L_INFO("only 1 pt; changing style to points\n", procName);
+        plotstyle = GPLOT_POINTS;
     }
 
         /* Save plotstyle and plottitle */
+    numaGetParameters(nay, &startx, &delx);
     numaAddNumber(gplot->plotstyles, plotstyle);
     if (plottitle) {
         title = stringNew(plottitle);
@@ -396,7 +397,10 @@ l_int32  ignore;
     snprintf(buf, L_BUF_SIZE, "wgnuplot %s", cmdname);
 #endif  /* _WIN32 */
 
+#ifndef OS_IOS /* iOS 11 does not support system() */
     ignore = system(buf);  /* gnuplot || wgnuplot */
+#endif /* !OS_IOS */
+
     LEPT_FREE(cmdname);
     return 0;
 }
@@ -668,9 +672,7 @@ GPLOT  *gplot;
 
     if (!nay)
         return ERROR_INT("nay not defined", procName, 1);
-    if (plotstyle != GPLOT_LINES && plotstyle != GPLOT_POINTS &&
-        plotstyle != GPLOT_IMPULSES && plotstyle != GPLOT_LINESPOINTS &&
-        plotstyle != GPLOT_DOTS)
+    if (plotstyle < 0 || plotstyle >= NUM_GPLOT_STYLES)
         return ERROR_INT("invalid plotstyle", procName, 1);
     if (outformat != GPLOT_PNG && outformat != GPLOT_PS &&
         outformat != GPLOT_EPS && outformat != GPLOT_LATEX)
@@ -726,9 +728,7 @@ GPLOT  *gplot;
 
     if (!nay1 || !nay2)
         return ERROR_INT("nay1 and nay2 not both defined", procName, 1);
-    if (plotstyle != GPLOT_LINES && plotstyle != GPLOT_POINTS &&
-        plotstyle != GPLOT_IMPULSES && plotstyle != GPLOT_LINESPOINTS &&
-        plotstyle != GPLOT_DOTS)
+    if (plotstyle < 0 || plotstyle >= NUM_GPLOT_STYLES)
         return ERROR_INT("invalid plotstyle", procName, 1);
     if (outformat != GPLOT_PNG && outformat != GPLOT_PS &&
         outformat != GPLOT_EPS && outformat != GPLOT_LATEX)
@@ -787,9 +787,7 @@ NUMA    *nay;
         return ERROR_INT("naay not defined", procName, 1);
     if ((n = numaaGetCount(naay)) == 0)
         return ERROR_INT("no numa in array", procName, 1);
-    if (plotstyle != GPLOT_LINES && plotstyle != GPLOT_POINTS &&
-        plotstyle != GPLOT_IMPULSES && plotstyle != GPLOT_LINESPOINTS &&
-        plotstyle != GPLOT_DOTS)
+    if (plotstyle < 0 || plotstyle >= NUM_GPLOT_STYLES)
         return ERROR_INT("invalid plotstyle", procName, 1);
     if (outformat != GPLOT_PNG && outformat != GPLOT_PS &&
         outformat != GPLOT_EPS && outformat != GPLOT_LATEX)
@@ -859,14 +857,15 @@ GPLOT   *gplot;
     ylabel = stringNew(buf + 14);
     ylabel[strlen(ylabel) - 1] = '\0';
 
-    if (!(gplot = gplotCreate(rootname, outformat, title, xlabel, ylabel))) {
-        fclose(fp);
-        return (GPLOT *)ERROR_PTR("gplot not made", procName, NULL);
-    }
+    gplot = gplotCreate(rootname, outformat, title, xlabel, ylabel);
     LEPT_FREE(rootname);
     LEPT_FREE(title);
     LEPT_FREE(xlabel);
     LEPT_FREE(ylabel);
+    if (!gplot) {
+        fclose(fp);
+        return (GPLOT *)ERROR_PTR("gplot not made", procName, NULL);
+    }
     sarrayDestroy(&gplot->cmddata);
     sarrayDestroy(&gplot->datanames);
     sarrayDestroy(&gplot->plotdata);
