@@ -480,7 +480,7 @@ PIXCMAP   *cmap;
  *          for this expansion, it returns 1 (treated as a warning);
  *          the caller should check the return value.
  *      (3) This can be used to determine if the new colors will fit in
- *          the cmap, using null for \&na.  Returns 0 if they fit; 2 if
+ *          the cmap, using null for &na.  Returns 0 if they fit; 2 if
  *          they don't fit.
  *      (4) The mapping table contains, for each gray color found, the
  *          index of the corresponding colorized pixel.  Non-gray
@@ -622,8 +622,7 @@ PIXCMAP   *cmap;
 
         /* replace pixel value sindex by index when fg pixel in pixmc
          * overlays it */
-    w = pixGetWidth(pixs);
-    h = pixGetHeight(pixs);
+    pixGetDimensions(pixs, &w, &h, NULL);
     datas = pixGetData(pixs);
     wpls = pixGetWpl(pixs);
     wm = pixGetWidth(pixm);
@@ -638,15 +637,6 @@ PIXCMAP   *cmap;
             if (j + x < 0  || j + x >= w) continue;
             if (GET_DATA_BIT(linem, j)) {
                 switch (d) {
-                case 1:
-                    val = GET_DATA_BIT(lines, x + j);
-                    if (val == sindex) {
-                        if (index == 0)
-                            CLEAR_DATA_BIT(lines, x + j);
-                        else
-                            SET_DATA_BIT(lines, x + j);
-                    }
-                    break;
                 case 2:
                     val = GET_DATA_DIBIT(lines, x + j);
                     if (val == sindex)
@@ -749,8 +739,7 @@ PIXCMAP   *cmap;
         for (j = 0; j < wm; j++) {
             if (j + x < 0  || j + x >= w) continue;
             if (GET_DATA_BIT(linem, j)) {  /* paint color */
-                switch (d)
-                {
+                switch (d) {
                 case 2:
                     SET_DATA_DIBIT(line, j + x, index);
                     break;
