@@ -89,14 +89,16 @@
  *         Flags for 8 and 16 bit pixel sums
  *         Dithering flags
  *         Distance flags
+ *         Value flags
  *         Statistical measures
  *         Set selection flags
  *         Text orientation flags
  *         Edge orientation flags
  *         Line orientation flags
+ *         Image orientation flags
  *         Scan direction flags
  *         Box size adjustment flags
- *         Flags for selecting box boundaries from two choices
+ *         Flags for modifying box boundaries using a second box
  *         Handling overlapping bounding boxes in boxa
  *         Flags for replacing invalid boxes
  *         Horizontal warp
@@ -366,7 +368,7 @@ enum {
  *       (4) We use 32 bit pixels for both RGB and RGBA color images.
  *           The A (alpha) byte is ignored in most leptonica functions
  *           operating on color images.  Within each 4 byte pixel, the
- *           colors are ordered from MSB to LSB, as follows:
+ *           color samples are ordered from MSB to LSB, as follows:
  *
  *                |  MSB  |  2nd MSB  |  3rd MSB  |  LSB  |
  *                   red      green       blue      alpha
@@ -395,18 +397,19 @@ enum {
  *                 SET_DATA_BYTE(&pixel, COLOR_BLUE, blueval);
  *                 SET_DATA_BYTE(&pixel, L_ALPHA_CHANNEL, alphaval);
  *
- *           For extra speed we extract these components directly
+ *           More efficiently, these components can be extracted directly
  *           by shifting and masking, explicitly using the values in
  *           L_RED_SHIFT, etc.:
  *                 (pixel32 >> L_RED_SHIFT) & 0xff;         (red)
  *                 (pixel32 >> L_GREEN_SHIFT) & 0xff;       (green)
  *                 (pixel32 >> L_BLUE_SHIFT) & 0xff;        (blue)
  *                 (pixel32 >> L_ALPHA_SHIFT) & 0xff;       (alpha)
- *           All these operations work properly on both big- and little-endians.
+ *           The functions extractRGBValues() and extractRGBAValues() are
+ *           provided to do this.  Likewise, the pixels can be set
+ *           directly by shifting, using composeRGBPixel() and
+ *           composeRGBAPixel().
  *
- *           For a few situations, these color shift values are hard-coded.
- *           Changing the RGB color component ordering through the assignments
- *           in this file will cause functions marked with "***" to fail.
+ *           All these operations work properly on both big- and little-endians.
  *
  *       (5) A reference count is held within each pix, giving the
  *           number of ptrs to the pix.  When a pixClone() call
@@ -797,7 +800,7 @@ enum {
 
 
 /*-------------------------------------------------------------------------*
- *                     Color component selection flags                     *
+ *                    Color component selection flags                      *
  *-------------------------------------------------------------------------*/
 
 /*! Color component selection flags */
@@ -807,7 +810,9 @@ enum {
     L_SELECT_BLUE = 3,          /*!< use blue component                    */
     L_SELECT_MIN = 4,           /*!< use min color component               */
     L_SELECT_MAX = 5,           /*!< use max color component               */
-    L_SELECT_AVERAGE = 6        /*!< use average of color components       */
+    L_SELECT_AVERAGE = 6,       /*!< use average of color components       */
+    L_SELECT_HUE = 7,           /*!< use hue value (in HSV color space)    */
+    L_SELECT_SATURATION = 8     /*!< use saturation value (in HSV space)   */
 };
 
 
@@ -819,10 +824,11 @@ enum {
 enum {
     L_LS_BYTE = 1,              /*!< use LSB                               */
     L_MS_BYTE = 2,              /*!< use MSB                               */
-    L_CLIP_TO_FF = 3,           /*!< use max(val, 255)                     */
-    L_LS_TWO_BYTES = 4,         /*!< use two LSB                           */
-    L_MS_TWO_BYTES = 5,         /*!< use two MSB                           */
-    L_CLIP_TO_FFFF = 6          /*!< use max(val, 65535)                   */
+    L_AUTO_BYTE = 3,            /*!< use LSB if max(val) < 256; else MSB   */
+    L_CLIP_TO_FF = 4,           /*!< use max(val, 255)                     */
+    L_LS_TWO_BYTES = 5,         /*!< use two LSB                           */
+    L_MS_TWO_BYTES = 6,         /*!< use two MSB                           */
+    L_CLIP_TO_FFFF = 7          /*!< use max(val, 65535)                   */
 };
 
 
@@ -935,6 +941,21 @@ enum {
 };
 
 
+/*-------------------------------------------------------------------------*
+ *                               Value flags                               *
+ *-------------------------------------------------------------------------*/
+
+/*! Value flags */
+enum {
+    L_NEGATIVE = 1,      /*!< values < 0                                   */
+    L_NON_NEGATIVE = 2,  /*!< values >= 0                                  */
+    L_POSITIVE = 3,      /*!< values > 0                                   */
+    L_NON_POSITIVE = 4,  /*!< values <= 0                                  */
+    L_ZERO = 5,          /*!< values = 0                                   */
+    L_ALL = 6            /*!< all values                                   */
+};
+
+
 /*-------------------------------------------------------------------------*
  *                         Statistical measures                            *
  *-------------------------------------------------------------------------*/
@@ -1002,6 +1023,17 @@ enum {
 };
 
 
+/*-------------------------------------------------------------------------*
+ *                         Image orientation flags                         *
+ *-------------------------------------------------------------------------*/
+
+/*! Image orientation flags */
+enum {
+    L_PORTRAIT_MODE = 0,   /*!< typical: page is viewed with height > width  */
+    L_LANDSCAPE_MODE = 1   /*!< page is viewed at 90 deg to portrait mode    */
+};
+
+
 /*-------------------------------------------------------------------------*
  *                           Scan direction flags                          *
  *-------------------------------------------------------------------------*/
@@ -1047,16 +1079,17 @@ enum {
 
 
 /*-------------------------------------------------------------------------*
- *          Flags for selecting box boundaries from two choices            *
+ *          Flags for modifying box boundaries using a second box          *
  *-------------------------------------------------------------------------*/
 
-/*! Flags for selecting box boundaries from two choices */
+/*! Flags for modifying box boundaries using a second box */
 enum {
     L_USE_MINSIZE = 1,           /*!< use boundaries giving min size       */
     L_USE_MAXSIZE = 2,           /*!< use boundaries giving max size       */
-    L_SUB_ON_BIG_DIFF = 3,       /*!< substitute boundary if big abs diff  */
-    L_USE_CAPPED_MIN = 4,        /*!< substitute boundary with capped min  */
-    L_USE_CAPPED_MAX = 5         /*!< substitute boundary with capped max  */
+    L_SUB_ON_LOC_DIFF = 3,       /*!< modify boundary if big location diff */
+    L_SUB_ON_SIZE_DIFF = 4,      /*!< modify boundary if big size diff     */
+    L_USE_CAPPED_MIN = 5,        /*!< modify boundary with capped min      */
+    L_USE_CAPPED_MAX = 6         /*!< modify boundary with capped max      */
 };
 
 /*-------------------------------------------------------------------------*
