@@ -54,6 +54,7 @@
  *           l_int32   pixaSetBoxa()
  *           PIX     **pixaGetPixArray()
  *           l_int32   pixaVerifyDepth()
+ *           l_int32   pixaVerifyDimensions()
  *           l_int32   pixaIsFull()
  *           l_int32   pixaCountText()
  *           l_int32   pixaSetText()
@@ -92,6 +93,7 @@
  *           BOXA     *pixaaGetBoxa()
  *           PIX      *pixaaGetPix()
  *           l_int32   pixaaVerifyDepth()
+ *           l_int32   pixaaVerifyDimensions()
  *           l_int32   pixaaIsFull()
  *
  *      Pixaa array modifiers
@@ -165,17 +167,16 @@ PIXA  *pixa;
     if (n <= 0)
         n = INITIAL_PTR_ARRAYSIZE;
 
-    if ((pixa = (PIXA *)LEPT_CALLOC(1, sizeof(PIXA))) == NULL)
-        return (PIXA *)ERROR_PTR("pixa not made", procName, NULL);
+    pixa = (PIXA *)LEPT_CALLOC(1, sizeof(PIXA));
     pixa->n = 0;
     pixa->nalloc = n;
     pixa->refcount = 1;
-
-    if ((pixa->pix = (PIX **)LEPT_CALLOC(n, sizeof(PIX *))) == NULL)
-        return (PIXA *)ERROR_PTR("pix ptrs not made", procName, NULL);
-    if ((pixa->boxa = boxaCreate(n)) == NULL)
-        return (PIXA *)ERROR_PTR("boxa not made", procName, NULL);
-
+    pixa->pix = (PIX **)LEPT_CALLOC(n, sizeof(PIX *));
+    pixa->boxa = boxaCreate(n);
+    if (!pixa->pix || !pixa->boxa) {
+        pixaDestroy(&pixa);
+        return (PIXA *)ERROR_PTR("pix or boxa not made", procName, NULL);
+    }
     return pixa;
 }
 
@@ -202,7 +203,7 @@ pixaCreateFromPix(PIX     *pixs,
                   l_int32  cellh)
 {
 l_int32  w, h, d, nw, nh, i, j, index;
-PIX     *pix, *pixt;
+PIX     *pix1, *pix2;
 PIXA    *pixa;
 
     PROCNAME("pixaCreateFromPix");
@@ -215,23 +216,25 @@ PIXA    *pixa;
     if ((pixa = pixaCreate(n)) == NULL)
         return (PIXA *)ERROR_PTR("pixa not made", procName, NULL);
     pixGetDimensions(pixs, &w, &h, &d);
-    if ((pixt = pixCreate(cellw, cellh, d)) == NULL)
-        return (PIXA *)ERROR_PTR("pixt not made", procName, NULL);
+    if ((pix1 = pixCreate(cellw, cellh, d)) == NULL) {
+        pixaDestroy(&pixa);
+        return (PIXA *)ERROR_PTR("pix1 not made", procName, NULL);
+    }
 
     nw = (w + cellw - 1) / cellw;
     nh = (h + cellh - 1) / cellh;
     for (i = 0, index = 0; i < nh; i++) {
         for (j = 0; j < nw && index < n; j++, index++) {
-            pixRasterop(pixt, 0, 0, cellw, cellh, PIX_SRC, pixs,
+            pixRasterop(pix1, 0, 0, cellw, cellh, PIX_SRC, pixs,
                    j * cellw, i * cellh);
-            if (d == 1 && !pixClipToForeground(pixt, &pix, NULL))
-                pixaAddPix(pixa, pix, L_INSERT);
+            if (d == 1 && !pixClipToForeground(pix1, &pix2, NULL))
+                pixaAddPix(pixa, pix2, L_INSERT);
             else
-                pixaAddPix(pixa, pixt, L_COPY);
+                pixaAddPix(pixa, pix1, L_COPY);
         }
     }
 
-    pixDestroy(&pixt);
+    pixDestroy(&pix1);
     return pixa;
 }
 
@@ -337,7 +340,7 @@ pixaSplitPix(PIX      *pixs,
              l_uint32  bordercolor)
 {
 l_int32  w, h, d, cellw, cellh, i, j;
-PIX     *pixt;
+PIX     *pix1;
 PIXA    *pixa;
 
     PROCNAME("pixaSplitPix");
@@ -356,21 +359,23 @@ PIXA    *pixa;
 
     for (i = 0; i < ny; i++) {
         for (j = 0; j < nx; j++) {
-            if ((pixt = pixCreate(cellw + 2 * borderwidth,
-                                  cellh + 2 * borderwidth, d)) == NULL)
-                return (PIXA *)ERROR_PTR("pixt not made", procName, NULL);
-            pixCopyColormap(pixt, pixs);
+            if ((pix1 = pixCreate(cellw + 2 * borderwidth,
+                                  cellh + 2 * borderwidth, d)) == NULL) {
+                pixaDestroy(&pixa);
+                return (PIXA *)ERROR_PTR("pix1 not made", procName, NULL);
+            }
+            pixCopyColormap(pix1, pixs);
             if (borderwidth == 0) {  /* initialize full image to white */
                 if (d == 1)
-                    pixClearAll(pixt);
+                    pixClearAll(pix1);
                 else
-                    pixSetAll(pixt);
+                    pixSetAll(pix1);
             } else {
-                pixSetAllArbitrary(pixt, bordercolor);
+                pixSetAllArbitrary(pix1, bordercolor);
             }
-            pixRasterop(pixt, borderwidth, borderwidth, cellw, cellh,
+            pixRasterop(pix1, borderwidth, borderwidth, cellw, cellh,
                         PIX_SRC, pixs, j * cellw, i * cellh);
-            pixaAddPix(pixa, pixt, L_INSERT);
+            pixaAddPix(pixa, pix1, L_INSERT);
         }
     }
 
@@ -911,8 +916,9 @@ pixaGetPixArray(PIXA  *pixa)
  * \brief   pixaVerifyDepth()
  *
  * \param[in]    pixa
- * \param[out]   pmaxdepth [optional] max depth of all pix
- * \return  depth return 0 if they're not all the same, or on error
+ * \param[out]   psame   1 if depth is the same for all pix; 0 otherwise
+ * \param[out]   pmaxd   [optional] max depth of all pix
+ * \return  0 if OK, 1 on error
  *
  * <pre>
  * Notes:
@@ -921,31 +927,84 @@ pixaGetPixArray(PIXA  *pixa)
  */
 l_int32
 pixaVerifyDepth(PIXA     *pixa,
-                l_int32  *pmaxdepth)
+                l_int32  *psame,
+                l_int32  *pmaxd)
 {
-l_int32  i, n, d, depth, maxdepth, same;
+l_int32  i, n, d, maxd, same;
 
     PROCNAME("pixaVerifyDepth");
 
-    if (pmaxdepth) *pmaxdepth = 0;
+    if (pmaxd) *pmaxd = 0;
+    if (!psame)
+        return ERROR_INT("psame not defined", procName, 1);
     if (!pixa)
-        return ERROR_INT("pixa not defined", procName, 0);
+        return ERROR_INT("pixa not defined", procName, 1);
+    if ((n = pixaGetCount(pixa)) == 0)
+        return ERROR_INT("no pix in pixa", procName, 1);
 
-    depth = 0;
-    n = pixaGetCount(pixa);
-    maxdepth = 0;
     same = 1;
-    for (i = 0; i < n; i++) {
+    pixaGetPixDimensions(pixa, 0, NULL, NULL, &maxd);
+    for (i = 1; i < n; i++) {
         if (pixaGetPixDimensions(pixa, i, NULL, NULL, &d))
-            return ERROR_INT("pix depth not found", procName, 0);
-        maxdepth = L_MAX(maxdepth, d);
-        if (i == 0)
-            depth = d;
-        else if (d != depth)
+            return ERROR_INT("pix depth not found", procName, 1);
+        maxd = L_MAX(maxd, d);
+        if (d != maxd)
             same = 0;
     }
-    if (pmaxdepth) *pmaxdepth = maxdepth;
-    return (same == 1) ? depth : 0;
+    *psame = same;
+    if (pmaxd) *pmaxd = maxd;
+    return 0;
+}
+
+
+/*!
+ * \brief   pixaVerifyDimensions()
+ *
+ * \param[in]    pixa
+ * \param[out]   psame   1 if dimensions are the same for all pix; 0 otherwise
+ * \param[out]   pmaxw   [optional] max width of all pix
+ * \param[out]   pmaxh   [optional] max height of all pix
+ * \return  0 if OK, 1 on error
+ *
+ * <pre>
+ * Notes:
+ *      (1) It is considered to be an error if there are no pix.
+ * </pre>
+ */
+l_int32
+pixaVerifyDimensions(PIXA     *pixa,
+                     l_int32  *psame,
+                     l_int32  *pmaxw,
+                     l_int32  *pmaxh)
+{
+l_int32  i, n, w, h, maxw, maxh, same;
+
+    PROCNAME("pixaVerifyDimensions");
+
+    if (pmaxw) *pmaxw = 0;
+    if (pmaxh) *pmaxh = 0;
+    if (!psame)
+        return ERROR_INT("psame not defined", procName, 1);
+    *psame = 0;
+    if (!pixa)
+        return ERROR_INT("pixa not defined", procName, 1);
+    if ((n = pixaGetCount(pixa)) == 0)
+        return ERROR_INT("no pix in pixa", procName, 1);
+
+    same = 1;
+    pixaGetPixDimensions(pixa, 0, &maxw, &maxh, NULL);
+    for (i = 1; i < n; i++) {
+        if (pixaGetPixDimensions(pixa, i, &w, &h, NULL))
+            return ERROR_INT("pix dimensions not found", procName, 1);
+        maxw = L_MAX(maxw, w);
+        maxh = L_MAX(maxh, h);
+        if (w != maxw || h != maxh)
+            same = 0;
+    }
+    *psame = same;
+    if (pmaxw) *pmaxw = maxw;
+    if (pmaxh) *pmaxh = maxh;
+    return 0;
 }
 
 
@@ -1118,7 +1177,7 @@ void ***
 pixaGetLinePtrs(PIXA     *pixa,
                 l_int32  *psize)
 {
-l_int32  i, n;
+l_int32  i, n, same;
 void   **lineptrs;
 void  ***lineset;
 PIX     *pix;
@@ -1128,7 +1187,8 @@ PIX     *pix;
     if (psize) *psize = 0;
     if (!pixa)
         return (void ***)ERROR_PTR("pixa not defined", procName, NULL);
-    if (pixaVerifyDepth(pixa, NULL) == 0)
+    pixaVerifyDepth(pixa, &same, NULL);
+    if (!same)
         return (void ***)ERROR_PTR("pixa not all same depth", procName, NULL);
     n = pixaGetCount(pixa);
     if (psize) *psize = n;
@@ -1452,7 +1512,7 @@ pixaInitFull(PIXA  *pixa,
              BOX   *box)
 {
 l_int32  i, n;
-PIX     *pixt;
+PIX     *pix1;
 
     PROCNAME("pixaInitFull");
 
@@ -1463,10 +1523,10 @@ PIX     *pixt;
     pixa->n = n;
     for (i = 0; i < n; i++) {
         if (pix)
-            pixt = pixCopy(NULL, pix);
+            pix1 = pixCopy(NULL, pix);
         else
-            pixt = pixCreate(1, 1, 1);
-        pixaReplacePix(pixa, i, pixt, NULL);
+            pix1 = pixCreate(1, 1, 1);
+        pixaReplacePix(pixa, i, pix1, NULL);
     }
     if (box)
         boxaInitFull(pixa->boxa, box);
@@ -2134,36 +2194,104 @@ PIXA  *pixa;
  * \brief   pixaaVerifyDepth()
  *
  * \param[in]    paa
- * \param[out]   pmaxdepth [optional] max depth of all pix in pixaa
- * \return  depth return 0 if they're not all the same, or on error
+ * \param[out]   psame   1 if all pix have the same depth; 0 otherwise
+ * \param[out]   pmaxd   [optional] max depth of all pix in pixaa
+ * \return   0 if OK; 1 on error
+ *
+ * <pre>
+ * Notes:
+ *      (1) It is considered to be an error if any pixa have no pix.
+ * </pre>
  */
 l_int32
 pixaaVerifyDepth(PIXAA    *paa,
-                 l_int32  *pmaxdepth)
+                 l_int32  *psame,
+                 l_int32  *pmaxd)
 {
-l_int32  i, npixa, d, maxd, maxdepth, same;
+l_int32  i, n, d, maxd, same, samed;
 PIXA    *pixa;
 
     PROCNAME("pixaaVerifyDepth");
 
-    if (pmaxdepth) *pmaxdepth = 0;
+    if (pmaxd) *pmaxd = 0;
+    if (!psame)
+        return ERROR_INT("psame not defined", procName, 1);
+    *psame = 0;
     if (!paa)
-        return ERROR_INT("paa not defined", procName, 0);
+        return ERROR_INT("paa not defined", procName, 1);
+    if ((n = pixaaGetCount(paa, NULL)) == 0)
+        return ERROR_INT("no pixa in paa", procName, 1);
 
-    npixa = pixaaGetCount(paa, NULL);
-    maxdepth = 0;
-    same = 1;
-    for (i = 0; i < npixa; i++) {
+    pixa = pixaaGetPixa(paa, 0, L_CLONE);
+    pixaVerifyDepth(pixa, &same, &maxd);  /* init same, maxd with first pixa */
+    pixaDestroy(&pixa);
+    for (i = 1; i < n; i++) {
         pixa = pixaaGetPixa(paa, i, L_CLONE);
-        if (pixaGetCount(pixa) > 0) {
-            d = pixaVerifyDepth(pixa, &maxd);
-            maxdepth = L_MAX(maxdepth, maxd);  /* biggest up to this point */
-            if (d != maxdepth) same = 0;
-        }
+        pixaVerifyDepth(pixa, &samed, &d);
         pixaDestroy(&pixa);
+        maxd = L_MAX(maxd, d);
+        if (!samed || maxd != d)
+            same = 0;
     }
-    if (pmaxdepth) *pmaxdepth = maxdepth;
-    return (same == 1) ? maxdepth : 0;
+    *psame = same;
+    if (pmaxd) *pmaxd = maxd;
+    return 0;
+}
+
+
+/*!
+ * \brief   pixaaVerifyDimensions()
+ *
+ * \param[in]    paa
+ * \param[out]   psame   1 if all pix have the same depth; 0 otherwise
+ * \param[out]   pmaxw   [optional] max width of all pix in pixaa
+ * \param[out]   pmaxh   [optional] max height of all pix in pixaa
+ * \return   0 if OK; 1 on error
+ *
+ * <pre>
+ * Notes:
+ *      (1) It is considered to be an error if any pixa have no pix.
+ * </pre>
+ */
+l_int32
+pixaaVerifyDimensions(PIXAA    *paa,
+                      l_int32  *psame,
+                      l_int32  *pmaxw,
+                      l_int32  *pmaxh)
+{
+l_int32  i, n, w, h, maxw, maxh, same, same2;
+PIXA    *pixa;
+
+    PROCNAME("pixaaVerifyDimensions");
+
+    if (pmaxw) *pmaxw = 0;
+    if (pmaxh) *pmaxh = 0;
+    if (!psame)
+        return ERROR_INT("psame not defined", procName, 1);
+    *psame = 0;
+    if (!paa)
+        return ERROR_INT("paa not defined", procName, 1);
+    if ((n = pixaaGetCount(paa, NULL)) == 0)
+        return ERROR_INT("no pixa in paa", procName, 1);
+
+        /* Init same; init maxw and maxh from first pixa */
+    pixa = pixaaGetPixa(paa, 0, L_CLONE);
+    pixaVerifyDimensions(pixa, &same, &maxw, &maxh);
+    pixaDestroy(&pixa);
+
+    for (i = 1; i < n; i++) {
+        pixa = pixaaGetPixa(paa, i, L_CLONE);
+        pixaVerifyDimensions(pixa, &same2, &w, &h);
+        pixaDestroy(&pixa);
+        maxw = L_MAX(maxw, w);
+        maxh = L_MAX(maxh, h);
+        if (!same2 || maxw != w || maxh != h)
+            same = 0;
+    }
+    *psame = same;
+    if (pmaxw) *pmaxw = maxw;
+    if (pmaxh) *pmaxh = maxh;
+    return 0;
 }
 
 
@@ -2398,15 +2526,13 @@ PIXA  *pixa;
 
     if (!filename)
         return (PIXA *)ERROR_PTR("filename not defined", procName, NULL);
+
     if ((fp = fopenReadStream(filename)) == NULL)
         return (PIXA *)ERROR_PTR("stream not opened", procName, NULL);
-
-    if ((pixa = pixaReadStream(fp)) == NULL) {
-        fclose(fp);
-        return (PIXA *)ERROR_PTR("pixa not read", procName, NULL);
-    }
-
+    pixa = pixaReadStream(fp);
     fclose(fp);
+    if (!pixa)
+        return (PIXA *)ERROR_PTR("pixa not read", procName, NULL);
     return pixa;
 }
 
@@ -2520,7 +2646,8 @@ l_int32
 pixaWrite(const char  *filename,
           PIXA        *pixa)
 {
-FILE  *fp;
+l_int32  ret;
+FILE    *fp;
 
     PROCNAME("pixaWrite");
 
@@ -2535,9 +2662,10 @@ FILE  *fp;
 
     if ((fp = fopenWriteStream(filename, "wb")) == NULL)
         return ERROR_INT("stream not opened", procName, 1);
-    if (pixaWriteStream(fp, pixa))
-        return ERROR_INT("pixa not written to stream", procName, 1);
+    ret = pixaWriteStream(fp, pixa);
     fclose(fp);
+    if (ret)
+        return ERROR_INT("pixa not written to stream", procName, 1);
     return 0;
 }
 
@@ -2776,15 +2904,13 @@ PIXAA  *paa;
 
     if (!filename)
         return (PIXAA *)ERROR_PTR("filename not defined", procName, NULL);
+
     if ((fp = fopenReadStream(filename)) == NULL)
         return (PIXAA *)ERROR_PTR("stream not opened", procName, NULL);
-
-    if ((paa = pixaaReadStream(fp)) == NULL) {
-        fclose(fp);
-        return (PIXAA *)ERROR_PTR("paa not read", procName, NULL);
-    }
-
+    paa = pixaaReadStream(fp);
     fclose(fp);
+    if (!paa)
+        return (PIXAA *)ERROR_PTR("paa not read", procName, NULL);
     return paa;
 }
 
@@ -2828,18 +2954,23 @@ PIXAA   *paa;
 
     if ((paa = pixaaCreate(n)) == NULL)
         return (PIXAA *)ERROR_PTR("paa not made", procName, NULL);
-    if ((boxa = boxaReadStream(fp)) == NULL)
+    if ((boxa = boxaReadStream(fp)) == NULL) {
+        pixaaDestroy(&paa);
         return (PIXAA *)ERROR_PTR("boxa not made", procName, NULL);
+    }
     boxaDestroy(&paa->boxa);
     paa->boxa = boxa;
 
     for (i = 0; i < n; i++) {
         if ((fscanf(fp, "\n\n --------------- pixa[%d] ---------------\n",
                     &ignore)) != 1) {
+            pixaaDestroy(&paa);
             return (PIXAA *)ERROR_PTR("text reading", procName, NULL);
         }
-        if ((pixa = pixaReadStream(fp)) == NULL)
+        if ((pixa = pixaReadStream(fp)) == NULL) {
+            pixaaDestroy(&paa);
             return (PIXAA *)ERROR_PTR("pixa not read", procName, NULL);
+        }
         pixaaAddPixa(paa, pixa, L_INSERT);
     }
 
@@ -2892,7 +3023,8 @@ l_int32
 pixaaWrite(const char  *filename,
            PIXAA       *paa)
 {
-FILE  *fp;
+l_int32  ret;
+FILE    *fp;
 
     PROCNAME("pixaaWrite");
 
@@ -2907,10 +3039,10 @@ FILE  *fp;
 
     if ((fp = fopenWriteStream(filename, "wb")) == NULL)
         return ERROR_INT("stream not opened", procName, 1);
-    if (pixaaWriteStream(fp, paa))
-        return ERROR_INT("paa not written to stream", procName, 1);
+    ret = pixaaWriteStream(fp, paa);
     fclose(fp);
-
+    if (ret)
+        return ERROR_INT("paa not written to stream", procName, 1);
     return 0;
 }
 
