@@ -58,7 +58,7 @@
  *     pixaccMultConst(pacc, 0.4);
  *     pixaccMultConstAccumulate(pacc, pix2, 0.6);  // Add in 0.6 of the second
  *     pixd = pixaccFinal(pacc, d);  // Get the result
- *     pixaccDestroy(\&pacc);
+ *     pixaccDestroy(&pacc);
  * </pre>
  */
 
@@ -100,8 +100,10 @@ PIXACC  *pixacc;
     pixacc->w = w;
     pixacc->h = h;
 
-    if ((pixacc->pix = pixCreate(w, h, 32)) == NULL)
+    if ((pixacc->pix = pixCreate(w, h, 32)) == NULL) {
+        pixaccDestroy(&pixacc);
         return (PIXACC *)ERROR_PTR("pix not made", procName, NULL);
+    }
 
     if (negflag) {
         pixacc->offset = 0x40000000;
