@@ -41,6 +41,7 @@
  *           PIXA     *pixaSelectByAreaFraction()
  *           PIX      *pixSelectByWidthHeightRatio()
  *           PIXA     *pixaSelectByWidthHeightRatio()
+ *           PIXA     *pixaSelectByNumConnComp()
  *
  *           PIXA     *pixaSelectWithIndicator()
  *           l_int32   pixRemoveWithIndicator()
@@ -71,6 +72,7 @@
  *           l_int32   pixaaSizeRange()
  *           l_int32   pixaSizeRange()
  *           PIXA     *pixaClipToPix()
+ *           PIXA     *pixaClipToForeground()
  *           l_int32   pixaGetRenderingDepth()
  *           l_int32   pixaHasColor()
  *           l_int32   pixaAnyColormaps()
@@ -78,6 +80,7 @@
  *           PIXA     *pixaConvertToSameDepth()
  *           l_int32   pixaEqual()
  *           PIXA     *pixaRotateOrth()
+ *           l_int32   pixaSetFullSizeBoxa()
  * </pre>
  */
 
@@ -173,14 +176,14 @@ static const l_int32   MIN_COMPS_FOR_BIN_SORT = 200;
 /*!
  * \brief   pixSelectBySize()
  *
- * \param[in]    pixs 1 bpp
- * \param[in]    width, height threshold dimensions
- * \param[in]    connectivity 4 or 8
- * \param[in]    type L_SELECT_WIDTH, L_SELECT_HEIGHT,
- *                    L_SELECT_IF_EITHER, L_SELECT_IF_BOTH
- * \param[in]    relation L_SELECT_IF_LT, L_SELECT_IF_GT,
- *                        L_SELECT_IF_LTE, L_SELECT_IF_GTE
- * \param[out]   pchanged [optional] 1 if changed; 0 otherwise
+ * \param[in]    pixs           1 bpp
+ * \param[in]    width, height  threshold dimensions
+ * \param[in]    connectivity   4 or 8
+ * \param[in]    type           L_SELECT_WIDTH, L_SELECT_HEIGHT,
+ *                              L_SELECT_IF_EITHER, L_SELECT_IF_BOTH
+ * \param[in]    relation       L_SELECT_IF_LT, L_SELECT_IF_GT,
+ *                              L_SELECT_IF_LTE, L_SELECT_IF_GTE
+ * \param[out]   pchanged       [optional] 1 if changed; 0 otherwise
  * \return  filtered pixd, or NULL on error
  *
  * <pre>
@@ -263,12 +266,12 @@ PIXA    *pixas, *pixad;
  * \brief   pixaSelectBySize()
  *
  * \param[in]    pixas
- * \param[in]    width, height threshold dimensions
- * \param[in]    type L_SELECT_WIDTH, L_SELECT_HEIGHT,
- *                    L_SELECT_IF_EITHER, L_SELECT_IF_BOTH
- * \param[in]    relation L_SELECT_IF_LT, L_SELECT_IF_GT,
- *                        L_SELECT_IF_LTE, L_SELECT_IF_GTE
- * \param[out]   pchanged [optional] 1 if changed; 0 otherwise
+ * \param[in]    width, height  threshold dimensions
+ * \param[in]    type           L_SELECT_WIDTH, L_SELECT_HEIGHT,
+ *                              L_SELECT_IF_EITHER, L_SELECT_IF_BOTH
+ * \param[in]    relation       L_SELECT_IF_LT, L_SELECT_IF_GT,
+ *                              L_SELECT_IF_LTE, L_SELECT_IF_GTE
+ * \param[out]   pchanged       [optional] 1 if changed; 0 otherwise
  * \return  pixad, or NULL on error
  *
  * <pre>
@@ -321,11 +324,11 @@ PIXA  *pixad;
  * \brief   pixaMakeSizeIndicator()
  *
  * \param[in]    pixa
- * \param[in]    width, height threshold dimensions
- * \param[in]    type L_SELECT_WIDTH, L_SELECT_HEIGHT,
- *                    L_SELECT_IF_EITHER, L_SELECT_IF_BOTH
- * \param[in]    relation L_SELECT_IF_LT, L_SELECT_IF_GT,
- *                        L_SELECT_IF_LTE, L_SELECT_IF_GTE
+ * \param[in]    width, height  threshold dimensions
+ * \param[in]    type           L_SELECT_WIDTH, L_SELECT_HEIGHT,
+ *                              L_SELECT_IF_EITHER, L_SELECT_IF_BOTH
+ * \param[in]    relation       L_SELECT_IF_LT, L_SELECT_IF_GT,
+ *                              L_SELECT_IF_LTE, L_SELECT_IF_GTE
  * \return  na indicator array, or NULL on error
  *
  * <pre>
@@ -410,12 +413,12 @@ NUMA    *na;
 /*!
  * \brief   pixSelectByPerimToAreaRatio()
  *
- * \param[in]    pixs 1 bpp
- * \param[in]    thresh threshold ratio of fg boundary to fg pixels
- * \param[in]    connectivity 4 or 8
- * \param[in]    type L_SELECT_IF_LT, L_SELECT_IF_GT,
- *                    L_SELECT_IF_LTE, L_SELECT_IF_GTE
- * \param[out]   pchanged [optional] 1 if changed; 0 if clone returned
+ * \param[in]    pixs          1 bpp
+ * \param[in]    thresh        threshold ratio of fg boundary to fg pixels
+ * \param[in]    connectivity  4 or 8
+ * \param[in]    type          L_SELECT_IF_LT, L_SELECT_IF_GT,
+ *                             L_SELECT_IF_LTE, L_SELECT_IF_GTE
+ * \param[out]   pchanged      [optional] 1 if changed; 0 if clone returned
  * \return  pixd, or NULL on error
  *
  * <pre>
@@ -492,10 +495,10 @@ PIXA    *pixas, *pixad;
  * \brief   pixaSelectByPerimToAreaRatio()
  *
  * \param[in]    pixas
- * \param[in]    thresh threshold ratio of fg boundary to fg pixels
- * \param[in]    type L_SELECT_IF_LT, L_SELECT_IF_GT,
- *                    L_SELECT_IF_LTE, L_SELECT_IF_GTE
- * \param[out]   pchanged [optional] 1 if changed; 0 if clone returned
+ * \param[in]    thresh     threshold ratio of fg boundary to fg pixels
+ * \param[in]    type       L_SELECT_IF_LT, L_SELECT_IF_GT,
+ *                          L_SELECT_IF_LTE, L_SELECT_IF_GTE
+ * \param[out]   pchanged   [optional] 1 if changed; 0 if clone returned
  * \return  pixad, or NULL on error
  *
  * <pre>
@@ -540,12 +543,12 @@ PIXA  *pixad;
 /*!
  * \brief   pixSelectByPerimSizeRatio()
  *
- * \param[in]    pixs 1 bpp
- * \param[in]    thresh threshold ratio of fg boundary to fg pixels
- * \param[in]    connectivity 4 or 8
- * \param[in]    type L_SELECT_IF_LT, L_SELECT_IF_GT,
- *                    L_SELECT_IF_LTE, L_SELECT_IF_GTE
- * \param[out]   pchanged [optional] 1 if changed; 0 if clone returned
+ * \param[in]    pixs          1 bpp
+ * \param[in]    thresh        threshold ratio of fg boundary to fg pixels
+ * \param[in]    connectivity  4 or 8
+ * \param[in]    type          L_SELECT_IF_LT, L_SELECT_IF_GT,
+ *                             L_SELECT_IF_LTE, L_SELECT_IF_GTE
+ * \param[out]   pchanged      [optional] 1 if changed; 0 if clone returned
  * \return  pixd, or NULL on error
  *
  * <pre>
@@ -623,10 +626,10 @@ PIXA    *pixas, *pixad;
  * \brief   pixaSelectByPerimSizeRatio()
  *
  * \param[in]    pixas
- * \param[in]    thresh threshold ratio of fg boundary to b.b. circumference
- * \param[in]    type L_SELECT_IF_LT, L_SELECT_IF_GT,
- *                    L_SELECT_IF_LTE, L_SELECT_IF_GTE
- * \param[out]   pchanged [optional] 1 if changed; 0 if clone returned
+ * \param[in]    thresh    threshold ratio of fg boundary to b.b. circumference
+ * \param[in]    type      L_SELECT_IF_LT, L_SELECT_IF_GT,
+ *                         L_SELECT_IF_LTE, L_SELECT_IF_GTE
+ * \param[out]   pchanged  [optional] 1 if changed; 0 if clone returned
  * \return  pixad, or NULL on error
  *
  * <pre>
@@ -671,12 +674,12 @@ PIXA  *pixad;
 /*!
  * \brief   pixSelectByAreaFraction()
  *
- * \param[in]    pixs 1 bpp
- * \param[in]    thresh threshold ratio of fg pixels to (w * h)
- * \param[in]    connectivity 4 or 8
- * \param[in]    type L_SELECT_IF_LT, L_SELECT_IF_GT,
- *                    L_SELECT_IF_LTE, L_SELECT_IF_GTE
- * \param[out]   pchanged [optional] 1 if changed; 0 if clone returned
+ * \param[in]    pixs          1 bpp
+ * \param[in]    thresh        threshold ratio of fg pixels to (w * h)
+ * \param[in]    connectivity  4 or 8
+ * \param[in]    type          L_SELECT_IF_LT, L_SELECT_IF_GT,
+ *                             L_SELECT_IF_LTE, L_SELECT_IF_GTE
+ * \param[out]   pchanged      [optional] 1 if changed; 0 if clone returned
  * \return  pixd, or NULL on error
  *
  * <pre>
@@ -753,10 +756,10 @@ PIXA    *pixas, *pixad;
  * \brief   pixaSelectByAreaFraction()
  *
  * \param[in]    pixas
- * \param[in]    thresh threshold ratio of fg pixels to (w * h)
- * \param[in]    type L_SELECT_IF_LT, L_SELECT_IF_GT,
- *                    L_SELECT_IF_LTE, L_SELECT_IF_GTE
- * \param[out]   pchanged [optional] 1 if changed; 0 if clone returned
+ * \param[in]    thresh      threshold ratio of fg pixels to (w * h)
+ * \param[in]    type        L_SELECT_IF_LT, L_SELECT_IF_GT,
+ *                           L_SELECT_IF_LTE, L_SELECT_IF_GTE
+ * \param[out]   pchanged    [optional] 1 if changed; 0 if clone returned
  * \return  pixad, or NULL on error
  *
  * <pre>
@@ -805,12 +808,12 @@ PIXA  *pixad;
 /*!
  * \brief   pixSelectByWidthHeightRatio()
  *
- * \param[in]    pixs 1 bpp
- * \param[in]    thresh threshold ratio of width/height
- * \param[in]    connectivity 4 or 8
- * \param[in]    type L_SELECT_IF_LT, L_SELECT_IF_GT,
- *                    L_SELECT_IF_LTE, L_SELECT_IF_GTE
- * \param[out]   pchanged [optional] 1 if changed; 0 if clone returned
+ * \param[in]    pixs          1 bpp
+ * \param[in]    thresh        threshold ratio of width/height
+ * \param[in]    connectivity  4 or 8
+ * \param[in]    type          L_SELECT_IF_LT, L_SELECT_IF_GT,
+ *                             L_SELECT_IF_LTE, L_SELECT_IF_GTE
+ * \param[out]   pchanged      [optional] 1 if changed; 0 if clone returned
  * \return  pixd, or NULL on error
  *
  * <pre>
@@ -886,10 +889,10 @@ PIXA    *pixas, *pixad;
  * \brief   pixaSelectByWidthHeightRatio()
  *
  * \param[in]    pixas
- * \param[in]    thresh threshold ratio of width/height
- * \param[in]    type L_SELECT_IF_LT, L_SELECT_IF_GT,
- *                    L_SELECT_IF_LTE, L_SELECT_IF_GTE
- * \param[out]   pchanged [optional] 1 if changed; 0 if clone returned
+ * \param[in]    thresh      threshold ratio of width/height
+ * \param[in]    type        L_SELECT_IF_LT, L_SELECT_IF_GT,
+ *                           L_SELECT_IF_LTE, L_SELECT_IF_GTE
+ * \param[out]   pchanged    [optional] 1 if changed; 0 if clone returned
  * \return  pixad, or NULL on error
  *
  * <pre>
@@ -935,12 +938,72 @@ PIXA  *pixad;
 }
 
 
+/*!
+ * \brief   pixaSelectByNumConnComp()
+ *
+ * \param[in]    pixas
+ * \param[in]    nmin          minimum number of components
+ * \param[in]    nmax          maximum number of components
+ * \param[in]    connectivity  4 or 8
+ * \param[out]   pchanged      [optional] 1 if changed; 0 if clone returned
+ * \return  pixad, or NULL on error
+ *
+ * <pre>
+ * Notes:
+ *      (1) Returns a pixa clone if no components are removed.
+ *      (2) Uses pix and box clones in the new pixa.
+ *      (3) This filters by the number of connected components in
+ *          a given range.
+ * </pre>
+ */
+PIXA *
+pixaSelectByNumConnComp(PIXA      *pixas,
+                        l_int32    nmin,
+                        l_int32    nmax,
+                        l_int32    connectivity,
+                        l_int32   *pchanged)
+{
+l_int32  n, i, count;
+NUMA    *na;
+PIX     *pix;
+PIXA    *pixad;
+
+    PROCNAME("pixaSelectByNumConnComp");
+
+    if (pchanged) *pchanged = 0;
+    if (!pixas)
+        return (PIXA *)ERROR_PTR("pixas not defined", procName, NULL);
+    if (nmin > nmax)
+        return (PIXA *)ERROR_PTR("nmin > nmax", procName, NULL);
+    if (connectivity != 4 && connectivity != 8)
+        return (PIXA *)ERROR_PTR("connectivity not 4 or 8", procName, NULL);
+
+        /* Get indicator array based on number of c.c. */
+    n = pixaGetCount(pixas);
+    na = numaCreate(n);
+    for (i = 0; i < n; i++) {
+        pix = pixaGetPix(pixas, i, L_CLONE);
+        pixCountConnComp(pix, connectivity, &count);
+        if (count >= nmin && count <= nmax)
+            numaAddNumber(na, 1);
+        else
+            numaAddNumber(na, 0);
+        pixDestroy(&pix);
+    }
+
+        /* Filter to get output */
+    pixad = pixaSelectWithIndicator(pixas, na, pchanged);
+    numaDestroy(&na);
+    return pixad;
+}
+
+
 /*!
  * \brief   pixaSelectWithIndicator()
  *
  * \param[in]    pixas
- * \param[in]    na indicator numa
- * \param[out]   pchanged [optional] 1 if changed; 0 if clone returned
+ * \param[in]    na         indicator numa
+ * \param[out]   pchanged   [optional] 1 if changed; 0 if clone returned
  * \return  pixad, or NULL on error
  *
  * <pre>
@@ -959,7 +1022,7 @@ pixaSelectWithIndicator(PIXA     *pixas,
 {
 l_int32  i, n, nbox, ival, nsave;
 BOX     *box;
-PIX     *pixt;
+PIX     *pix1;
 PIXA    *pixad;
 
     PROCNAME("pixaSelectWithIndicator");
@@ -986,8 +1049,8 @@ PIXA    *pixad;
     for (i = 0; i < n; i++) {
         numaGetIValue(na, i, &ival);
         if (ival == 0) continue;
-        pixt = pixaGetPix(pixas, i, L_CLONE);
-        pixaAddPix(pixad, pixt, L_INSERT);
+        pix1 = pixaGetPix(pixas, i, L_CLONE);
+        pixaAddPix(pixad, pix1, L_INSERT);
         if (nbox == n) {   /* fully populated boxa */
             box = pixaGetBox(pixas, i, L_CLONE);
             pixaAddBox(pixad, box, L_INSERT);
@@ -1001,9 +1064,9 @@ PIXA    *pixad;
 /*!
  * \brief   pixRemoveWithIndicator()
  *
- * \param[in]    pixs 1 bpp pix from which components are removed; in-place
- * \param[in]    pixa of connected components in pixs
- * \param[in]    na numa indicator: remove components corresponding to 1s
+ * \param[in]    pixs     1 bpp pix from which components are removed; in-place
+ * \param[in]    pixa     of connected components in pixs
+ * \param[in]    na       numa indicator: remove components corresponding to 1s
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -1053,10 +1116,10 @@ PIX     *pix;
 /*!
  * \brief   pixAddWithIndicator()
  *
- * \param[in]    pixs 1 bpp pix from which components are added; in-place
- * \param[in]    pixa of connected components, some of which will be put
- *                    into pixs
- * \param[in]    na numa indicator: add components corresponding to 1s
+ * \param[in]    pixs     1 bpp pix from which components are added; in-place
+ * \param[in]    pixa     of connected components, some of which will be put
+ *                        into pixs
+ * \param[in]    na       numa indicator: add components corresponding to 1s
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -1106,9 +1169,10 @@ PIX     *pix;
  * \brief   pixaSelectWithString()
  *
  * \param[in]    pixas
- * \param[in]    str string of indices into pixa, giving the pix to be selected
- * \param[out]   perror [optional] 1 if any indices are invalid;
- *                      0 if all indices are valid
+ * \param[in]    str      string of indices into pixa, giving the pix to
+ *                        be selected
+ * \param[out]   perror   [optional] 1 if any indices are invalid;
+ *                        0 if all indices are valid
  * \return  pixad, or NULL on error
  *
  * <pre>
@@ -1174,11 +1238,11 @@ PIXA      *pixad;
 /*!
  * \brief   pixaRenderComponent()
  *
- * \param[in]    pixs [optional] 1 bpp pix
- * \param[in]    pixa of 1 bpp connected components, one of which will
- *                    be rendered in pixs, with its origin determined
- *                    by the associated box.
- * \param[in]    index of component to be rendered
+ * \param[in]    pixs    [optional] 1 bpp pix
+ * \param[in]    pixa    of 1 bpp connected components, one of which will
+ *                       be rendered in pixs, with its origin determined
+ *                       by the associated box.
+ * \param[in]    index   of component to be rendered
  * \return  pixd, or NULL on error
  *
  * <pre>
@@ -1193,7 +1257,7 @@ pixaRenderComponent(PIX     *pixs,
                     PIXA    *pixa,
                     l_int32  index)
 {
-l_int32  n, x, y, w, h, maxdepth;
+l_int32  n, x, y, w, h, same, maxd;
 BOX     *box;
 BOXA    *boxa;
 PIX     *pix;
@@ -1207,8 +1271,8 @@ PIX     *pix;
         return (PIX *)ERROR_PTR("invalid index", procName, pixs);
     if (pixs && (pixGetDepth(pixs) != 1))
         return (PIX *)ERROR_PTR("pixs not 1 bpp", procName, pixs);
-    pixaVerifyDepth(pixa, &maxdepth);
-    if (maxdepth > 1)
+    pixaVerifyDepth(pixa, &same, &maxd);
+    if (maxd > 1)
         return (PIX *)ERROR_PTR("not all pix with d == 1", procName, pixs);
 
     boxa = pixaGetBoxa(pixa, L_CLONE);
@@ -1236,21 +1300,24 @@ PIX     *pix;
  * \brief   pixaSort()
  *
  * \param[in]    pixas
- * \param[in]    sorttype L_SORT_BY_X, L_SORT_BY_Y, L_SORT_BY_WIDTH,
- *                        L_SORT_BY_HEIGHT, L_SORT_BY_MIN_DIMENSION,
- *                        L_SORT_BY_MAX_DIMENSION, L_SORT_BY_PERIMETER,
- *                        L_SORT_BY_AREA, L_SORT_BY_ASPECT_RATIO
+ * \param[in]    sorttype   L_SORT_BY_X, L_SORT_BY_Y, L_SORT_BY_WIDTH,
+ *                          L_SORT_BY_HEIGHT, L_SORT_BY_MIN_DIMENSION,
+ *                          L_SORT_BY_MAX_DIMENSION, L_SORT_BY_PERIMETER,
+ *                          L_SORT_BY_AREA, L_SORT_BY_ASPECT_RATIO
  * \param[in]    sortorder  L_SORT_INCREASING, L_SORT_DECREASING
- * \param[out]   pnaindex [optional] index of sorted order into
- *                        original array
- * \param[in]    copyflag L_COPY, L_CLONE
+ * \param[out]   pnaindex   [optional] index of sorted order into
+ *                          original array
+ * \param[in]    copyflag   L_COPY, L_CLONE
  * \return  pixad sorted version of pixas, or NULL on error
  *
  * <pre>
  * Notes:
  *      (1) This sorts based on the data in the boxa.  If the boxa
  *          count is not the same as the pixa count, this returns an error.
- *      (2) The copyflag refers to the pix and box copies that are
+ *      (2) If the boxa is empty, it makes one corresponding to the
+ *          dimensions of each pix, which allows meaningful sorting on
+ *          all types except x and y.
+ *      (3) The copyflag refers to the pix and box copies that are
  *          inserted into the sorted pixa.  These are either L_COPY
  *          or L_CLONE.
  * </pre>
@@ -1262,7 +1329,7 @@ pixaSort(PIXA    *pixas,
          NUMA   **pnaindex,
          l_int32  copyflag)
 {
-l_int32  i, n, x, y, w, h;
+l_int32  i, n, nb, x, y, w, h;
 BOXA    *boxa;
 NUMA    *na, *naindex;
 PIXA    *pixad;
@@ -1285,10 +1352,22 @@ PIXA    *pixad;
     if (copyflag != L_COPY && copyflag != L_CLONE)
         return (PIXA *)ERROR_PTR("invalid copy flag", procName, NULL);
 
+        /* Check the pixa and boxa counts. Make a boxa if required. */
+    if ((n = pixaGetCount(pixas)) == 0) {
+        L_INFO("no pix in pixa\n", procName);
+        return pixaCopy(pixas, copyflag);
+    }
     if ((boxa = pixas->boxa) == NULL)   /* not owned; do not destroy */
-        return (PIXA *)ERROR_PTR("boxa not found", procName, NULL);
-    n = pixaGetCount(pixas);
-    if (boxaGetCount(boxa) != n)
+        return (PIXA *)ERROR_PTR("boxa not found!", procName, NULL);
+    nb = boxaGetCount(boxa);
+    if (nb == 0) {
+        pixaSetFullSizeBoxa(pixas);
+        nb = n;
+        boxa = pixas->boxa;  /* not owned */
+        if (sorttype == L_SORT_BY_X || sorttype == L_SORT_BY_Y)
+            L_WARNING("sort by x or y where all values are 0\n", procName);
+    }
+    if (nb != n)
         return (PIXA *)ERROR_PTR("boxa and pixa counts differ", procName, NULL);
 
         /* Use O(n) binsort if possible */
@@ -1338,18 +1417,21 @@ PIXA    *pixad;
     }
 
         /* Get the sort index for data array */
-    if ((naindex = numaGetSortIndex(na, sortorder)) == NULL)
+    naindex = numaGetSortIndex(na, sortorder);
+    numaDestroy(&na);
+    if (!naindex)
         return (PIXA *)ERROR_PTR("naindex not made", procName, NULL);
 
         /* Build up sorted pixa using sort index */
-    if ((pixad = pixaSortByIndex(pixas, naindex, copyflag)) == NULL)
+    if ((pixad = pixaSortByIndex(pixas, naindex, copyflag)) == NULL) {
+        numaDestroy(&naindex);
         return (PIXA *)ERROR_PTR("pixad not made", procName, NULL);
+    }
 
     if (pnaindex)
         *pnaindex = naindex;
     else
         numaDestroy(&naindex);
-    numaDestroy(&na);
     return pixad;
 }
 
@@ -1358,12 +1440,12 @@ PIXA    *pixad;
  * \brief   pixaBinSort()
  *
  * \param[in]    pixas
- * \param[in]    sorttype L_SORT_BY_X, L_SORT_BY_Y, L_SORT_BY_WIDTH,
- *                        L_SORT_BY_HEIGHT, L_SORT_BY_PERIMETER
- * \param[in]    sortorder  L_SORT_INCREASING, L_SORT_DECREASING
- * \param[out]   pnaindex [optional] index of sorted order into
- *                        original array
- * \param[in]    copyflag L_COPY, L_CLONE
+ * \param[in]    sorttype    L_SORT_BY_X, L_SORT_BY_Y, L_SORT_BY_WIDTH,
+ *                           L_SORT_BY_HEIGHT, L_SORT_BY_PERIMETER
+ * \param[in]    sortorder   L_SORT_INCREASING, L_SORT_DECREASING
+ * \param[out]   pnaindex    [optional] index of sorted order into
+ *                           original array
+ * \param[in]    copyflag    L_COPY, L_CLONE
  * \return  pixad sorted version of pixas, or NULL on error
  *
  * <pre>
@@ -1441,18 +1523,21 @@ PIXA    *pixad;
     }
 
         /* Get the sort index for data array */
-    if ((naindex = numaGetBinSortIndex(na, sortorder)) == NULL)
+    naindex = numaGetBinSortIndex(na, sortorder);
+    numaDestroy(&na);
+    if (!naindex)
         return (PIXA *)ERROR_PTR("naindex not made", procName, NULL);
 
         /* Build up sorted pixa using sort index */
-    if ((pixad = pixaSortByIndex(pixas, naindex, copyflag)) == NULL)
+    if ((pixad = pixaSortByIndex(pixas, naindex, copyflag)) == NULL) {
+        numaDestroy(&naindex);
         return (PIXA *)ERROR_PTR("pixad not made", procName, NULL);
+    }
 
     if (pnaindex)
         *pnaindex = naindex;
     else
         numaDestroy(&naindex);
-    numaDestroy(&na);
     return pixad;
 }
 
@@ -1461,8 +1546,8 @@ PIXA    *pixad;
  * \brief   pixaSortByIndex()
  *
  * \param[in]    pixas
- * \param[in]    naindex na that maps from the new pixa to the input pixa
- * \param[in]    copyflag L_COPY, L_CLONE
+ * \param[in]    naindex    na that maps from the new pixa to the input pixa
+ * \param[in]    copyflag   L_COPY, L_CLONE
  * \return  pixad sorted, or NULL on error
  */
 PIXA *
@@ -1502,8 +1587,8 @@ PIXA    *pixad;
  * \brief   pixaSort2dByIndex()
  *
  * \param[in]    pixas
- * \param[in]    naa numaa that maps from the new pixaa to the input pixas
- * \param[in]    copyflag L_CLONE or L_COPY
+ * \param[in]    naa       numaa that maps from the new pixaa to the input pixas
+ * \param[in]    copyflag  L_CLONE or L_COPY
  * \return  paa sorted, or NULL on error
  */
 PIXAA *
@@ -1559,9 +1644,9 @@ PIXAA   *paa;
  * \brief   pixaSelectRange()
  *
  * \param[in]    pixas
- * \param[in]    first use 0 to select from the beginning
- * \param[in]    last use 0 to select to the end
- * \param[in]    copyflag L_COPY, L_CLONE
+ * \param[in]    first     use 0 to select from the beginning
+ * \param[in]    last      use 0 to select to the end
+ * \param[in]    copyflag  L_COPY, L_CLONE
  * \return  pixad, or NULL on error
  *
  * <pre>
@@ -1609,8 +1694,8 @@ PIXA    *pixad;
  * \brief   pixaaSelectRange()
  *
  * \param[in]    paas
- * \param[in]    first use 0 to select from the beginning
- * \param[in]    last use 0 to select to the end
+ * \param[in]    first    use 0 to select from the beginning
+ * \param[in]    last     use 0 to select to the end
  * \param[in]    copyflag L_COPY, L_CLONE
  * \return  paad, or NULL on error
  *
@@ -1662,8 +1747,8 @@ PIXAA   *paad;
  * \brief   pixaaScaleToSize()
  *
  * \param[in]    paas
- * \param[in]    wd  target width; use 0 if using height as target
- * \param[in]    hd  target height; use 0 if using width as target
+ * \param[in]    wd    target width; use 0 if using height as target
+ * \param[in]    hd    target height; use 0 if using width as target
  * \return  paad, or NULL on error
  *
  * <pre>
@@ -1765,8 +1850,8 @@ PIXAA   *paad;
  * \brief   pixaScaleToSize()
  *
  * \param[in]    pixas
- * \param[in]    wd  target width; use 0 if using height as target
- * \param[in]    hd  target height; use 0 if using width as target
+ * \param[in]    wd    target width; use 0 if using height as target
+ * \param[in]    hd    target height; use 0 if using width as target
  * \return  pixad, or NULL on error
  *
  * <pre>
@@ -1787,8 +1872,9 @@ PIXA    *pixad;
 
     if (!pixas)
         return (PIXA *)ERROR_PTR("pixas not defined", procName, NULL);
-    if (wd <= 0 && hd <= 0)
-        return (PIXA *)ERROR_PTR("neither wd nor hd > 0", procName, NULL);
+
+    if (wd <= 0 && hd <= 0)  /* no scaling requested */
+        return pixaCopy(pixas, L_CLONE);
 
     n = pixaGetCount(pixas);
     pixad = pixaCreate(n);
@@ -1807,8 +1893,8 @@ PIXA    *pixad;
  * \brief   pixaScaleToSizeRel()
  *
  * \param[in]    pixas
- * \param[in]    delw  change in width, in pixels; 0 means no change
- * \param[in]    delh  change in height, in pixels; 0 means no change
+ * \param[in]    delw   change in width, in pixels; 0 means no change
+ * \param[in]    delh   change in height, in pixels; 0 means no change
  * return  pixad, or NULL on error
  *
  * <pre>
@@ -1840,7 +1926,7 @@ PIXA    *pixad;
         if (pix2) {
             pixaAddPix(pixad, pix2, L_INSERT);
         } else {
-            L_WARNING("relative scale to size failed; use a copy\n", procName); 
+            L_WARNING("relative scale to size failed; use a copy\n", procName);
             pixaAddPix(pixad, pix1, L_COPY);
         }
         pixDestroy(&pix1);
@@ -1894,9 +1980,8 @@ PIXA    *pixad;
     if (nb == n) {
         boxa2 = boxaTransform(boxa1, 0, 0, scalex, scaley);
         pixaSetBoxa(pixad, boxa2, L_INSERT);
-    } else {
-        boxaDestroy(&boxa1);
     }
+    boxaDestroy(&boxa1);
     return pixad;
 }
 
@@ -1907,10 +1992,10 @@ PIXA    *pixad;
 /*!
  * \brief   pixaAddBorderGeneral()
  *
- * \param[in]    pixad can be null or equal to pixas
- * \param[in]    pixas containing pix of all depths; colormap ok
- * \param[in]    left, right, top, bot  number of pixels added
- * \param[in]    val   value of added border pixels
+ * \param[in]    pixad    can be null or equal to pixas
+ * \param[in]    pixas    containing pix of all depths; colormap ok
+ * \param[in]    left, right, top, bot     number of pixels added
+ * \param[in]    val      value of added border pixels
  * \return  pixad with border added to each pix, including on error
  *
  * <pre>
@@ -2000,7 +2085,7 @@ PIX     *pixs, *pixd;
  * Notes:
  *      (1) This 'flattens' the pixaa to a pixa, taking the pix in
  *          order in the first pixa, then the second, etc.
- *      (2) If \&naindex is defined, we generate a Numa that gives, for
+ *      (2) If &naindex is defined, we generate a Numa that gives, for
  *          each pix in the pixaa, the index of the pixa to which it belongs.
  * </pre>
  */
@@ -2055,8 +2140,8 @@ PIXA    *pixa, *pixat;
  * \brief   pixaaSizeRange()
  *
  * \param[in]    paa
- * \param[out]   pminw, pminh, pmaxw, pmaxh [optional] range of
- *                                          dimensions of all boxes
+ * \param[out]   pminw, pminh, pmaxw, pmaxh   [optional] range of
+ *                                            dimensions of all boxes
  * \return  0 if OK, 1 on error
  */
 l_int32
@@ -2109,8 +2194,8 @@ PIXA    *pixa;
  * \brief   pixaSizeRange()
  *
  * \param[in]    pixa
- * \param[out]   pminw, pminh, pmaxw, pmaxh [optional] range of
- *                                          dimensions of pix in the array
+ * \param[out]   pminw, pminh, pmaxw, pmaxh   [optional] range of
+ *                                            dimensions of pix in the array
  * \return  0 if OK, 1 on error
  */
 l_int32
@@ -2217,12 +2302,65 @@ PIXA    *pixad;
 }
 
 
+/*!
+ * \brief   pixaClipToForeground()
+ *
+ * \param[in]    pixas
+ * \param[out]   ppixad   [optional] pixa of clipped pix returned
+ * \param[out]   pboxa    [optional] clipping boxes returned
+ * \return  0 if OK, 1 on error
+ *
+ * <pre>
+ * Notes:
+ *      (1) At least one of [&pixd, &boxa] must be specified.
+ *      (2) Any pix with no fg pixels is skipped.
+ *      (3) See pixClipToForeground().
+ * </pre>
+ */
+l_int32
+pixaClipToForeground(PIXA   *pixas,
+                     PIXA  **ppixad,
+                     BOXA  **pboxa)
+{
+l_int32  i, n;
+BOX     *box1;
+PIX     *pix1, *pix2;
+
+    PROCNAME("pixaClipToForeground");
+
+    if (ppixad) *ppixad = NULL;
+    if (pboxa) *pboxa = NULL;
+    if (!pixas)
+        return ERROR_INT("pixas not defined", procName, 1);
+    if (!ppixad && !pboxa)
+        return ERROR_INT("no output requested", procName, 1);
+
+    n = pixaGetCount(pixas);
+    if (ppixad) *ppixad = pixaCreate(n);
+    if (pboxa) *pboxa = boxaCreate(n);
+    for (i = 0; i < n; i++) {
+        pix1 = pixaGetPix(pixas, i, L_CLONE);
+        pixClipToForeground(pix1, &pix2, &box1);
+        pixDestroy(&pix1);
+        if (ppixad)
+            pixaAddPix(*ppixad, pix2, L_INSERT);
+        else
+            pixDestroy(&pix2);
+        if (pboxa)
+            boxaAddBox(*pboxa, box1, L_INSERT);
+        else
+            boxDestroy(&box1);
+    }
+
+    return 0;
+}
+
+
 /*!
  * \brief   pixaGetRenderingDepth()
  *
  * \param[in]    pixa
- * \param[out]   pdepth depth required to render if all
- *                      colormaps are removed
+ * \param[out]   pdepth   depth required to render if all colormaps are removed
  * \return  0 if OK; 1 on error
  */
 l_int32
@@ -2258,8 +2396,8 @@ l_int32  hascolor, maxdepth;
  * \brief   pixaHasColor()
  *
  * \param[in]    pixa
- * \param[out]   phascolor 1 if any pix is rgb or has
- *                         a colormap with color; 0 otherwise
+ * \param[out]   phascolor   1 if any pix is rgb or has a colormap with color;
+ *                           0 otherwise
  * \return  0 if OK; 1 on error
  */
 l_int32
@@ -2279,6 +2417,7 @@ PIXCMAP  *cmap;
         return ERROR_INT("pixa not defined", procName, 1);
 
     n = pixaGetCount(pixa);
+    hascolor = 0;
     for (i = 0; i < n; i++) {
         pix = pixaGetPix(pixa, i, L_CLONE);
         if ((cmap = pixGetColormap(pix)) != NULL)
@@ -2299,7 +2438,7 @@ PIXCMAP  *cmap;
  * \brief   pixaAnyColormaps()
  *
  * \param[in]    pixa
- * \param[out]   phascmap 1 if any pix has a colormap; 0 otherwise
+ * \param[out]   phascmap    1 if any pix has a colormap; 0 otherwise
  * \return  0 if OK; 1 on error
  */
 l_int32
@@ -2337,8 +2476,8 @@ PIXCMAP  *cmap;
  * \brief   pixaGetDepthInfo()
  *
  * \param[in]    pixa
- * \param[out]   pmaxdepth [optional] max pixel depth of pix in pixa
- * \param[out]   psame [optional] true if all depths are equal
+ * \param[out]   pmaxdepth  [optional] max pixel depth of pix in pixa
+ * \param[out]   psame      [optional] true if all depths are equal
  * \return  0 if OK; 1 on error
  */
 l_int32
@@ -2394,8 +2533,9 @@ PIXA *
 pixaConvertToSameDepth(PIXA  *pixas)
 {
 l_int32  i, n, same, hascmap, maxdepth;
-PIX     *pix, *pixt;
-PIXA    *pixat, *pixad;
+BOXA    *boxa;
+PIX     *pix1, *pix2;
+PIXA    *pixa1, *pixad;
 
     PROCNAME("pixaConvertToSameDepth");
 
@@ -2407,33 +2547,36 @@ PIXA    *pixat, *pixad;
         return (PIXA *)ERROR_PTR("no components", procName, NULL);
     pixaAnyColormaps(pixas, &hascmap);
     if (hascmap) {
-        pixat = pixaCreate(n);
+        pixa1 = pixaCreate(n);
         for (i = 0; i < n; i++) {
-            pixt = pixaGetPix(pixas, i, L_CLONE);
-            pix = pixConvertTo32(pixt);
-            pixaAddPix(pixat, pix, L_INSERT);
-            pixDestroy(&pixt);
+            pix1 = pixaGetPix(pixas, i, L_CLONE);
+            pix2 = pixConvertTo32(pix1);
+            pixaAddPix(pixa1, pix2, L_INSERT);
+            pixDestroy(&pix1);
         }
     } else {
-        pixat = pixaCopy(pixas, L_CLONE);
+        pixa1 = pixaCopy(pixas, L_CLONE);
     }
 
-    pixaGetDepthInfo(pixat, &maxdepth, &same);
+    pixaGetDepthInfo(pixa1, &maxdepth, &same);
     if (!same) {  /* at least one pix has depth < maxdepth */
         pixad = pixaCreate(n);
         for (i = 0; i < n; i++) {
-            pixt = pixaGetPix(pixat, i, L_CLONE);
+            pix1 = pixaGetPix(pixa1, i, L_CLONE);
             if (maxdepth <= 8)
-                pix = pixConvertTo8(pixt, 0);
+                pix2 = pixConvertTo8(pix1, 0);
             else
-                pix = pixConvertTo32(pixt);
-            pixaAddPix(pixad, pix, L_INSERT);
-            pixDestroy(&pixt);
+                pix2 = pixConvertTo32(pix1);
+            pixaAddPix(pixad, pix2, L_INSERT);
+            pixDestroy(&pix1);
         }
     } else {
-        pixad = pixaCopy(pixat, L_CLONE);
+        pixad = pixaCopy(pixa1, L_CLONE);
     }
-    pixaDestroy(&pixat);
+
+    boxa = pixaGetBoxa(pixas, L_COPY);
+    pixaSetBoxa(pixad, boxa, L_INSERT);
+    pixaDestroy(&pixa1);
     return pixad;
 }
 
@@ -2444,7 +2587,7 @@ PIXA    *pixat, *pixad;
  * \param[in]    pixa1
  * \param[in]    pixa2
  * \param[in]    maxdist
- * \param[out]   pnaindex [optional] index array of correspondences
+ * \param[out]   pnaindex    [optional] index array of correspondences
  *           [out]   psame (1 if equal; 0 otherwise
  * \return  0 if OK, 1 on error
  *
@@ -2542,8 +2685,8 @@ PIX      *pix1, *pix2;
  * \brief   pixaRotateOrth()
  *
  * \param[in]    pixas
- * \param[in]    rotation 0 = noop, 1 = 90 deg, 2 = 180 deg, 3 = 270 deg;
- *                        all rotations are clockwise
+ * \param[in]    rotation    0 = noop, 1 = 90 deg, 2 = 180 deg, 3 = 270 deg;
+ *                           all rotations are clockwise
  * \return  pixad, or NULL on error
  *
  * <pre>
@@ -2575,8 +2718,10 @@ PIXA    *pixad;
     if ((pixad = pixaCreate(n)) == NULL)
         return (PIXA *)ERROR_PTR("pixad not made", procName, NULL);
     for (i = 0; i < n; i++) {
-        if ((pixs = pixaGetPix(pixas, i, L_CLONE)) == NULL)
+        if ((pixs = pixaGetPix(pixas, i, L_CLONE)) == NULL) {
+            pixaDestroy(&pixad);
             return (PIXA *)ERROR_PTR("pixs not found", procName, NULL);
+        }
         pixd = pixRotateOrth(pixs, rotation);
         pixaAddPix(pixad, pixd, L_INSERT);
         if (n == nb) {
@@ -2591,3 +2736,47 @@ PIXA    *pixad;
 
     return pixad;
 }
+
+
+/*!
+ * \brief   pixaSetFullSizeBoxa()
+ *
+ * \param[in]    pixa
+ * \return  0 if OK, 1 on error
+ *
+ * <pre>
+ * Notes:
+ *      (1) Replaces the existing boxa.  Each box gives the dimensions
+ *          of the corresponding pix.  This is needed for functions
+ *          like pixaSort() that sort based on the boxes.
+ * </pre>
+ */
+l_int32
+pixaSetFullSizeBoxa(PIXA  *pixa)
+{
+l_int32  i, n, w, h;
+BOX     *box;
+BOXA    *boxa;
+PIX     *pix;
+
+    PROCNAME("pixaSetFullSizeBoxa");
+
+    if (!pixa)
+        return ERROR_INT("pixa not defined", procName, 1);
+    if ((n = pixaGetCount(pixa)) == 0) {
+        L_INFO("pixa contains no pix\n", procName);
+        return 0;
+    }
+
+    boxa = boxaCreate(n);
+    pixaSetBoxa(pixa, boxa, L_INSERT);
+    for (i = 0; i < n; i++) {
+        pix = pixaGetPix(pixa, i, L_CLONE);
+        pixGetDimensions(pix, &w, &h, NULL);
+        box = boxCreate(0, 0, w, h);
+        boxaAddBox(boxa, box, L_INSERT);
+        pixDestroy(&pix);
+    }
+    return 0;
+}
+
