@@ -24,8 +24,9 @@
  -  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *====================================================================*/
 
-/*
- *  projective.c
+/*!
+ * \file projective.c
+ * <pre>
  *
  *      Projective (4 pt) image transformation using a sampled
  *      (to nearest integer) transform on each dest point
@@ -87,9 +88,9 @@
  *      (x',y') in the src.  That computed point in the src is then used
  *      to determine the corresponding dest pixel value in one of two ways:
  *
- *       - sampling: simply take the value of the src pixel in which this
+ *       ~ sampling: simply take the value of the src pixel in which this
  *                   point falls
- *       - interpolation: take appropriate linear combinations of the
+ *       ~ interpolation: take appropriate linear combinations of the
  *                        four src pixels that this dest pixel would
  *                        overlap, with the coefficients proportional
  *                        to the amount of overlap
@@ -104,6 +105,7 @@
  *               interpolated   1.6
  *      Additionally, the computation time/pixel is nearly the same
  *      for 8 bpp and 32 bpp, for both sampled and interpolated.
+ * </pre>
  */
 
 #include <string.h>
@@ -117,21 +119,23 @@ extern l_float32  AlphaMaskBorderVals[2];
  *            Sampled projective image transformation          *
  *-------------------------------------------------------------*/
 /*!
- *  pixProjectiveSampledPta()
+ * \brief   pixProjectiveSampledPta()
  *
- *      Input:  pixs (all depths)
- *              ptad  (4 pts of final coordinate space)
- *              ptas  (4 pts of initial coordinate space)
- *              incolor (L_BRING_IN_WHITE, L_BRING_IN_BLACK)
- *      Return: pixd, or null on error
+ * \param[in]    pixs all depths
+ * \param[in]    ptad  4 pts of final coordinate space
+ * \param[in]    ptas  4 pts of initial coordinate space
+ * \param[in]    incolor L_BRING_IN_WHITE, L_BRING_IN_BLACK
+ * \return  pixd, or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) Brings in either black or white pixels from the boundary.
  *      (2) Retains colormap, which you can do for a sampled transform..
  *      (3) No 3 of the 4 points may be collinear.
  *      (4) For 8 and 32 bpp pix, better quality is obtained by the
  *          somewhat slower pixProjectivePta().  See that
  *          function for relative timings between sampled and interpolated.
+ * </pre>
  */
 PIX *
 pixProjectiveSampledPta(PIX     *pixs,
@@ -160,26 +164,28 @@ PIX        *pixd;
         /* Get backwards transform from dest to src, and apply it */
     getProjectiveXformCoeffs(ptad, ptas, &vc);
     pixd = pixProjectiveSampled(pixs, vc, incolor);
-    FREE(vc);
+    LEPT_FREE(vc);
 
     return pixd;
 }
 
 
 /*!
- *  pixProjectiveSampled()
+ * \brief   pixProjectiveSampled()
  *
- *      Input:  pixs (all depths)
- *              vc  (vector of 8 coefficients for projective transformation)
- *              incolor (L_BRING_IN_WHITE, L_BRING_IN_BLACK)
- *      Return: pixd, or null on error
+ * \param[in]    pixs all depths
+ * \param[in]    vc  vector of 8 coefficients for projective transformation
+ * \param[in]    incolor L_BRING_IN_WHITE, L_BRING_IN_BLACK
+ * \return  pixd, or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) Brings in either black or white pixels from the boundary.
  *      (2) Retains colormap, which you can do for a sampled transform..
  *      (3) For 8 or 32 bpp, much better quality is obtained by the
  *          somewhat slower pixProjective().  See that function
  *          for relative timings between sampled and interpolated.
+ * </pre>
  */
 PIX *
 pixProjectiveSampled(PIX        *pixs,
@@ -260,17 +266,19 @@ PIXCMAP    *cmap;
  *            Interpolated projective image transformation             *
  *---------------------------------------------------------------------*/
 /*!
- *  pixProjectivePta()
+ * \brief   pixProjectivePta()
  *
- *      Input:  pixs (all depths; colormap ok)
- *              ptad  (4 pts of final coordinate space)
- *              ptas  (4 pts of initial coordinate space)
- *              incolor (L_BRING_IN_WHITE, L_BRING_IN_BLACK)
- *      Return: pixd, or null on error
+ * \param[in]    pixs all depths; colormap ok
+ * \param[in]    ptad  4 pts of final coordinate space
+ * \param[in]    ptas  4 pts of initial coordinate space
+ * \param[in]    incolor L_BRING_IN_WHITE, L_BRING_IN_BLACK
+ * \return  pixd, or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) Brings in either black or white pixels from the boundary
  *      (2) Removes any existing colormap, if necessary, before transforming
+ * </pre>
  */
 PIX *
 pixProjectivePta(PIX     *pixs,
@@ -329,16 +337,18 @@ PIX      *pixt1, *pixt2, *pixd;
 
 
 /*!
- *  pixProjective()
+ * \brief   pixProjective()
  *
- *      Input:  pixs (all depths; colormap ok)
- *              vc  (vector of 8 coefficients for projective transformation)
- *              incolor (L_BRING_IN_WHITE, L_BRING_IN_BLACK)
- *      Return: pixd, or null on error
+ * \param[in]    pixs all depths; colormap ok
+ * \param[in]    vc  vector of 8 coefficients for projective transformation
+ * \param[in]    incolor L_BRING_IN_WHITE, L_BRING_IN_BLACK
+ * \return  pixd, or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) Brings in either black or white pixels from the boundary
  *      (2) Removes any existing colormap, if necessary, before transforming
+ * </pre>
  */
 PIX *
 pixProjective(PIX        *pixs,
@@ -388,13 +398,13 @@ PIX      *pixt1, *pixt2, *pixd;
 
 
 /*!
- *  pixProjectivePtaColor()
+ * \brief   pixProjectivePtaColor()
  *
- *      Input:  pixs (32 bpp)
- *              ptad  (4 pts of final coordinate space)
- *              ptas  (4 pts of initial coordinate space)
- *              colorval (e.g., 0 to bring in BLACK, 0xffffff00 for WHITE)
- *      Return: pixd, or null on error
+ * \param[in]    pixs 32 bpp
+ * \param[in]    ptad  4 pts of final coordinate space
+ * \param[in]    ptas  4 pts of initial coordinate space
+ * \param[in]    colorval e.g., 0 to bring in BLACK, 0xffffff00 for WHITE
+ * \return  pixd, or NULL on error
  */
 PIX *
 pixProjectivePtaColor(PIX      *pixs,
@@ -423,19 +433,19 @@ PIX        *pixd;
         /* Get backwards transform from dest to src, and apply it */
     getProjectiveXformCoeffs(ptad, ptas, &vc);
     pixd = pixProjectiveColor(pixs, vc, colorval);
-    FREE(vc);
+    LEPT_FREE(vc);
 
     return pixd;
 }
 
 
 /*!
- *  pixProjectiveColor()
+ * \brief   pixProjectiveColor()
  *
- *      Input:  pixs (32 bpp)
- *              vc  (vector of 8 coefficients for projective transformation)
- *              colorval (e.g., 0 to bring in BLACK, 0xffffff00 for WHITE)
- *      Return: pixd, or null on error
+ * \param[in]    pixs 32 bpp
+ * \param[in]    vc  vector of 8 coefficients for projective transformation
+ * \param[in]    colorval e.g., 0 to bring in BLACK, 0xffffff00 for WHITE
+ * \return  pixd, or NULL on error
  */
 PIX *
 pixProjectiveColor(PIX        *pixs,
@@ -491,13 +501,13 @@ PIX       *pix1, *pix2, *pixd;
 
 
 /*!
- *  pixProjectivePtaGray()
+ * \brief   pixProjectivePtaGray()
  *
- *      Input:  pixs (8 bpp)
- *              ptad  (4 pts of final coordinate space)
- *              ptas  (4 pts of initial coordinate space)
- *              grayval (0 to bring in BLACK, 255 for WHITE)
- *      Return: pixd, or null on error
+ * \param[in]    pixs 8 bpp
+ * \param[in]    ptad  4 pts of final coordinate space
+ * \param[in]    ptas  4 pts of initial coordinate space
+ * \param[in]    grayval 0 to bring in BLACK, 255 for WHITE
+ * \return  pixd, or NULL on error
  */
 PIX *
 pixProjectivePtaGray(PIX     *pixs,
@@ -526,7 +536,7 @@ PIX        *pixd;
         /* Get backwards transform from dest to src, and apply it */
     getProjectiveXformCoeffs(ptad, ptas, &vc);
     pixd = pixProjectiveGray(pixs, vc, grayval);
-    FREE(vc);
+    LEPT_FREE(vc);
 
     return pixd;
 }
@@ -534,12 +544,12 @@ PIX        *pixd;
 
 
 /*!
- *  pixProjectiveGray()
+ * \brief   pixProjectiveGray()
  *
- *      Input:  pixs (8 bpp)
- *              vc  (vector of 8 coefficients for projective transformation)
- *              grayval (0 to bring in BLACK, 255 for WHITE)
- *      Return: pixd, or null on error
+ * \param[in]    pixs 8 bpp
+ * \param[in]    vc  vector of 8 coefficients for projective transformation
+ * \param[in]    grayval 0 to bring in BLACK, 255 for WHITE
+ * \return  pixd, or NULL on error
  */
 PIX *
 pixProjectiveGray(PIX        *pixs,
@@ -587,26 +597,27 @@ PIX       *pixd;
  *            Projective transform including alpha (blend) component         *
  *---------------------------------------------------------------------------*/
 /*!
- *  pixProjectivePtaWithAlpha()
+ * \brief   pixProjectivePtaWithAlpha()
  *
- *      Input:  pixs (32 bpp rgb)
- *              ptad  (4 pts of final coordinate space)
- *              ptas  (4 pts of initial coordinate space)
- *              pixg (<optional> 8 bpp, for alpha channel, can be null)
- *              fract (between 0.0 and 1.0, with 0.0 fully transparent
- *                     and 1.0 fully opaque)
- *              border (of pixels added to capture transformed source pixels)
- *      Return: pixd, or null on error
+ * \param[in]    pixs 32 bpp rgb
+ * \param[in]    ptad  4 pts of final coordinate space
+ * \param[in]    ptas  4 pts of initial coordinate space
+ * \param[in]    pixg [optional] 8 bpp, for alpha channel, can be null
+ * \param[in]    fract between 0.0 and 1.0, with 0.0 fully transparent
+ *                     and 1.0 fully opaque
+ * \param[in]    border of pixels added to capture transformed source pixels
+ * \return  pixd, or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) The alpha channel is transformed separately from pixs,
  *          and aligns with it, being fully transparent outside the
  *          boundary of the transformed pixs.  For pixels that are fully
  *          transparent, a blending function like pixBlendWithGrayMask()
  *          will give zero weight to corresponding pixels in pixs.
  *      (2) If pixg is NULL, it is generated as an alpha layer that is
- *          partially opaque, using @fract.  Otherwise, it is cropped
- *          to pixs if required and @fract is ignored.  The alpha channel
+ *          partially opaque, using %fract.  Otherwise, it is cropped
+ *          to pixs if required and %fract is ignored.  The alpha channel
  *          in pixs is never used.
  *      (3) Colormaps are removed.
  *      (4) When pixs is transformed, it doesn't matter what color is brought
@@ -615,7 +626,7 @@ PIX       *pixd;
  *          necessary to add a border to the source pix before doing
  *          the projective transformation.  This can be any non-negative
  *          number.
- *      (6) The input @ptad and @ptas are in a coordinate space before
+ *      (6) The input %ptad and %ptas are in a coordinate space before
  *          the border is added.  Internally, we compensate for this
  *          before doing the projective transform on the image after
  *          the border is added.
@@ -627,6 +638,7 @@ PIX       *pixd;
  *              with an image below, and
  *          (b) softens the edges by weakening the aliasing there.
  *          Use l_setAlphaMaskBorder() to change these values.
+ * </pre>
  */
 PIX *
 pixProjectivePtaWithAlpha(PIX       *pixs,
@@ -648,7 +660,8 @@ PTA     *ptad2, *ptas2;
     if (d != 32 && pixGetColormap(pixs) == NULL)
         return (PIX *)ERROR_PTR("pixs not cmapped or 32 bpp", procName, NULL);
     if (pixg && pixGetDepth(pixg) != 8) {
-        L_WARNING("pixg not 8 bpp; using @fract transparent alpha\n", procName);
+        L_WARNING("pixg not 8 bpp; using 'fract' transparent alpha\n",
+                  procName);
         pixg = NULL;
     }
     if (!pixg && (fract < 0.0 || fract > 1.0)) {
@@ -707,25 +720,25 @@ PTA     *ptad2, *ptas2;
  *                Projective coordinate transformation         *
  *-------------------------------------------------------------*/
 /*!
- *  getProjectiveXformCoeffs()
+ * \brief   getProjectiveXformCoeffs()
  *
- *      Input:  ptas  (source 4 points; unprimed)
- *              ptad  (transformed 4 points; primed)
- *              &vc   (<return> vector of coefficients of transform)
- *      Return: 0 if OK; 1 on error
+ * \param[in]    ptas  source 4 points; unprimed
+ * \param[in]    ptad  transformed 4 points; primed
+ * \param[out]   pvc   vector of coefficients of transform
+ * \return  0 if OK; 1 on error
  *
  *  We have a set of 8 equations, describing the projective
- *  transformation that takes 4 points (ptas) into 4 other
- *  points (ptad).  These equations are:
+ *  transformation that takes 4 points ptas into 4 other
+ *  points ptad.  These equations are:
  *
- *          x1' = (c[0]*x1 + c[1]*y1 + c[2]) / (c[6]*x1 + c[7]*y1 + 1)
- *          y1' = (c[3]*x1 + c[4]*y1 + c[5]) / (c[6]*x1 + c[7]*y1 + 1)
- *          x2' = (c[0]*x2 + c[1]*y2 + c[2]) / (c[6]*x2 + c[7]*y2 + 1)
- *          y2' = (c[3]*x2 + c[4]*y2 + c[5]) / (c[6]*x2 + c[7]*y2 + 1)
- *          x3' = (c[0]*x3 + c[1]*y3 + c[2]) / (c[6]*x3 + c[7]*y3 + 1)
- *          y3' = (c[3]*x3 + c[4]*y3 + c[5]) / (c[6]*x3 + c[7]*y3 + 1)
- *          x4' = (c[0]*x4 + c[1]*y4 + c[2]) / (c[6]*x4 + c[7]*y4 + 1)
- *          y4' = (c[3]*x4 + c[4]*y4 + c[5]) / (c[6]*x4 + c[7]*y4 + 1)
+ *          x1' = c[0]*x1 + c[1]*y1 + c[2]) / (c[6]*x1 + c[7]*y1 + 1
+ *          y1' = c[3]*x1 + c[4]*y1 + c[5]) / (c[6]*x1 + c[7]*y1 + 1
+ *          x2' = c[0]*x2 + c[1]*y2 + c[2]) / (c[6]*x2 + c[7]*y2 + 1
+ *          y2' = c[3]*x2 + c[4]*y2 + c[5]) / (c[6]*x2 + c[7]*y2 + 1
+ *          x3' = c[0]*x3 + c[1]*y3 + c[2]) / (c[6]*x3 + c[7]*y3 + 1
+ *          y3' = c[3]*x3 + c[4]*y3 + c[5]) / (c[6]*x3 + c[7]*y3 + 1
+ *          x4' = c[0]*x4 + c[1]*y4 + c[2]) / (c[6]*x4 + c[7]*y4 + 1
+ *          y4' = c[3]*x4 + c[4]*y4 + c[5]) / (c[6]*x4 + c[7]*y4 + 1
  *
  *  Multiplying both sides of each eqn by the denominator, we get
  *
@@ -750,13 +763,13 @@ PTA     *ptad2, *ptas2;
  *  These eight equations are solved here for the coefficients C.
  *
  *  These eight coefficients can then be used to find the mapping
- *  (x,y) --> (x',y'):
+ *  x,y) --> (x',y':
  *
- *           x' = (c[0]x + c[1]y + c[2]) / (c[6]x + c[7]y + 1)
- *           y' = (c[3]x + c[4]y + c[5]) / (c[6]x + c[7]y + 1)
+ *           x' = c[0]x + c[1]y + c[2]) / (c[6]x + c[7]y + 1
+ *           y' = c[3]x + c[4]y + c[5]) / (c[6]x + c[7]y + 1
  *
- *  that is implemented in projectiveXformSampled() and
- *  projectiveXFormInterpolated().
+ *  that is implemented in projectiveXformSampled and
+ *  projectiveXFormInterpolated.
  */
 l_int32
 getProjectiveXformCoeffs(PTA         *ptas,
@@ -777,7 +790,7 @@ l_float32  *a[8];  /* 8x8 matrix A  */
     if (!pvc)
         return ERROR_INT("&vc not defined", procName, 1);
 
-    if ((b = (l_float32 *)CALLOC(8, sizeof(l_float32))) == NULL)
+    if ((b = (l_float32 *)LEPT_CALLOC(8, sizeof(l_float32))) == NULL)
         return ERROR_INT("b not made", procName, 1);
     *pvc = b;
 
@@ -791,7 +804,7 @@ l_float32  *a[8];  /* 8x8 matrix A  */
     ptaGetPt(ptad, 3, &b[6], &b[7]);
 
     for (i = 0; i < 8; i++) {
-        if ((a[i] = (l_float32 *)CALLOC(8, sizeof(l_float32))) == NULL)
+        if ((a[i] = (l_float32 *)LEPT_CALLOC(8, sizeof(l_float32))) == NULL)
             return ERROR_INT("a[i] not made", procName, 1);
     }
 
@@ -839,23 +852,25 @@ l_float32  *a[8];  /* 8x8 matrix A  */
     gaussjordan(a, b, 8);
 
     for (i = 0; i < 8; i++)
-        FREE(a[i]);
+        LEPT_FREE(a[i]);
 
     return 0;
 }
 
 
 /*!
- *  projectiveXformSampledPt()
+ * \brief   projectiveXformSampledPt()
  *
- *      Input:  vc (vector of 8 coefficients)
- *              (x, y)  (initial point)
- *              (&xp, &yp)   (<return> transformed point)
- *      Return: 0 if OK; 1 on error
+ * \param[in]    vc vector of 8 coefficients
+ * \param[in]    x, y  initial point
+ * \param[out]   pxp, pyp   transformed point
+ * \return  0 if OK; 1 on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) This finds the nearest pixel coordinates of the transformed point.
  *      (2) It does not check ptrs for returned data!
+ * </pre>
  */
 l_int32
 projectiveXformSampledPt(l_float32  *vc,
@@ -879,16 +894,18 @@ l_float32  factor;
 
 
 /*!
- *  projectiveXformPt()
+ * \brief   projectiveXformPt()
  *
- *      Input:  vc (vector of 8 coefficients)
- *              (x, y)  (initial point)
- *              (&xp, &yp)   (<return> transformed point)
- *      Return: 0 if OK; 1 on error
+ * \param[in]    vc vector of 8 coefficients
+ * \param[in]    x, y  initial point
+ * \param[out]   pxp, pyp   transformed point
+ * \return  0 if OK; 1 on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) This computes the floating point location of the transformed point.
  *      (2) It does not check ptrs for returned data!
+ * </pre>
  */
 l_int32
 projectiveXformPt(l_float32  *vc,
