@@ -487,10 +487,9 @@ SARRAY  *sa;
     else  /* boxflag == 1 */
         sarrayAddString(sa, (char *)"\ngrestore", L_COPY);
 
-    if ((outstr = sarrayToString(sa, 1)) == NULL)
-        return (char *)ERROR_PTR("outstr not made", procName, NULL);
-
+    outstr = sarrayToString(sa, 1);
     sarrayDestroy(&sa);
+    if (!outstr) L_ERROR("outstr not made\n", procName);
     return outstr;
 }
 
@@ -643,7 +642,7 @@ convertJpegToPSEmbed(const char  *filein,
                      const char  *fileout)
 {
 char         *outstr;
-l_int32       w, h, nbytes;
+l_int32       w, h, nbytes, ret;
 l_float32     xpt, ypt, wpt, hpt;
 L_COMP_DATA  *cid;
 
@@ -675,15 +674,15 @@ L_COMP_DATA  *cid;
         /* Generate the PS.
          * The bounding box information should be inserted (default). */
     outstr = generateJpegPS(NULL, cid, xpt, ypt, wpt, hpt, 1, 1);
+    l_CIDataDestroy(&cid);
     if (!outstr)
         return ERROR_INT("outstr not made", procName, 1);
     nbytes = strlen(outstr);
 
-    if (l_binaryWrite(fileout, "w", outstr, nbytes))
-        return ERROR_INT("ps string not written to file", procName, 1);
+    ret = l_binaryWrite(fileout, "w", outstr, nbytes);
     LEPT_FREE(outstr);
-    l_CIDataDestroy(&cid);
-    return 0;
+    if (ret) L_ERROR("ps string not written to file\n", procName);
+    return ret;
 }
 
 
@@ -857,8 +856,6 @@ L_COMP_DATA  *cid;
         /* Get scaled location in pts */
     if (scale == 0.0)
         scale = 1.0;
-    if (res == 0)
-        res = DEFAULT_INPUT_RES;
     xpt = scale * x * 72. / res;
     ypt = scale * y * 72. / res;
     wpt = scale * cid->w * 72. / res;
@@ -1042,7 +1039,7 @@ convertG4ToPSEmbed(const char  *filein,
                    const char  *fileout)
 {
 char         *outstr;
-l_int32       w, h, nbytes;
+l_int32       w, h, nbytes, ret;
 l_float32     xpt, ypt, wpt, hpt;
 L_COMP_DATA  *cid;
 
@@ -1073,15 +1070,15 @@ L_COMP_DATA  *cid;
         /* Generate the PS, painting through the image mask.
          * The bounding box information should be inserted (default). */
     outstr = generateG4PS(NULL, cid, xpt, ypt, wpt, hpt, 1, 1, 1);
+    l_CIDataDestroy(&cid);
     if (!outstr)
         return ERROR_INT("outstr not made", procName, 1);
     nbytes = strlen(outstr);
 
-    if (l_binaryWrite(fileout, "w", outstr, nbytes))
-        return ERROR_INT("ps string not written to file", procName, 1);
+    ret = l_binaryWrite(fileout, "w", outstr, nbytes);
     LEPT_FREE(outstr);
-    l_CIDataDestroy(&cid);
-    return 0;
+    if (ret) L_ERROR("ps string not written to file\n", procName);
+    return ret;
 }
 
 
@@ -1521,7 +1518,7 @@ convertFlateToPSEmbed(const char  *filein,
                       const char  *fileout)
 {
 char         *outstr;
-l_int32       w, h, nbytes;
+l_int32       w, h, nbytes, ret;
 l_float32     xpt, ypt, wpt, hpt;
 L_COMP_DATA  *cid;
 
@@ -1552,15 +1549,15 @@ L_COMP_DATA  *cid;
         /* Generate the PS.
          * The bounding box information should be inserted (default). */
     outstr = generateFlatePS(NULL, cid, xpt, ypt, wpt, hpt, 1, 1);
+    l_CIDataDestroy(&cid);
     if (!outstr)
         return ERROR_INT("outstr not made", procName, 1);
     nbytes = strlen(outstr);
 
-    if (l_binaryWrite(fileout, "w", outstr, nbytes))
-        return ERROR_INT("ps string not written to file", procName, 1);
+    ret = l_binaryWrite(fileout, "w", outstr, nbytes);
     LEPT_FREE(outstr);
-    l_CIDataDestroy(&cid);
-    return 0;
+    if (ret) L_ERROR("ps string not written to file\n", procName);
+    return ret;
 }
 
 
@@ -1641,7 +1638,7 @@ convertFlateToPS(const char  *filein,
                  l_int32      endpage)
 {
 char    *outstr;
-l_int32  nbytes;
+l_int32  nbytes, ret;
 
     PROCNAME("convertFlateToPS");
 
@@ -1656,11 +1653,10 @@ l_int32  nbytes;
                                pageno, endpage))
         return ERROR_INT("ps string not made", procName, 1);
 
-    if (l_binaryWrite(fileout, operation, outstr, nbytes))
-        return ERROR_INT("ps string not written to file", procName, 1);
-
+    ret = l_binaryWrite(fileout, operation, outstr, nbytes);
     LEPT_FREE(outstr);
-    return 0;
+    if (ret) L_ERROR("ps string not written to file\n", procName);
+    return ret;
 }
 
 
