@@ -24,10 +24,12 @@
  -  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *====================================================================*/
 
-/*
- *  psio2stub.c
+/*!
+ * \file psio2stub.c
+ * <pre>
  *
  *     Stubs for psio2.c functions
+ * </pre>
  */
 
 #include "allheaders.h"
@@ -160,7 +162,7 @@ char * generateG4PS(const char *filein, L_COMP_DATA *cid, l_float32 xpt,
 /* ----------------------------------------------------------------------*/
 
 l_int32 convertTiffMultipageToPS(const char *filein, const char *fileout,
-                                 const char *tempfile, l_float32 fillfract)
+                                 l_float32 fillfract)
 {
     return ERROR_INT("function not present", "convertTiffMultipageToPS", 1);
 }
