@@ -29,15 +29,15 @@
  * <pre>
  *
  *      Top-level identification
- *         l_int32           recogDecode()
+ *         BOXA             *recogDecode()
  *
  *      Generate decoding arrays
- *         l_int32           recogMakeDecodingArrays()
+ *         static l_int32    recogPrepareForDecoding()
  *         static l_int32    recogMakeDecodingArray()
  *
  *      Dynamic programming for best path
- *         l_int32           recogRunViterbi()
- *         l_int32           recogRescoreDidResult()
+ *         static l_int32    recogRunViterbi()
+ *         static l_int32    recogRescoreDidResult()
  *         static PIX       *recogShowPath()
  *
  *      Create/destroy temporary DID data
@@ -63,7 +63,7 @@
  *  an image from a message, and the MAP message is derived from the
  *  observed image using Bayes' theorem.  This approach can also be used
  *  to build the model, using the iterative expectation/maximization
- *  method from labelled but errorful data.
+ *  method from labeled but errorful data.
  *
  *  In a little more detail: The model comprises three things: the ideal
  *  printed character templates, the independent bit-flip noise model, and
@@ -157,8 +157,11 @@
 #include <math.h>
 #include "allheaders.h"
 
+static l_int32 recogPrepareForDecoding(L_RECOG *recog, PIX *pixs,
+                                       l_int32 debug);
 static l_int32 recogMakeDecodingArray(L_RECOG *recog, l_int32 index,
                                       l_int32 debug);
+static l_int32 recogRunViterbi(L_RECOG *recog, PIX **ppixdb);
 static l_int32 recogRescoreDidResult(L_RECOG *recog, PIX **ppixdb);
 static PIX *recogShowPath(L_RECOG *recog, l_int32 select);
 static l_int32 recogGetWindowedArea(L_RECOG *recog, l_int32 index,
@@ -189,49 +192,74 @@ static const l_float32  DefaultAlpha4[] = {0.95f, 0.9f, 0.75f, 0.25f};
  * \param[in]    pixs typically of multiple touching characters, 1 bpp
  * \param[in]    nlevels of templates; 2 for now
  * \param[out]   ppixdb [optional] debug result; can be null
- * \return  0 if OK, 1 on error
+ * \return  boxa  segmentation of pixs into characters, or NULL on error
+ *
+ * <pre>
+ * Notes:
+ *      (1) The input pixs has been filtered so that it is likely to be
+ *          composed of more than one touching character.  Specifically,
+ *          its height can only slightly exceed that of the tallest
+ *          unscaled template, the width is somewhat larger than the
+ *          width of the widest unscaled template, and the w/h aspect ratio
+ *          is bounded by max_wh_ratio.
+ *      (2) This uses the DID mechanism with labeled templates to
+ *          segment the input %pixs.  The resulting segmentation is
+ *          returned.  (It is given by did->boxa).
+ *      (3) In debug mode, the Viterbi path is rescored based on all
+ *          the templates.  In non-debug mode, the same procedure is
+ *          carried out by recogIdentifyPix() on the result of the
+ *          segmentation.
+ * </pre>
  */
-l_int32
+BOXA  *
 recogDecode(L_RECOG  *recog,
             PIX      *pixs,
             l_int32   nlevels,
             PIX     **ppixdb)
 {
 l_int32  debug;
-PIX     *pixt;
+PIX     *pix1;
 PIXA    *pixa;
 
     PROCNAME("recogDecode");
 
     if (ppixdb) *ppixdb = NULL;
     if (!recog)
-        return ERROR_INT("recog not defined", procName, 1);
+        return (BOXA *)ERROR_PTR("recog not defined", procName, NULL);
     if (!pixs || pixGetDepth(pixs) != 1)
-        return ERROR_INT("pixs not defined or not 1 bpp", procName, 1);
+        return (BOXA *)ERROR_PTR("pixs undefined or not 1 bpp", procName, NULL);
     if (!recog->train_done)
-        return ERROR_INT("training not finished", procName, 1);
+        return (BOXA *)ERROR_PTR("training not finished", procName, NULL);
     if (nlevels != 2)
-        return ERROR_INT("nlevels != 2 (for now)", procName, 1);
+        return (BOXA *)ERROR_PTR("nlevels != 2 (for now)", procName, NULL);
 
-    pixa = (ppixdb) ? pixaCreate(2) : NULL;
     debug = (ppixdb) ? 1 : 0;
-    if (recogMakeDecodingArrays(recog, pixs, debug))
-        return ERROR_INT("error making arrays", procName, 1);
-
+    if (recogPrepareForDecoding(recog, pixs, debug))
+        return (BOXA *)ERROR_PTR("error making arrays", procName, NULL);
     recogSetChannelParams(recog, nlevels);
 
-    if (recogRunViterbi(recog, &pixt))
-        return ERROR_INT("error in viterbi", procName, 1);
-    if (ppixdb) pixaAddPix(pixa, pixt, L_INSERT);
+        /* Normal path; just run Viterbi */
+    if (!debug) {
+        if (recogRunViterbi(recog, NULL) == 0)
+            return boxaCopy(recog->did->boxa, L_COPY);
+        else
+            return (BOXA *)ERROR_PTR("error in Viterbi", procName, NULL);
+    }
 
-    if (recogRescoreDidResult(recog, &pixt))
-        return ERROR_INT("error in rescoring", procName, 1);
-    if (ppixdb) pixaAddPix(pixa, pixt, L_INSERT);
-
-    *ppixdb = pixaDisplayTiledInRows(pixa, 32, 2 * pixGetWidth(pixt) + 100,
+        /* Debug path */
+    if (recogRunViterbi(recog, &pix1))
+        return (BOXA *)ERROR_PTR("error in viterbi", procName, NULL);
+    pixa = pixaCreate(2);
+    pixaAddPix(pixa, pix1, L_INSERT);
+    if (recogRescoreDidResult(recog, &pix1)) {
+        pixaDestroy(&pixa);
+        return (BOXA *)ERROR_PTR("error in rescoring", procName, NULL);
+    }
+    pixaAddPix(pixa, pix1, L_INSERT);
+    *ppixdb = pixaDisplayTiledInRows(pixa, 32, 2 * pixGetWidth(pix1) + 100,
                                      1.0, 0, 30, 2);
     pixaDestroy(&pixa);
-    return 0;
+    return boxaCopy(recog->did->boxa, L_COPY);
 }
 
 
@@ -239,7 +267,7 @@ PIXA    *pixa;
  *                       Generate decoding arrays                         *
  *------------------------------------------------------------------------*/
 /*!
- * \brief   recogMakeDecodingArrays()
+ * \brief   recogPrepareForDecoding()
  *
  * \param[in]    recog with LUT's pre-computed
  * \param[in]    pixs typically of multiple touching characters, 1 bpp
@@ -248,17 +276,19 @@ PIXA    *pixa;
  *
  * <pre>
  * Notes:
- *      (1) Generates the bit-and sum arrays for each character template
- *          along pixs.  These are used in the dynamic programming step.
- *      (2) Previous arrays are destroyed and the new arrays are allocated.
- *      (3) The values are saved in the scoring arrays at the left edge
- *          of the template.  They are used in the viterbi process
+ *      (1) Binarizes and crops input %pixs.
+ *      (2) Removes previous L_RDID struct and makes a new one.
+ *      (3) Generates the bit-and sum arrays for each character template
+ *          at each pixel position in %pixs.  These are used in the
+ *          Viterbi dynamic programming step.
+ *      (4) The values are saved in the scoring arrays at the left edge
+ *          of the template.  They are used in the Viterbi process
  *          at the setwidth position (which is near the RHS of the template
  *          as it is positioned on pixs) in the generated trellis.
  * </pre>
  */
-l_int32
-recogMakeDecodingArrays(L_RECOG  *recog,
+static l_int32
+recogPrepareForDecoding(L_RECOG  *recog,
                         PIX      *pixs,
                         l_int32   debug)
 {
@@ -266,7 +296,7 @@ l_int32  i;
 PIX     *pix1;
 L_RDID  *did;
 
-    PROCNAME("recogMakeDecodingArrays");
+    PROCNAME("recogPrepareForDecoding");
 
     if (!recog)
         return ERROR_INT("recog not defined", procName, 1);
@@ -275,6 +305,9 @@ L_RDID  *did;
     if (!recog->train_done)
         return ERROR_INT("training not finished", procName, 1);
 
+    if (!recog->ave_done)
+        recogAverageSamples(&recog, 0);
+
         /* Binarize and crop to foreground if necessary */
     if ((pix1 = recogProcessToIdentify(recog, pixs, 0)) == NULL)
         return ERROR_INT("pix1 not made", procName, 1);
@@ -383,6 +416,7 @@ L_RDID   *did;
     for (i = 0; i < nx; i++) {
         shifty = (l_int32)(ycent1[i] - ycent2 + 0.5);
         maxcount = 0;
+        maxdely = 0;
         for (j = -MaxYShift; j <= MaxYShift; j++) {
             pixClearAll(pix3);
             dely = shifty + j;  /* amount pix2 is shifted relative to pix1 */
@@ -420,7 +454,10 @@ L_RDID   *did;
  *
  * <pre>
  * Notes:
- *      (1) This is recursive, in that
+ *      (1) This can be used when the templates are unscaled.  It works by
+ *          matching the average, unscaled templates of each class to
+ *          all positions.
+ *      (2) It is recursive, in that
  *          (a) we compute the score successively at all pixel positions x,
  *          (b) to compute the score at x in the trellis, for each
  *              template we look backwards to (x - setwidth) to get the
@@ -429,19 +466,26 @@ L_RDID   *did;
  *              score that maximizes the sum of the score at (x - setwidth)
  *              and the log-likelihood for the template to be printed with
  *              its LHS there.
+ *      (3) The primary output is a boxa of the locations for splitting
+ *          the input image.  These locations are used later to split the
+ *          image and send the pieces individually for recognition.
+ *          This can be done in either recogIdentifyMultiple(), or
+ *          for debugging in recogRescoreDidResult().
  * </pre>
  */
-l_int32
+static l_int32
 recogRunViterbi(L_RECOG  *recog,
                 PIX     **ppixdb)
 {
-l_int32     i, w1, x, narray, minsetw, first, templ, xloc, dely, counts, area1;
+l_int32     i, w1, w2, h1, xnz, x, narray, minsetw;
+l_int32     first, templ, xloc, dely, counts, area1;
 l_int32     besttempl, spacetempl;
 l_int32    *setw, *didtempl;
 l_int32    *area2;  /* must be freed */
 l_float32   prevscore, matchscore, maxscore, correl;
 l_float32  *didscore;
-PIX        *pixt;
+BOX        *box;
+PIX        *pix1;
 L_RDID     *did;
 
     PROCNAME("recogRunViterbi");
@@ -454,11 +498,8 @@ L_RDID     *did;
     if (did->fullarrays == 0)
         return ERROR_INT("did full arrays not made", procName, 1);
 
-        /* The score array is initialized to 0.0.  As we proceed to
-         * the left, the log likelihood for the partial paths goes
-         * negative, and we prune for the max (least negative) path.
-         * No matches will be computed until we reach x = min(setwidth);
-         * until then first == TRUE after looping over templates. */
+        /* Compute the minimum setwidth. Bad templates with very small
+         * width can cause havoc because the setwidth is too small. */
     w1 = did->size;
     narray = did->narray;
     spacetempl = narray;
@@ -468,10 +509,14 @@ L_RDID     *did;
         if (setw[i] < minsetw)
             minsetw = setw[i];
     }
-    if (minsetw <= 0) {
-        L_ERROR("minsetw <= 0; shouldn't happen\n", procName);
-        minsetw = 1;
-    }
+    if (minsetw <= 2)
+        return ERROR_INT("minsetw <= 2; bad templates", procName, 1);
+
+        /* The score array is initialized to 0.0.  As we proceed to
+         * the left, the log likelihood for the partial paths goes
+         * negative, and we prune for the max (least negative) path.
+         * No matches will be computed until we reach x = min(setwidth);
+         * until then first == TRUE after looping over templates. */
     didscore = did->trellisscore;
     didtempl = did->trellistempl;
     area2 = numaGetIArray(recog->nasum_u);
@@ -510,6 +555,7 @@ L_RDID     *did;
     for (x = w1 - 1; x >= 0; x--) {
         if (didtempl[x] != spacetempl) break;
     }
+    h1 = pixGetHeight(did->pixs);
     while (x > 0) {
         if (didtempl[x] == spacetempl) {  /* skip over spaces */
             x--;
@@ -520,14 +566,19 @@ L_RDID     *did;
         if (xloc < 0) break;
         counts = did->counta[templ][xloc];  /* bit-and counts */
         recogGetWindowedArea(recog, templ, xloc, &dely, &area1);
-        correl = (counts * counts) / (l_float32)(area2[templ] * area1);
-        pixt = pixaGetPix(recog->pixa_u, templ, L_CLONE);
+        correl = ((l_float32)(counts) * counts) /
+                  (l_float32)(area2[templ] * area1);
+        pix1 = pixaGetPix(recog->pixa_u, templ, L_CLONE);
+        w2 = pixGetWidth(pix1);
         numaAddNumber(did->natempl, templ);
         numaAddNumber(did->naxloc, xloc);
         numaAddNumber(did->nadely, dely);
-        numaAddNumber(did->nawidth, pixGetWidth(pixt));
+        numaAddNumber(did->nawidth, pixGetWidth(pix1));
         numaAddNumber(did->nascore, correl);
-        pixDestroy(&pixt);
+        xnz = L_MAX(xloc, 0);
+        box = boxCreate(xnz, dely, w2, h1);
+        boxaAddBox(did->boxa, box, L_INSERT);
+        pixDestroy(&pix1);
         x = xloc;
     }
 
@@ -537,6 +588,7 @@ L_RDID     *did;
         numaWriteStream(stderr, did->nadely);
         numaWriteStream(stderr, did->nawidth);
         numaWriteStream(stderr, did->nascore);
+        boxaWriteStream(stderr, did->boxa);
         *ppixdb = recogShowPath(recog, 0);
     }
 
@@ -554,15 +606,15 @@ L_RDID     *did;
  *
  * <pre>
  * Notes:
- *      (1) This does correlation matching with all templates using the
- *          viterbi path segmentation.
+ *      (1) This does correlation matching with all unscaled templates,
+ *          using the character segmentation determined by the Viterbi path.
  * </pre>
  */
 static l_int32
 recogRescoreDidResult(L_RECOG  *recog,
                       PIX     **ppixdb)
 {
-l_int32    i, n, w2, h1, templ, x, xloc, dely, index;
+l_int32    i, n, sample, x, dely, index;
 char      *text;
 l_float32  score;
 BOX       *box1;
@@ -582,31 +634,23 @@ L_RDID    *did;
         return ERROR_INT("no elements in path", procName, 1);
 
     pixs = did->pixs;
-    h1 = pixGetHeight(pixs);
     for (i = 0; i < n; i++) {
-        numaGetIValue(did->natempl, i, &templ);
-        numaGetIValue(did->naxloc, i, &xloc);
-        numaGetIValue(did->nadely, i, &dely);
-        pixaGetPixDimensions(recog->pixa_u, templ, &w2, NULL, NULL);
-        /* TODO: try to fix xloc - 4, etc. */
-        x = L_MAX(xloc, 0);
-        box1 = boxCreate(x, dely, w2, h1);
+        box1 = boxaGetBox(did->boxa, i, L_COPY);
+        boxGetGeometry(box1, &x, &dely, NULL, NULL);
         pix1 = pixClipRectangle(pixs, box1, NULL);
         recogIdentifyPix(recog, pix1, NULL);
         recogTransferRchToDid(recog, x, dely);
         if (ppixdb) {
             rchExtract(recog->rch, &index, &score, &text,
-                       NULL, NULL, NULL, NULL);
-            fprintf(stderr, "text = %s, index = %d, score = %5.3f\n",
-                    text, index, score);
+                       &sample, NULL, NULL, NULL);
+            fprintf(stderr, "text = %s, index = %d, sample = %d,"
+                    " score = %5.3f\n", text, index, sample, score);
         }
         pixDestroy(&pix1);
         boxDestroy(&box1);
         LEPT_FREE(text);
     }
 
-/*    numaWriteStream(stderr, recog->did->nadely_r);  */
-
     if (ppixdb)
         *ppixdb = recogShowPath(recog, 1);
 
@@ -626,10 +670,10 @@ recogShowPath(L_RECOG  *recog,
               l_int32   select)
 {
 char       textstr[16];
-l_int32    i, n, index, xloc, dely;
+l_int32    i, j, n, index, xloc, dely;
 l_float32  score;
 L_BMF     *bmf;
-NUMA      *natempl_s, *nascore_s, *naxloc_s, *nadely_s;
+NUMA      *natempl_s, *nasample_s, *nascore_s, *naxloc_s, *nadely_s;
 PIX       *pixs, *pix0, *pix1, *pix2, *pix3, *pix4, *pix5;
 L_RDID    *did;
 
@@ -651,6 +695,7 @@ L_RDID    *did;
         nadely_s = did->nadely;
     } else {  /* rescored */
         natempl_s = did->natempl_r;
+        nasample_s = did->nasample_r;
         nascore_s = did->nascore_r;
         naxloc_s = did->naxloc_r;
         nadely_s = did->nadely_r;
@@ -659,7 +704,12 @@ L_RDID    *did;
     n = numaGetCount(natempl_s);
     for (i = 0; i < n; i++) {
         numaGetIValue(natempl_s, i, &index);
-        pix2 = pixaGetPix(recog->pixa_u, index, L_CLONE);
+        if (select == 0) {
+            pix2 = pixaGetPix(recog->pixa_u, index, L_CLONE);
+        } else {
+            numaGetIValue(nasample_s, i, &j);
+            pix2 = pixaaGetPix(recog->pixaa_u, index, j, L_CLONE);
+        }
         pix3 = pixScale(pix2, 4.0, 4.0);
         pix4 = pixErodeBrick(NULL, pix3, 5, 5);
         pixXor(pix4, pix4, pix3);
@@ -695,8 +745,8 @@ l_int32
 recogCreateDid(L_RECOG  *recog,
                PIX      *pixs)
 {
-l_int32      i;
-PIX         *pixt;
+l_int32  i;
+PIX     *pix1;
 L_RDID  *did;
 
     PROCNAME("recogCreateDid");
@@ -717,8 +767,10 @@ L_RDID  *did;
     did->naxloc = numaCreate(5);
     did->nadely = numaCreate(5);
     did->nawidth = numaCreate(5);
+    did->boxa = boxaCreate(5);
     did->nascore = numaCreate(5);
     did->natempl_r = numaCreate(5);
+    did->nasample_r = numaCreate(5);
     did->naxloc_r = numaCreate(5);
     did->nadely_r = numaCreate(5);
     did->nawidth_r = numaCreate(5);
@@ -739,9 +791,9 @@ L_RDID  *did;
 
         /* Populate the setwidth array */
     for (i = 0; i < did->narray; i++) {
-        pixt = pixaGetPix(recog->pixa_u, i, L_CLONE);
-        did->setwidth[i] = (l_int32)(SetwidthFraction * pixGetWidth(pixt));
-        pixDestroy(&pixt);
+        pix1 = pixaGetPix(recog->pixa_u, i, L_CLONE);
+        did->setwidth[i] = (l_int32)(SetwidthFraction * pixGetWidth(pix1));
+        pixDestroy(&pix1);
     }
 
     return 0;
@@ -793,8 +845,10 @@ L_RDID  *did;
     numaDestroy(&did->naxloc);
     numaDestroy(&did->nadely);
     numaDestroy(&did->nawidth);
+    boxaDestroy(&did->boxa);
     numaDestroy(&did->nascore);
     numaDestroy(&did->natempl_r);
+    numaDestroy(&did->nasample_r);
     numaDestroy(&did->naxloc_r);
     numaDestroy(&did->nadely_r);
     numaDestroy(&did->nawidth_r);
@@ -876,8 +930,8 @@ L_RDID  *did;
  *      (1) This is called after the best path has been found through
  *          the trellis, in order to produce a correlation that can be used
  *          to evaluate the confidence we have in the identification.
- *          The correlation is |1 \& 2|^2 / (|1| * |2|).
- *          |1 \& 2| is given by the count array, |2| is found from
+ *          The correlation is |1 & 2|^2 / (|1| * |2|).
+ *          |1 & 2| is given by the count array, |2| is found from
  *          nasum_u[], and |1| is wsum returned from this function.
  * </pre>
  */
@@ -969,8 +1023,8 @@ L_RDID           *did;
     for (i = 1; i < nlevels; i++) {
         did->beta[i] = log((1.0 - da[i]) / da[0]);
         did->gamma[i] = log(da[0] * da[i] / ((1.0 - da[0]) * (1.0 - da[i])));
-        fprintf(stderr, "beta[%d] = %7.3f, gamma[%d] = %7.3f\n",
-                i, did->beta[i], i, did->gamma[i]);
+/*        fprintf(stderr, "beta[%d] = %7.3f, gamma[%d] = %7.3f\n",
+                i, did->beta[i], i, did->gamma[i]);  */
     }
 
     return 0;
@@ -1009,6 +1063,7 @@ L_RCH   *rch;
         return ERROR_INT("rch not defined", procName, 1);
 
     numaAddNumber(did->natempl_r, rch->index);
+    numaAddNumber(did->nasample_r, rch->sample);
     numaAddNumber(did->naxloc_r, rch->xloc + x);
     numaAddNumber(did->nadely_r, rch->yloc + y);
     numaAddNumber(did->nawidth_r, rch->width);
