@@ -67,9 +67,6 @@
  *         PIX                *recogShowMatchesInRange()
  *         PIX                *recogShowMatch()
  *
- *      Static helper
- *         static char        *l_charToString()
- *
  *  These abbreviations are for the type of template to be used:
  *    * SI (for the scanned images)
  *    * WNL (for width-normalized lines, formed by first skeletonizing
@@ -172,7 +169,6 @@ static l_int32 recogCharsetAvailable(l_int32 type);
 static PIX *pixDisplayOutliers(PIXA *pixas, NUMA *nas);
 static PIX *recogDisplayOutlier(L_RECOG *recog, l_int32 iclass, l_int32 jsamp,
                                 l_int32 maxclass, l_float32 maxscore);
-static char *l_charToString(char byte);
 
     /* Default parameters that are used in recogTemplatesAreOK() and
      * in outlier removal functions, and that use template set size
@@ -358,7 +354,7 @@ recogAddSample(L_RECOG  *recog,
                l_int32   debug)
 {
 char    *text;
-l_int32  i, n, npa, charint, index;
+l_int32  npa, charint, index;
 PIXA    *pixa1;
 PIXAA   *paa;
 
@@ -1496,7 +1492,7 @@ recogTrainFromBoot(L_RECOG   *recogboot,
                    l_int32    debug)
 {
 char      *text;
-l_int32    i, n, maxdepth, localboot, scaleh, linew;
+l_int32    i, n, same, maxd, scaleh, linew;
 l_float32  score;
 PIX       *pix1, *pix2, *pixdb;
 PIXA      *pixa1, *pixa2, *pixa3, *pixad;
@@ -1511,8 +1507,8 @@ PIXA      *pixa1, *pixa2, *pixa3, *pixad;
         /* Make sure all input pix are 1 bpp */
     if ((n = pixaGetCount(pixas)) == 0)
         return (PIXA *)ERROR_PTR("no pix in pixa", procName, NULL);
-    pixaVerifyDepth(pixas, &maxdepth);
-    if (maxdepth == 1) {
+    pixaVerifyDepth(pixas, &same, &maxd);
+    if (maxd == 1) {
         pixa1 = pixaCopy(pixas, L_COPY);
     } else {
         pixa1 = pixaCreate(n);
@@ -1658,7 +1654,6 @@ char      *str;
 l_int32    i, nt, min_nopad, nclass, allclasses;
 l_float32  minval;
 NUMA      *naclass;
-PIXAA     *paa;
 SARRAY    *sa;
 
     PROCNAME("recogIsPaddingNeeded");
@@ -1670,7 +1665,6 @@ SARRAY    *sa;
         return ERROR_INT("recog not defined", procName, 1);
 
         /* Do we have samples from all classes? */
-    paa = recog->pixaa_u;  /* unscaled bitmaps */
     nclass = pixaaGetCount(recog->pixaa_u, &naclass);  /* unscaled bitmaps */
     allclasses = (nclass == recog->charset_size) ? 1 : 0;
 
@@ -1716,9 +1710,8 @@ recogAddMissingClassStrings(L_RECOG  *recog)
 {
 char    *text;
 char     str[4];
-l_int32  i, nclass, index, ival, n;
+l_int32  i, nclass, index, ival;
 NUMA    *na;
-PIXAA   *paa;
 SARRAY  *sa;
 
     PROCNAME("recogAddMissingClassStrings");
@@ -1727,9 +1720,8 @@ SARRAY  *sa;
         return (SARRAY *)ERROR_PTR("recog not defined", procName, NULL);
 
         /* Only handling digits */
-    paa = recog->pixaa_u;  /* unscaled bitmaps */
     nclass = pixaaGetCount(recog->pixaa_u, NULL);  /* unscaled bitmaps */
-    if (recog->charset_type != 1 || (recog->charset_type == 1 && nclass == 10))
+    if (recog->charset_type != 1 || nclass == 10)
         return sarrayCreate(0);  /* empty */
 
         /* Make an indicator array for missing classes */
@@ -1890,8 +1882,6 @@ recogMakeBootDigitRecog(l_int32  scaleh,
 PIXA     *pixa;
 L_RECOG  *recog;
 
-    PROCNAME("recogMakeBootDigitRecog");
-
         /* Get the templates, extended by horizontal scaling */
     pixa = recogMakeBootDigitTemplates(debug);
 
@@ -1924,8 +1914,6 @@ NUMA  *na;
 PIX   *pix1, *pix2, *pix3;
 PIXA  *pixa1, *pixa2, *pixa3;
 
-    PROCNAME("recogMakeBootDigitTemplates");
-
     pixa1 = l_bootnum_gen1();
     pixa2 = l_bootnum_gen2();
     pixa3 = l_bootnum_gen3();
@@ -2467,17 +2455,3 @@ PIXA   *pixa;
 
     return pixd;
 }
-
-
-/*------------------------------------------------------------------------*
- *                             Static helper                              *
- *------------------------------------------------------------------------*/
-static char *
-l_charToString(char byte)
-{
-char  *str;
-
-  str = (char *)LEPT_CALLOC(2, sizeof(char));
-  str[0] = byte;
-  return str;
-}
