@@ -1,20 +1,32 @@
 /*====================================================================*
  -  Copyright (C) 2001 Leptonica.  All rights reserved.
- -  This software is distributed in the hope that it will be
- -  useful, but with NO WARRANTY OF ANY KIND.
- -  No author or distributor accepts responsibility to anyone for the
- -  consequences of using this software, or for whether it serves any
- -  particular purpose or works at all, unless he or she says so in
- -  writing.  Everyone is granted permission to copy, modify and
- -  redistribute this source code, for commercial or non-commercial
- -  purposes, with the following restrictions: (1) the origin of this
- -  source code must not be misrepresented; (2) modified versions must
- -  be plainly marked as such; and (3) this notice may not be removed
- -  or altered from any source or modified source distribution.
+ -
+ -  Redistribution and use in source and binary forms, with or without
+ -  modification, are permitted provided that the following conditions
+ -  are met:
+ -  1. Redistributions of source code must retain the above copyright
+ -     notice, this list of conditions and the following disclaimer.
+ -  2. Redistributions in binary form must reproduce the above
+ -     copyright notice, this list of conditions and the following
+ -     disclaimer in the documentation and/or other materials
+ -     provided with the distribution.
+ -
+ -  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ -  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ -  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ -  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL ANY
+ -  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+ -  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+ -  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+ -  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
+ -  OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ -  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ -  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *====================================================================*/
 
-/*
- *  rotateshear.c
+/*!
+ * \file rotateshear.c
+ * <pre>
  *
  *      Shear rotation about arbitrary point using 2 and 3 shears
  *
@@ -31,7 +43,7 @@
  *
  *  Rotation is measured in radians; clockwise rotations are positive.
  *
- *  Rotation by shear works on images of any depth, 
+ *  Rotation by shear works on images of any depth,
  *  including 8 bpp color paletted images and 32 bpp
  *  rgb images.  It works by translating each src pixel
  *  value to the appropriate pixel in the rotated dest.
@@ -41,9 +53,9 @@
  *  This speed and flexibility comes at the following cost,
  *  relative to area-mapped rotation:
  *
- *    -  Jaggies are created on edges of straight lines
+ *    ~  Jaggies are created on edges of straight lines
  *
- *    -  For large angles, where you must use 3 shears,
+ *    ~  For large angles, where you must use 3 shears,
  *       there is some extra clipping from the shears.
  *
  *  For small angles, typically less than 0.05 radians,
@@ -52,7 +64,7 @@
  *  shears on a pixel lattice that we have here) give
  *  a rotated image that has a distortion in the lengths
  *  of the two rotated and still-perpendicular axes.  The
- *  length/width ratio changes by a fraction 
+ *  length/width ratio changes by a fraction
  *
  *       0.5 * (angle)**2
  *
@@ -89,8 +101,8 @@
  *  does not work well for large rotation angles.  In fact, for
  *  rotation angles greater than about 7 degrees, more pixels are
  *  lost at the edges than when using pixRotationBySampling(), which
- *  only loses pixels because they are rotated out of the image. 
- *  For large rotations, use pixRotationBySampling() or, for
+ *  only loses pixels because they are rotated out of the image.
+ *  For larger rotations, use pixRotationBySampling() or, for
  *  more accuracy when d > 1 bpp, pixRotateAM().
  *
  *  For small angles, when comparing the quality of rotation by
@@ -99,7 +111,7 @@
  *  accuracy of rotation by sampling when compared to 3-shear and
  *  (for angles less than 2 degrees, when compared to 2-shear) is
  *  less than 1 pixel at any point.  For very small angles, rotation by
- *  sampling is slower than rotation by shear.  The speed difference
+ *  sampling is much slower than rotation by shear.  The speed difference
  *  depends on the pixel depth and the rotation angle.  Rotation
  *  by shear is very fast for small angles and for small depth (esp. 1 bpp).
  *  Rotation by sampling speed is independent of angle and relatively
@@ -112,8 +124,15 @@
  *          8                   5                  2.6
  *          32                  1.6                1.0
  *
- *  Consequently, for small angles and low bit depth, use rotation by shear.
- *  For large angles or large bit depth, use rotation by sampling.
+ *  In summary:
+ *    * For d == 1 and small angles, use rotation by shear.  By default
+ *      this will use 2-shear rotations, because 3-shears cause more
+ *      visible artifacts in straight lines and, for small angles, the
+ *      distortion in asperity ratio is small.
+ *    * For d > 1, shear is faster than sampling, which is faster than
+ *      area mapping.  However, area mapping gives the best results.
+ *  These results are used in selecting the rotation methods in
+ *  pixRotateShear().
  *
  *  There has been some work on what is called a "quasishear
  *  rotation" ("The Quasi-Shear Rotation, Eric Andres,
@@ -123,48 +142,55 @@
  *  a square pixel lattice.  They also use integers to specify
  *  the rotation angle and center offset, but that makes
  *  little sense on a machine where you have a few GFLOPS
- *  and only a few hundred floating point operations to do (!) 
+ *  and only a few hundred floating point operations to do (!)
  *  They also allow subpixel specification of the center of
  *  rotation, which I haven't bothered with, and claim that
  *  better results are possible if each of the 4 quadrants is
  *  handled separately.
- * 
- *  But the bottom line is that for binary images, the quality
- *  of the simple 3-shear rotation is about as good as you can do,
- *  visually, without dithering the result.  The effect of dither
- *  is to break up the horizontal and vertical shear lines.
- *  It's a bit tricky to dither with block shears -- you have to
- *  dither the pixels on the block boundaries!
+ *
+ *  But the bottom line is that you are going to see shear lines when
+ *  you rotate 1 bpp images.  Although the 3-shear rotation is
+ *  mathematically exact in the limit of infinitesimal pixels, artifacts
+ *  will be evident in real images.  One might imagine using dithering
+ *  to break up the horizontal and vertical shear lines, but this
+ *  is hard with block shears, where you need to dither on the block
+ *  boundaries.  Dithering (by accumulation of 'error') with sampling
+ *  makes more sense, but I haven't tried to do this.  There is only
+ *  so much you can do with 1 bpp images!
+ * </pre>
  */
 
-#include <stdio.h>
-#include <string.h>
-#include <stdlib.h>
 #include <math.h>
+#include <string.h>
 #include "allheaders.h"
 
-static const l_float32  VERY_SMALL_ANGLE = 0.001;  /* radians; ~0.06 degrees */
-static const l_float32  MAX_2_SHEAR_ANGLE = 0.05;  /* radians; ~3 degrees    */
-
+static const l_float32  MIN_ANGLE_TO_ROTATE = 0.001;  /* radians; ~0.06 deg */
+static const l_float32  MAX_2_SHEAR_ANGLE = 0.06;     /* radians; ~3 deg    */
+static const l_float32  LIMIT_SHEAR_ANGLE = 0.35;     /* radians; ~20 deg   */
 
 /*------------------------------------------------------------------*
  *                Rotations about an arbitrary point                *
  *------------------------------------------------------------------*/
 /*!
- *  pixRotateShear()
+ * \brief   pixRotateShear()
  *
- *      Input:  pixs
- *              xcen (x value for which there is no horizontal shear)
- *              ycen (y value for which there is no vertical shear)
- *              angle (radians)
- *              incolor (L_BRING_IN_WHITE, L_BRING_IN_BLACK);
- *      Return: pixd, or null on error.
+ * \param[in]    pixs
+ * \param[in]    xcen x value for which there is no horizontal shear
+ * \param[in]    ycen y value for which there is no vertical shear
+ * \param[in]    angle radians
+ * \param[in]    incolor L_BRING_IN_WHITE, L_BRING_IN_BLACK;
+ * \return  pixd, or NULL on error.
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) This rotates an image about the given point, using
  *          either 2 or 3 shears.
  *      (2) A positive angle gives a clockwise rotation.
  *      (3) This brings in 'incolor' pixels from outside the image.
+ *      (4) For rotation angles larger than about 0.35 radians, we issue
+ *          a warning because you should probably be using another method
+ *          (either sampling or area mapping)
+ * </pre>
  */
 PIX *
 pixRotateShear(PIX       *pixs,
@@ -180,30 +206,33 @@ pixRotateShear(PIX       *pixs,
     if (incolor != L_BRING_IN_WHITE && incolor != L_BRING_IN_BLACK)
         return (PIX *)(PIX *)ERROR_PTR("invalid incolor value", procName, NULL);
 
-    if (L_ABS(angle) < VERY_SMALL_ANGLE)
+    if (L_ABS(angle) < MIN_ANGLE_TO_ROTATE)
         return pixClone(pixs);
 
     if (L_ABS(angle) <= MAX_2_SHEAR_ANGLE)
         return pixRotate2Shear(pixs, xcen, ycen, angle, incolor);
-    else
-        return pixRotate3Shear(pixs, xcen, ycen, angle, incolor);
 
+    if (L_ABS(angle) > LIMIT_SHEAR_ANGLE)
+        L_WARNING("%6.2f radians; large angle for shear rotation\n",
+                  procName, L_ABS(angle));
+    return pixRotate3Shear(pixs, xcen, ycen, angle, incolor);
 }
 
 
 /*!
- *  pixRotate2Shear()
+ * \brief   pixRotate2Shear()
  *
- *      Input:  pixs
- *              xcen, ycen (center of rotation)
- *              angle (radians)
- *              incolor (L_BRING_IN_WHITE, L_BRING_IN_BLACK);
- *      Return: pixd, or null on error.
+ * \param[in]    pixs
+ * \param[in]    xcen, ycen center of rotation
+ * \param[in]    angle radians
+ * \param[in]    incolor L_BRING_IN_WHITE, L_BRING_IN_BLACK;
+ * \return  pixd, or NULL on error.
  *
- *  Notes:
- *      (1) This rotates the image about the given point,
- *          using the 2-shear method.  It should only
- *          be used for angles smaller than MAX_2_SHEAR_ANGLE.
+ * <pre>
+ * Notes:
+ *      (1) This rotates the image about the given point, using the 2-shear
+ *          method.  It should only be used for angles smaller than
+ *          MAX_2_SHEAR_ANGLE.  For larger angles, a warning is issued.
  *      (2) A positive angle gives a clockwise rotation.
  *      (3) 2-shear rotation by a specified angle is equivalent
  *          to the sequential transformations
@@ -211,6 +240,9 @@ pixRotateShear(PIX       *pixs,
  *             y' = y + tan(angle) * (x - xcen)     for y-shear
  *      (4) Computation of tan(angle) is performed within the shear operation.
  *      (5) This brings in 'incolor' pixels from outside the image.
+ *      (6) If the image has an alpha layer, it is rotated separately by
+ *          two shears.
+ * </pre>
  */
 PIX *
 pixRotate2Shear(PIX       *pixs,
@@ -219,7 +251,7 @@ pixRotate2Shear(PIX       *pixs,
                 l_float32  angle,
                 l_int32    incolor)
 {
-PIX  *pixt, *pixd;
+PIX  *pix1, *pix2, *pixd;
 
     PROCNAME("pixRotate2Shear");
 
@@ -227,33 +259,45 @@ PIX  *pixt, *pixd;
         return (PIX *)ERROR_PTR("pixs not defined", procName, NULL);
     if (incolor != L_BRING_IN_WHITE && incolor != L_BRING_IN_BLACK)
         return (PIX *)(PIX *)ERROR_PTR("invalid incolor value", procName, NULL);
-    
-    if (L_ABS(angle) < VERY_SMALL_ANGLE)
+
+    if (L_ABS(angle) < MIN_ANGLE_TO_ROTATE)
         return pixClone(pixs);
+    if (L_ABS(angle) > MAX_2_SHEAR_ANGLE)
+        L_WARNING("%6.2f radians; large angle for 2-shear rotation\n",
+                  procName, L_ABS(angle));
 
-    if ((pixt = pixHShear(NULL, pixs, ycen, angle, incolor)) == NULL)
-        return (PIX *)ERROR_PTR("pixt not made", procName, NULL);
-    if ((pixd = pixVShear(NULL, pixt, xcen, angle, incolor)) == NULL)
+    if ((pix1 = pixHShear(NULL, pixs, ycen, angle, incolor)) == NULL)
+        return (PIX *)ERROR_PTR("pix1 not made", procName, NULL);
+    pixd = pixVShear(NULL, pix1, xcen, angle, incolor);
+    pixDestroy(&pix1);
+    if (!pixd)
         return (PIX *)ERROR_PTR("pixd not made", procName, NULL);
-    pixDestroy(&pixt);
 
+    if (pixGetDepth(pixs) == 32 && pixGetSpp(pixs) == 4) {
+        pix1 = pixGetRGBComponent(pixs, L_ALPHA_CHANNEL);
+            /* L_BRING_IN_WHITE brings in opaque for the alpha component */
+        pix2 = pixRotate2Shear(pix1, xcen, ycen, angle, L_BRING_IN_WHITE);
+        pixSetRGBComponent(pixd, pix2, L_ALPHA_CHANNEL);
+        pixDestroy(&pix1);
+        pixDestroy(&pix2);
+    }
     return pixd;
 }
 
-
 /*!
- *  pixRotate3Shear()
+ * \brief   pixRotate3Shear()
  *
- *      Input:  pixs
- *              xcen, ycen (center of rotation)
- *              angle (radians)
- *              incolor (L_BRING_IN_WHITE, L_BRING_IN_BLACK);
- *      Return: pixd, or null on error.
+ * \param[in]    pixs
+ * \param[in]    xcen, ycen center of rotation
+ * \param[in]    angle radians
+ * \param[in]    incolor L_BRING_IN_WHITE, L_BRING_IN_BLACK;
+ * \return  pixd, or NULL on error.
  *
- *  Notes:
- *      (1) This rotates the image about the image center,
- *          using the 3-shear method.  It can be used for any angle, and
- *          should be used for angles larger than MAX_2_SHEAR_ANGLE.
+ * <pre>
+ * Notes:
+ *      (1) This rotates the image about the given point, using the 3-shear
+ *          method.  It should only be used for angles smaller than
+ *          LIMIT_SHEAR_ANGLE.  For larger angles, a warning is issued.
  *      (2) A positive angle gives a clockwise rotation.
  *      (3) 3-shear rotation by a specified angle is equivalent
  *          to the sequential transformations
@@ -262,11 +306,14 @@ PIX  *pixt, *pixd;
  *            y' = y + tan(angle/2) * (x - xcen)     for second y-shear
  *      (4) Computation of tan(angle) is performed in the shear operations.
  *      (5) This brings in 'incolor' pixels from outside the image.
- *      (6) The algorithm was published by Alan Paeth: "A Fast Algorithm
+ *      (6) If the image has an alpha layer, it is rotated separately by
+ *          two shears.
+ *      (7) The algorithm was published by Alan Paeth: "A Fast Algorithm
  *          for General Raster Rotation," Graphics Interface '86,
  *          pp. 77-81, May 1986.  A description of the method, along with
  *          an implementation, can be found in Graphics Gems, p. 179,
  *          edited by Andrew Glassner, published by Academic Press, 1990.
+ * </pre>
  */
 PIX *
 pixRotate3Shear(PIX       *pixs,
@@ -276,7 +323,7 @@ pixRotate3Shear(PIX       *pixs,
                 l_int32    incolor)
 {
 l_float32  hangle;
-PIX              *pixt, *pixd;
+PIX       *pix1, *pix2, *pixd;
 
     PROCNAME("pixRotate3Shear");
 
@@ -284,18 +331,32 @@ PIX              *pixt, *pixd;
         return (PIX *)ERROR_PTR("pixs not defined", procName, NULL);
     if (incolor != L_BRING_IN_WHITE && incolor != L_BRING_IN_BLACK)
         return (PIX *)(PIX *)ERROR_PTR("invalid incolor value", procName, NULL);
-    
-    if (L_ABS(angle) < VERY_SMALL_ANGLE)
+
+    if (L_ABS(angle) < MIN_ANGLE_TO_ROTATE)
         return pixClone(pixs);
+    if (L_ABS(angle) > LIMIT_SHEAR_ANGLE) {
+        L_WARNING("%6.2f radians; large angle for 3-shear rotation\n",
+                  procName, L_ABS(angle));
+    }
 
     hangle = atan(sin(angle));
     if ((pixd = pixVShear(NULL, pixs, xcen, angle / 2., incolor)) == NULL)
         return (PIX *)ERROR_PTR("pixd not made", procName, NULL);
-    if ((pixt = pixHShear(NULL, pixd, ycen, hangle, incolor)) == NULL)
-        return (PIX *)ERROR_PTR("pixt not made", procName, NULL);
-    pixVShear(pixd, pixt, xcen, angle / 2., incolor);
-    pixDestroy(&pixt);
+    if ((pix1 = pixHShear(NULL, pixd, ycen, hangle, incolor)) == NULL) {
+        pixDestroy(&pixd);
+        return (PIX *)ERROR_PTR("pix1 not made", procName, NULL);
+    }
+    pixVShear(pixd, pix1, xcen, angle / 2., incolor);
+    pixDestroy(&pix1);
 
+    if (pixGetDepth(pixs) == 32 && pixGetSpp(pixs) == 4) {
+        pix1 = pixGetRGBComponent(pixs, L_ALPHA_CHANNEL);
+            /* L_BRING_IN_WHITE brings in opaque for the alpha component */
+        pix2 = pixRotate3Shear(pix1, xcen, ycen, angle, L_BRING_IN_WHITE);
+        pixSetRGBComponent(pixd, pix2, L_ALPHA_CHANNEL);
+        pixDestroy(&pix1);
+        pixDestroy(&pix2);
+    }
     return pixd;
 }
 
@@ -304,17 +365,20 @@ PIX              *pixt, *pixd;
  *             Rotations in-place about an arbitrary point          *
  *------------------------------------------------------------------*/
 /*!
- *  pixRotateShearIP()
+ * \brief   pixRotateShearIP()
  *
- *      Input:  pixs (any depth; not colormapped)
- *              xcen, ycen (center of rotation)
- *              angle (radians)
- *              incolor (L_BRING_IN_WHITE, L_BRING_IN_BLACK)
- *      Return: 0 if OK; 1 on error
+ * \param[in]    pixs any depth; not colormapped
+ * \param[in]    xcen, ycen center of rotation
+ * \param[in]    angle radians
+ * \param[in]    incolor L_BRING_IN_WHITE, L_BRING_IN_BLACK
+ * \return  0 if OK; 1 on error
  *
- *  Notes:
- *      (1) This does an in-place rotation of the image
- *          about the image center, using the 3-shear method.
+ * <pre>
+ * Notes:
+ *      (1) This does an in-place rotation of the image about the
+ *          specified point, using the 3-shear method.  It should only
+ *          be used for angles smaller than LIMIT_SHEAR_ANGLE.
+ *          For larger angles, a warning is issued.
  *      (2) A positive angle gives a clockwise rotation.
  *      (3) 3-shear rotation by a specified angle is equivalent
  *          to the sequential transformations
@@ -325,6 +389,7 @@ PIX              *pixt, *pixd;
  *      (5) This brings in 'incolor' pixels from outside the image.
  *      (6) The pix cannot be colormapped, because the in-place operation
  *          only blits in 0 or 1 bits, not an arbitrary colormap index.
+ * </pre>
  */
 l_int32
 pixRotateShearIP(PIX       *pixs,
@@ -346,12 +411,15 @@ l_float32  hangle;
 
     if (angle == 0.0)
         return 0;
-    
+    if (L_ABS(angle) > LIMIT_SHEAR_ANGLE) {
+        L_WARNING("%6.2f radians; large angle for in-place 3-shear rotation\n",
+                  procName, L_ABS(angle));
+    }
+
     hangle = atan(sin(angle));
     pixHShearIP(pixs, ycen, angle / 2., incolor);
     pixVShearIP(pixs, xcen, hangle, incolor);
     pixHShearIP(pixs, ycen, angle / 2., incolor);
-
     return 0;
 }
 
@@ -360,12 +428,12 @@ l_float32  hangle;
  *                    Rotations about the image center              *
  *------------------------------------------------------------------*/
 /*!
- *  pixRotateShearCenter()
+ * \brief   pixRotateShearCenter()
  *
- *      Input:  pixs
- *              angle (radians)
- *              incolor (L_BRING_IN_WHITE, L_BRING_IN_BLACK)
- *      Return: pixd, or null on error
+ * \param[in]    pixs
+ * \param[in]    angle radians
+ * \param[in]    incolor L_BRING_IN_WHITE, L_BRING_IN_BLACK
+ * \return  pixd, or NULL on error
  */
 PIX *
 pixRotateShearCenter(PIX       *pixs,
@@ -376,19 +444,19 @@ pixRotateShearCenter(PIX       *pixs,
 
     if (!pixs)
         return (PIX *)ERROR_PTR("pixs not defined", procName, NULL);
-    
+
     return pixRotateShear(pixs, pixGetWidth(pixs) / 2,
                           pixGetHeight(pixs) / 2, angle, incolor);
 }
 
 
 /*!
- *  pixRotateShearCenterIP()
+ * \brief   pixRotateShearCenterIP()
  *
- *      Input:  pixs
- *              angle (radians)
- *              incolor (L_BRING_IN_WHITE, L_BRING_IN_BLACK)
- *      Return: 0 if OK, 1 on error
+ * \param[in]    pixs
+ * \param[in]    angle radians
+ * \param[in]    incolor L_BRING_IN_WHITE, L_BRING_IN_BLACK
+ * \return  0 if OK, 1 on error
  */
 l_int32
 pixRotateShearCenterIP(PIX       *pixs,
@@ -399,8 +467,7 @@ pixRotateShearCenterIP(PIX       *pixs,
 
     if (!pixs)
         return ERROR_INT("pixs not defined", procName, 1);
-    
+
     return pixRotateShearIP(pixs, pixGetWidth(pixs) / 2,
                             pixGetHeight(pixs) / 2, angle, incolor);
 }
-
