@@ -132,6 +132,7 @@
 #include <string.h>
 #ifndef _WIN32
 #include <dirent.h>     /* unix only */
+#include <sys/stat.h>
 #endif  /* ! _WIN32 */
 #include "allheaders.h"
 
@@ -304,6 +305,7 @@ SARRAY  *sa;
                     cstring[i - 1] = '\0';  /* also remove Windows CR */
                 if ((substring = stringNew(cstring + startptr)) == NULL) {
                     sarrayDestroy(&sa);
+                    LEPT_FREE(cstring);
                     return (SARRAY *)ERROR_PTR("substring not made",
                                                 procName, NULL);
                 }
@@ -315,6 +317,7 @@ SARRAY  *sa;
         if (startptr < size) {  /* no newline at end of last line */
             if ((substring = stringNew(cstring + startptr)) == NULL) {
                 sarrayDestroy(&sa);
+                LEPT_FREE(cstring);
                 return (SARRAY *)ERROR_PTR("substring not made",
                                            procName, NULL);
             }
@@ -1852,6 +1855,8 @@ l_int32         len;
 SARRAY         *safiles;
 DIR            *pdir;
 struct dirent  *pdirentry;
+int             dfd;
+struct stat     st;
 
     PROCNAME("getFilenamesInDirectory");
 
@@ -1864,15 +1869,14 @@ struct dirent  *pdirentry;
     if (!pdir)
         return (SARRAY *)ERROR_PTR("pdir not opened", procName, NULL);
     safiles = sarrayCreate(0);
+    dfd = dirfd(pdir);
     while ((pdirentry = readdir(pdir))) {
 
-        /* It's nice to ignore directories.  For this it is necessary to
-         * define _BSD_SOURCE in the CC command, because the DT_DIR
-         * flag is non-standard.  */
-#if !defined(__SOLARIS__)
-        if (pdirentry->d_type == DT_DIR)
+        /* It's nice to ignore directories.  */
+      if ((0 == fstatat(dfd, pdirentry->d_name, &st, 0))
+          && S_ISDIR(st.st_mode)) {
             continue;
-#endif
+      }
 
             /* Filter out "." and ".." if they're passed through */
         name = pdirentry->d_name;
