@@ -1,21 +1,33 @@
 /*====================================================================*
  -  Copyright (C) 2001 Leptonica.  All rights reserved.
- -  This software is distributed in the hope that it will be
- -  useful, but with NO WARRANTY OF ANY KIND.
- -  No author or distributor accepts responsibility to anyone for the
- -  consequences of using this software, or for whether it serves any
- -  particular purpose or works at all, unless he or she says so in
- -  writing.  Everyone is granted permission to copy, modify and
- -  redistribute this source code, for commercial or non-commercial
- -  purposes, with the following restrictions: (1) the origin of this
- -  source code must not be misrepresented; (2) modified versions must
- -  be plainly marked as such; and (3) this notice may not be removed
- -  or altered from any source or modified source distribution.
+ -
+ -  Redistribution and use in source and binary forms, with or without
+ -  modification, are permitted provided that the following conditions
+ -  are met:
+ -  1. Redistributions of source code must retain the above copyright
+ -     notice, this list of conditions and the following disclaimer.
+ -  2. Redistributions in binary form must reproduce the above
+ -     copyright notice, this list of conditions and the following
+ -     disclaimer in the documentation and/or other materials
+ -     provided with the distribution.
+ -
+ -  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ -  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ -  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ -  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL ANY
+ -  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+ -  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+ -  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+ -  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
+ -  OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ -  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ -  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *====================================================================*/
 
 
-/*
- *  shear.c
+/*!
+ * \file shear.c
+ * <pre>
  *
  *    About arbitrary lines
  *           PIX      *pixHShear()
@@ -37,10 +49,9 @@
  *
  *    Static helper
  *      static l_float32  normalizeAngleForShear()
+ * </pre>
  */
 
-#include <stdio.h>
-#include <stdlib.h>
 #include <string.h>
 #include <math.h>
 #include "allheaders.h"
@@ -60,17 +71,18 @@ static l_float32 normalizeAngleForShear(l_float32 radang, l_float32 mindif);
  *                    About arbitrary lines                    *
  *-------------------------------------------------------------*/
 /*!
- *  pixHShear()
+ * \brief   pixHShear()
  *
- *      Input:  pixd (<optional>, this can be null, equal to pixs,
- *                    or different from pixs)
- *              pixs (no restrictions on depth)
- *              liney  (location of horizontal line, measured from origin)
- *              angle (in radians)
- *              incolor (L_BRING_IN_WHITE, L_BRING_IN_BLACK);
- *      Return: pixd, always
+ * \param[in]    pixd [optional], this can be null, equal to pixs,
+ *                    or different from pixs
+ * \param[in]    pixs no restrictions on depth
+ * \param[in]    yloc location of horizontal line, measured from origin
+ * \param[in]    radang  angle in radians
+ * \param[in]    incolor L_BRING_IN_WHITE, L_BRING_IN_BLACK;
+ * \return  pixd, always
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) There are 3 cases:
  *            (a) pixd == null (make a new pixd)
  *            (b) pixd == pixs (in-place)
@@ -79,14 +91,14 @@ static l_float32 normalizeAngleForShear(l_float32 radang, l_float32 mindif);
  *              pixd = pixHShear(NULL, pixs, ...);
  *              pixHShear(pixs, pixs, ...);
  *              pixHShear(pixd, pixs, ...);
- *      (3) This shear leaves the horizontal line of pixels at y = liney
+ *      (3) This shear leaves the horizontal line of pixels at y = yloc
  *          invariant.  For a positive shear angle, pixels above this
  *          line are shoved to the right, and pixels below this line
  *          move to the left.
  *      (4) With positive shear angle, this can be used, along with
  *          pixVShear(), to perform a cw rotation, either with 2 shears
  *          (for small angles) or in the general case with 3 shears.
- *      (5) Changing the value of liney is equivalent to translating
+ *      (5) Changing the value of yloc is equivalent to translating
  *          the result horizontally.
  *      (6) This brings in 'incolor' pixels from outside the image.
  *      (7) For in-place operation, pixs cannot be colormapped,
@@ -95,11 +107,12 @@ static l_float32 normalizeAngleForShear(l_float32 radang, l_float32 mindif);
  *      (8) The angle is brought into the range [-pi, -pi].  It is
  *          not permitted to be within MIN_DIFF_FROM_HALF_PI radians
  *          from either -pi/2 or pi/2.
+ * </pre>
  */
 PIX *
 pixHShear(PIX       *pixd,
           PIX       *pixs,
-          l_int32    liney,
+          l_int32    yloc,
           l_float32  radang,
           l_int32    incolor)
 {
@@ -115,9 +128,9 @@ l_float32  tanangle, invangle;
         return (PIX *)ERROR_PTR("invalid incolor value", procName, pixd);
 
     if (pixd == pixs) {  /* in place */
-        if (pixGetColormap(pixs) != NULL)
+        if (pixGetColormap(pixs))
             return (PIX *)ERROR_PTR("pixs is colormapped", procName, pixd);
-        pixHShearIP(pixd, liney, radang, incolor);
+        pixHShearIP(pixd, yloc, radang, incolor);
         return pixd;
     }
 
@@ -125,9 +138,9 @@ l_float32  tanangle, invangle;
     if (!pixd) {
         if ((pixd = pixCreateTemplate(pixs)) == NULL)
             return (PIX *)ERROR_PTR("pixd not made", procName, NULL);
-    }
-    else  /* pixd != pixs */
+    } else {  /* pixd != pixs */
         pixResizeImageData(pixd, pixs);
+    }
 
         /* Normalize angle.  If no rotation, return a copy */
     radang = normalizeAngleForShear(radang, MIN_DIFF_FROM_HALF_PI);
@@ -140,14 +153,14 @@ l_float32  tanangle, invangle;
     pixGetDimensions(pixs, &w, &h, NULL);
     sign = L_SIGN(radang);
     tanangle = tan(radang);
-    invangle = L_ABS(1. / tanangle); 
+    invangle = L_ABS(1. / tanangle);
     inityincr = (l_int32)(invangle / 2.);
     yincr = (l_int32)invangle;
-    pixRasterop(pixd, 0, liney - inityincr, w, 2 * inityincr, PIX_SRC,
-                pixs, 0, liney - inityincr);
+    pixRasterop(pixd, 0, yloc - inityincr, w, 2 * inityincr, PIX_SRC,
+                pixs, 0, yloc - inityincr);
 
-    for (hshift = 1, y = liney + inityincr; y < h; hshift++) {
-        yincr = (l_int32)(invangle * (hshift + 0.5) + 0.5) - (y - liney);
+    for (hshift = 1, y = yloc + inityincr; y < h; hshift++) {
+        yincr = (l_int32)(invangle * (hshift + 0.5) + 0.5) - (y - yloc);
         if (h - y < yincr)  /* reduce for last one if req'd */
             yincr = h - y;
         pixRasterop(pixd, -sign*hshift, y, w, yincr, PIX_SRC, pixs, 0, y);
@@ -157,8 +170,8 @@ l_float32  tanangle, invangle;
         y += yincr;
     }
 
-    for (hshift = -1, y = liney - inityincr; y > 0; hshift--) {
-        yincr = (y - liney) - (l_int32)(invangle * (hshift - 0.5) + 0.5);
+    for (hshift = -1, y = yloc - inityincr; y > 0; hshift--) {
+        yincr = (y - yloc) - (l_int32)(invangle * (hshift - 0.5) + 0.5);
         if (y < yincr)  /* reduce for last one if req'd */
             yincr = y;
         pixRasterop(pixd, -sign*hshift, y - yincr, w, yincr, PIX_SRC,
@@ -172,20 +185,21 @@ l_float32  tanangle, invangle;
 
     return pixd;
 }
-                        
+
 
 /*!
- *  pixVShear()
+ * \brief   pixVShear()
  *
- *      Input:  pixd (<optional>, this can be null, equal to pixs,
- *                    or different from pixs)
- *              pixs (no restrictions on depth)
- *              linex  (location of vertical line, measured from origin)
- *              angle (in radians; not too close to +-(pi / 2))
- *              incolor (L_BRING_IN_WHITE, L_BRING_IN_BLACK);
- *      Return: pixd, or null on error
+ * \param[in]    pixd [optional], this can be null, equal to pixs,
+ *                    or different from pixs
+ * \param[in]    pixs no restrictions on depth
+ * \param[in]    xloc location of vertical line, measured from origin
+ * \param[in]    radang  angle in radians; not too close to +-(pi / 2)
+ * \param[in]    incolor L_BRING_IN_WHITE, L_BRING_IN_BLACK;
+ * \return  pixd, or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) There are 3 cases:
  *            (a) pixd == null (make a new pixd)
  *            (b) pixd == pixs (in-place)
@@ -194,14 +208,14 @@ l_float32  tanangle, invangle;
  *              pixd = pixVShear(NULL, pixs, ...);
  *              pixVShear(pixs, pixs, ...);
  *              pixVShear(pixd, pixs, ...);
- *      (3) This shear leaves the vertical line of pixels at x = linex
+ *      (3) This shear leaves the vertical line of pixels at x = xloc
  *          invariant.  For a positive shear angle, pixels to the right
  *          of this line are shoved downward, and pixels to the left
  *          of the line move upward.
  *      (4) With positive shear angle, this can be used, along with
  *          pixHShear(), to perform a cw rotation, either with 2 shears
  *          (for small angles) or in the general case with 3 shears.
- *      (5) Changing the value of linex is equivalent to translating
+ *      (5) Changing the value of xloc is equivalent to translating
  *          the result vertically.
  *      (6) This brings in 'incolor' pixels from outside the image.
  *      (7) For in-place operation, pixs cannot be colormapped,
@@ -210,11 +224,12 @@ l_float32  tanangle, invangle;
  *      (8) The angle is brought into the range [-pi, -pi].  It is
  *          not permitted to be within MIN_DIFF_FROM_HALF_PI radians
  *          from either -pi/2 or pi/2.
+ * </pre>
  */
 PIX *
 pixVShear(PIX       *pixd,
           PIX       *pixs,
-          l_int32    linex,
+          l_int32    xloc,
           l_float32  radang,
           l_int32    incolor)
 {
@@ -230,9 +245,9 @@ l_float32  tanangle, invangle;
         return (PIX *)ERROR_PTR("invalid incolor value", procName, NULL);
 
     if (pixd == pixs) {  /* in place */
-        if (pixGetColormap(pixs) != NULL)
+        if (pixGetColormap(pixs))
             return (PIX *)ERROR_PTR("pixs is colormapped", procName, pixd);
-        pixVShearIP(pixd, linex, radang, incolor);
+        pixVShearIP(pixd, xloc, radang, incolor);
         return pixd;
     }
 
@@ -240,9 +255,9 @@ l_float32  tanangle, invangle;
     if (!pixd) {
         if ((pixd = pixCreateTemplate(pixs)) == NULL)
             return (PIX *)ERROR_PTR("pixd not made", procName, NULL);
-    }
-    else  /* pixd != pixs */
+    } else {  /* pixd != pixs */
         pixResizeImageData(pixd, pixs);
+    }
 
         /* Normalize angle.  If no rotation, return a copy */
     radang = normalizeAngleForShear(radang, MIN_DIFF_FROM_HALF_PI);
@@ -255,14 +270,14 @@ l_float32  tanangle, invangle;
     pixGetDimensions(pixs, &w, &h, NULL);
     sign = L_SIGN(radang);
     tanangle = tan(radang);
-    invangle = L_ABS(1. / tanangle); 
+    invangle = L_ABS(1. / tanangle);
     initxincr = (l_int32)(invangle / 2.);
     xincr = (l_int32)invangle;
-    pixRasterop(pixd, linex - initxincr, 0, 2 * initxincr, h, PIX_SRC,
-                pixs, linex - initxincr, 0);
+    pixRasterop(pixd, xloc - initxincr, 0, 2 * initxincr, h, PIX_SRC,
+                pixs, xloc - initxincr, 0);
 
-    for (vshift = 1, x = linex + initxincr; x < w; vshift++) {
-        xincr = (l_int32)(invangle * (vshift + 0.5) + 0.5) - (x - linex);
+    for (vshift = 1, x = xloc + initxincr; x < w; vshift++) {
+        xincr = (l_int32)(invangle * (vshift + 0.5) + 0.5) - (x - xloc);
         if (w - x < xincr)  /* reduce for last one if req'd */
             xincr = w - x;
         pixRasterop(pixd, x, sign*vshift, xincr, h, PIX_SRC, pixs, x, 0);
@@ -272,8 +287,8 @@ l_float32  tanangle, invangle;
         x += xincr;
     }
 
-    for (vshift = -1, x = linex - initxincr; x > 0; vshift--) {
-        xincr = (x - linex) - (l_int32)(invangle * (vshift - 0.5) + 0.5);
+    for (vshift = -1, x = xloc - initxincr; x > 0; vshift--) {
+        xincr = (x - xloc) - (l_int32)(invangle * (vshift - 0.5) + 0.5);
         if (x < xincr)  /* reduce for last one if req'd */
             xincr = x;
         pixRasterop(pixd, x - xincr, sign*vshift, xincr, h, PIX_SRC,
@@ -287,25 +302,27 @@ l_float32  tanangle, invangle;
 
     return pixd;
 }
-                        
+
 
 
 /*-------------------------------------------------------------*
  *             Shears about UL corner and center               *
  *-------------------------------------------------------------*/
 /*!
- *  pixHShearCorner()
+ * \brief   pixHShearCorner()
  *
- *      Input:  pixd (<optional>, if not null, must be equal to pixs)
- *              pixs
- *              angle (in radians)
- *              incolor (L_BRING_IN_WHITE, L_BRING_IN_BLACK);
- *      Return: pixd, or null on error.
+ * \param[in]    pixd [optional], if not null, must be equal to pixs
+ * \param[in]    pixs
+ * \param[in]    radang  angle in radians
+ * \param[in]    incolor L_BRING_IN_WHITE, L_BRING_IN_BLACK;
+ * \return  pixd, or NULL on error.
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) See pixHShear() for usage.
  *      (2) This does a horizontal shear about the UL corner, with (+) shear
- *          pushing increasingly leftward (-x) with increasing y. 
+ *          pushing increasingly leftward (-x) with increasing y.
+ * </pre>
  */
 PIX *
 pixHShearCorner(PIX       *pixd,
@@ -323,18 +340,20 @@ pixHShearCorner(PIX       *pixd,
 
 
 /*!
- *  pixVShearCorner()
+ * \brief   pixVShearCorner()
  *
- *      Input:  pixd (<optional>, if not null, must be equal to pixs)
- *              pixs
- *              angle (in radians)
- *              incolor (L_BRING_IN_WHITE, L_BRING_IN_BLACK);
- *      Return: pixd, or null on error.
+ * \param[in]    pixd [optional], if not null, must be equal to pixs
+ * \param[in]    pixs
+ * \param[in]    radang  angle in radians
+ * \param[in]    incolor L_BRING_IN_WHITE, L_BRING_IN_BLACK;
+ * \return  pixd, or NULL on error.
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) See pixVShear() for usage.
  *      (2) This does a vertical shear about the UL corner, with (+) shear
- *          pushing increasingly downward (+y) with increasing x. 
+ *          pushing increasingly downward (+y) with increasing x.
+ * </pre>
  */
 PIX *
 pixVShearCorner(PIX       *pixd,
@@ -349,21 +368,23 @@ pixVShearCorner(PIX       *pixd,
 
     return pixVShear(pixd, pixs, 0, radang, incolor);
 }
-                        
+
 
 /*!
- *  pixHShearCenter()
+ * \brief   pixHShearCenter()
  *
- *      Input:  pixd (<optional>, if not null, must be equal to pixs)
- *              pixs
- *              angle (in radians)
- *              incolor (L_BRING_IN_WHITE, L_BRING_IN_BLACK);
- *      Return: pixd, or null on error.
+ * \param[in]    pixd [optional], if not null, must be equal to pixs
+ * \param[in]    pixs
+ * \param[in]    radang  angle in radians
+ * \param[in]    incolor L_BRING_IN_WHITE, L_BRING_IN_BLACK;
+ * \return  pixd, or NULL on error.
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) See pixHShear() for usage.
  *      (2) This does a horizontal shear about the center, with (+) shear
- *          pushing increasingly leftward (-x) with increasing y. 
+ *          pushing increasingly leftward (-x) with increasing y.
+ * </pre>
  */
 PIX *
 pixHShearCenter(PIX       *pixd,
@@ -381,18 +402,20 @@ pixHShearCenter(PIX       *pixd,
 
 
 /*!
- *  pixVShearCenter()
+ * \brief   pixVShearCenter()
  *
- *      Input:  pixd (<optional>, if not null, must be equal to pixs)
- *              pixs
- *              angle (in radians)
- *              incolor (L_BRING_IN_WHITE, L_BRING_IN_BLACK);
- *      Return: pixd, or null on error.
+ * \param[in]    pixd [optional], if not null, must be equal to pixs
+ * \param[in]    pixs
+ * \param[in]    radang  angle in radians
+ * \param[in]    incolor L_BRING_IN_WHITE, L_BRING_IN_BLACK;
+ * \return  pixd, or NULL on error.
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) See pixVShear() for usage.
  *      (2) This does a vertical shear about the center, with (+) shear
- *          pushing increasingly downward (+y) with increasing x. 
+ *          pushing increasingly downward (+y) with increasing x.
+ * </pre>
  */
 PIX *
 pixVShearCenter(PIX       *pixd,
@@ -414,25 +437,27 @@ pixVShearCenter(PIX       *pixd,
  *                       In place about arbitrary lines                     *
  *--------------------------------------------------------------------------*/
 /*!
- *  pixHShearIP()
+ * \brief   pixHShearIP()
  *
- *      Input:  pixs
- *              liney  (location of horizontal line, measured from origin)
- *              angle (in radians)
- *              incolor (L_BRING_IN_WHITE, L_BRING_IN_BLACK);
- *      Return: 0 if OK; 1 on error
+ * \param[in]    pixs
+ * \param[in]    yloc location of horizontal line, measured from origin
+ * \param[in]    radang  angle in radians
+ * \param[in]    incolor L_BRING_IN_WHITE, L_BRING_IN_BLACK;
+ * \return  0 if OK; 1 on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) This is an in-place version of pixHShear(); see comments there.
  *      (2) This brings in 'incolor' pixels from outside the image.
  *      (3) pixs cannot be colormapped, because the in-place operation
  *          only blits in 0 or 1 bits, not an arbitrary colormap index.
  *      (4) Does a horizontal full-band shear about the line with (+) shear
- *          pushing increasingly leftward (-x) with increasing y. 
+ *          pushing increasingly leftward (-x) with increasing y.
+ * </pre>
  */
 l_int32
 pixHShearIP(PIX       *pixs,
-            l_int32    liney,
+            l_int32    yloc,
             l_float32  radang,
             l_int32    incolor)
 {
@@ -446,7 +471,7 @@ l_float32  tanangle, invangle;
         return ERROR_INT("pixs not defined", procName, 1);
     if (incolor != L_BRING_IN_WHITE && incolor != L_BRING_IN_BLACK)
         return ERROR_INT("invalid incolor value", procName, 1);
-    if (pixGetColormap(pixs) != NULL)
+    if (pixGetColormap(pixs))
         return ERROR_INT("pixs is colormapped", procName, 1);
 
         /* Normalize angle */
@@ -457,22 +482,25 @@ l_float32  tanangle, invangle;
     sign = L_SIGN(radang);
     pixGetDimensions(pixs, &w, &h, NULL);
     tanangle = tan(radang);
-    invangle = L_ABS(1. / tanangle); 
+    invangle = L_ABS(1. / tanangle);
     inityincr = (l_int32)(invangle / 2.);
     yincr = (l_int32)invangle;
 
-    pixRasteropHip(pixs, liney - inityincr, 2 * inityincr, 0, incolor);
+    if (inityincr > 0)
+        pixRasteropHip(pixs, yloc - inityincr, 2 * inityincr, 0, incolor);
 
-    for (hshift = 1, y = liney + inityincr; y < h; hshift++) {
-        yincr = (l_int32)(invangle * (hshift + 0.5) + 0.5) - (y - liney);
+    for (hshift = 1, y = yloc + inityincr; y < h; hshift++) {
+        yincr = (l_int32)(invangle * (hshift + 0.5) + 0.5) - (y - yloc);
+        if (yincr == 0) continue;
         if (h - y < yincr)  /* reduce for last one if req'd */
             yincr = h - y;
         pixRasteropHip(pixs, y, yincr, -sign*hshift, incolor);
         y += yincr;
     }
 
-    for (hshift = -1, y = liney - inityincr; y > 0; hshift--) {
-        yincr = (y - liney) - (l_int32)(invangle * (hshift - 0.5) + 0.5);
+    for (hshift = -1, y = yloc - inityincr; y > 0; hshift--) {
+        yincr = (y - yloc) - (l_int32)(invangle * (hshift - 0.5) + 0.5);
+        if (yincr == 0) continue;
         if (y < yincr)  /* reduce for last one if req'd */
             yincr = y;
         pixRasteropHip(pixs, y - yincr, yincr, -sign*hshift, incolor);
@@ -481,28 +509,30 @@ l_float32  tanangle, invangle;
 
     return 0;
 }
-                        
+
 
 /*!
- *  pixVShearIP()
+ * \brief   pixVShearIP()
  *
- *      Input:  pixs (all depths; not colormapped)
- *              linex  (location of vertical line, measured from origin)
- *              angle (in radians)
- *              incolor (L_BRING_IN_WHITE, L_BRING_IN_BLACK);
- *      Return: 0 if OK; 1 on error
+ * \param[in]    pixs all depths; not colormapped
+ * \param[in]    xloc  location of vertical line, measured from origin
+ * \param[in]    radang  angle in radians
+ * \param[in]    incolor L_BRING_IN_WHITE, L_BRING_IN_BLACK;
+ * \return  0 if OK; 1 on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) This is an in-place version of pixVShear(); see comments there.
  *      (2) This brings in 'incolor' pixels from outside the image.
  *      (3) pixs cannot be colormapped, because the in-place operation
  *          only blits in 0 or 1 bits, not an arbitrary colormap index.
  *      (4) Does a vertical full-band shear about the line with (+) shear
- *          pushing increasingly downward (+y) with increasing x. 
+ *          pushing increasingly downward (+y) with increasing x.
+ * </pre>
  */
 l_int32
 pixVShearIP(PIX       *pixs,
-            l_int32    linex,
+            l_int32    xloc,
             l_float32  radang,
             l_int32    incolor)
 {
@@ -516,7 +546,7 @@ l_float32  tanangle, invangle;
         return ERROR_INT("pixs not defined", procName, 1);
     if (incolor != L_BRING_IN_WHITE && incolor != L_BRING_IN_BLACK)
         return ERROR_INT("invalid incolor value", procName, 1);
-    if (pixGetColormap(pixs) != NULL)
+    if (pixGetColormap(pixs))
         return ERROR_INT("pixs is colormapped", procName, 1);
 
         /* Normalize angle */
@@ -527,22 +557,25 @@ l_float32  tanangle, invangle;
     sign = L_SIGN(radang);
     pixGetDimensions(pixs, &w, &h, NULL);
     tanangle = tan(radang);
-    invangle = L_ABS(1. / tanangle); 
+    invangle = L_ABS(1. / tanangle);
     initxincr = (l_int32)(invangle / 2.);
     xincr = (l_int32)invangle;
 
-    pixRasteropVip(pixs, linex - initxincr, 2 * initxincr, 0, incolor);
+    if (initxincr > 0)
+        pixRasteropVip(pixs, xloc - initxincr, 2 * initxincr, 0, incolor);
 
-    for (vshift = 1, x = linex + initxincr; x < w; vshift++) {
-        xincr = (l_int32)(invangle * (vshift + 0.5) + 0.5) - (x - linex);
+    for (vshift = 1, x = xloc + initxincr; x < w; vshift++) {
+        xincr = (l_int32)(invangle * (vshift + 0.5) + 0.5) - (x - xloc);
+        if (xincr == 0) continue;
         if (w - x < xincr)  /* reduce for last one if req'd */
             xincr = w - x;
         pixRasteropVip(pixs, x, xincr, sign*vshift, incolor);
         x += xincr;
     }
 
-    for (vshift = -1, x = linex - initxincr; x > 0; vshift--) {
-        xincr = (x - linex) - (l_int32)(invangle * (vshift - 0.5) + 0.5);
+    for (vshift = -1, x = xloc - initxincr; x > 0; vshift--) {
+        xincr = (x - xloc) - (l_int32)(invangle * (vshift - 0.5) + 0.5);
+        if (xincr == 0) continue;
         if (x < xincr)  /* reduce for last one if req'd */
             xincr = x;
         pixRasteropVip(pixs, x - xincr, xincr, sign*vshift, incolor);
@@ -557,30 +590,32 @@ l_float32  tanangle, invangle;
  *              Linear interpolated shear about arbitrary lines            *
  *-------------------------------------------------------------------------*/
 /*!
- *  pixHShearLI()
+ * \brief   pixHShearLI()
  *
- *      Input:  pixs (8 bpp or 32 bpp, or colormapped)
- *              liney  (location of horizontal line, measured from origin)
- *              angle (in radians, in range (-pi/2 ... pi/2))
- *              incolor (L_BRING_IN_WHITE, L_BRING_IN_BLACK);
- *      Return: pixd (sheared), or null on error
+ * \param[in]    pixs 8 bpp or 32 bpp, or colormapped
+ * \param[in]    yloc location of horizontal line, measured from origin
+ * \param[in]    radang  angle in radians, in range (-pi/2 ... pi/2)
+ * \param[in]    incolor L_BRING_IN_WHITE, L_BRING_IN_BLACK;
+ * \return  pixd sheared, or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) This does horizontal shear with linear interpolation for
  *          accurate results on 8 bpp gray, 32 bpp rgb, or cmapped images.
  *          It is relatively slow compared to the sampled version
  *          implemented by rasterop, but the result is much smoother.
- *      (2) This shear leaves the horizontal line of pixels at y = liney
+ *      (2) This shear leaves the horizontal line of pixels at y = yloc
  *          invariant.  For a positive shear angle, pixels above this
  *          line are shoved to the right, and pixels below this line
  *          move to the left.
  *      (3) Any colormap is removed.
  *      (4) The angle is brought into the range [-pi/2 + del, pi/2 - del],
  *          where del == MIN_DIFF_FROM_HALF_PI.
+ * </pre>
  */
 PIX *
 pixHShearLI(PIX       *pixs,
-            l_int32    liney,
+            l_int32    yloc,
             l_float32  radang,
             l_int32    incolor)
 {
@@ -599,8 +634,8 @@ PIX       *pix, *pixd;
         return (PIX *)ERROR_PTR("pixs not 8, 32 bpp, or cmap", procName, NULL);
     if (incolor != L_BRING_IN_WHITE && incolor != L_BRING_IN_BLACK)
         return (PIX *)ERROR_PTR("invalid incolor value", procName, NULL);
-    if (liney < 0 || liney >= h)
-        return (PIX *)ERROR_PTR("liney not in [0 ... h-1]", procName, NULL);
+    if (yloc < 0 || yloc >= h)
+        return (PIX *)ERROR_PTR("yloc not in [0 ... h-1]", procName, NULL);
 
     if (pixGetColormap(pixs))
         pix = pixRemoveColormap(pixs, REMOVE_CMAP_BASED_ON_SRC);
@@ -628,7 +663,7 @@ PIX       *pix, *pixd;
     for (i = 0; i < h; i++) {
         lines = datas + i * wpls;
         lined = datad + i * wpld;
-        xshift = (liney - i) * tanangle;
+        xshift = (yloc - i) * tanangle;
         for (jd = 0; jd < w; jd++) {
             x = (l_int32)(64.0 * (-xshift + jd) + 0.5);
             xp = x / 64;
@@ -636,14 +671,14 @@ PIX       *pix, *pixd;
             wm = w - 1;
             if (xp < 0 || xp > wm) continue;
             if (d == 8) {
-                if (xp < wm)
+                if (xp < wm) {
                     val = ((63 - xf) * GET_DATA_BYTE(lines, xp) +
                            xf * GET_DATA_BYTE(lines, xp + 1) + 31) / 63;
-                else  /* xp == wm */
+                } else {  /* xp == wm */
                     val = GET_DATA_BYTE(lines, xp);
+                }
                 SET_DATA_BYTE(lined, jd, val);
-            }
-            else {  /* d == 32 */
+            } else {  /* d == 32 */
                 if (xp < wm) {
                     word0 = *(lines + xp);
                     word1 = *(lines + xp + 1);
@@ -654,9 +689,9 @@ PIX       *pix, *pixd;
                     bval = ((63 - xf) * ((word0 >> L_BLUE_SHIFT) & 0xff) +
                            xf * ((word1 >> L_BLUE_SHIFT) & 0xff) + 31) / 63;
                     composeRGBPixel(rval, gval, bval, lined + jd);
-                }
-                else  /* xp == wm */
+                } else {  /* xp == wm */
                     lined[jd] = lines[xp];
+                }
             }
         }
     }
@@ -667,30 +702,32 @@ PIX       *pix, *pixd;
 
 
 /*!
- *  pixVShearLI()
+ * \brief   pixVShearLI()
  *
- *      Input:  pixs (8 bpp or 32 bpp, or colormapped)
- *              linex  (location of vertical line, measured from origin)
- *              angle (in radians, in range (-pi/2 ... pi/2))
- *              incolor (L_BRING_IN_WHITE, L_BRING_IN_BLACK);
- *      Return: pixd (sheared), or null on error
+ * \param[in]    pixs 8 bpp or 32 bpp, or colormapped
+ * \param[in]    xloc  location of vertical line, measured from origin
+ * \param[in]    radang  angle in radians, in range (-pi/2 ... pi/2)
+ * \param[in]    incolor L_BRING_IN_WHITE, L_BRING_IN_BLACK;
+ * \return  pixd sheared, or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) This does vertical shear with linear interpolation for
  *          accurate results on 8 bpp gray, 32 bpp rgb, or cmapped images.
  *          It is relatively slow compared to the sampled version
  *          implemented by rasterop, but the result is much smoother.
- *      (2) This shear leaves the vertical line of pixels at x = linex
+ *      (2) This shear leaves the vertical line of pixels at x = xloc
  *          invariant.  For a positive shear angle, pixels to the right
  *          of this line are shoved downward, and pixels to the left
  *          of the line move upward.
  *      (3) Any colormap is removed.
  *      (4) The angle is brought into the range [-pi/2 + del, pi/2 - del],
  *          where del == MIN_DIFF_FROM_HALF_PI.
+ * </pre>
  */
 PIX *
 pixVShearLI(PIX       *pixs,
-            l_int32    linex,
+            l_int32    xloc,
             l_float32  radang,
             l_int32    incolor)
 {
@@ -709,8 +746,8 @@ PIX       *pix, *pixd;
         return (PIX *)ERROR_PTR("pixs not 8, 32 bpp, or cmap", procName, NULL);
     if (incolor != L_BRING_IN_WHITE && incolor != L_BRING_IN_BLACK)
         return (PIX *)ERROR_PTR("invalid incolor value", procName, NULL);
-    if (linex < 0 || linex >= w)
-        return (PIX *)ERROR_PTR("linex not in [0 ... w-1]", procName, NULL);
+    if (xloc < 0 || xloc >= w)
+        return (PIX *)ERROR_PTR("xloc not in [0 ... w-1]", procName, NULL);
 
     if (pixGetColormap(pixs))
         pix = pixRemoveColormap(pixs, REMOVE_CMAP_BASED_ON_SRC);
@@ -736,7 +773,7 @@ PIX       *pix, *pixd;
     wpld = pixGetWpl(pixd);
     tanangle = tan(radang);
     for (j = 0; j < w; j++) {
-        yshift = (j - linex) * tanangle;
+        yshift = (j - xloc) * tanangle;
         for (id = 0; id < h; id++) {
             y = (l_int32)(64.0 * (-yshift + id) + 0.5);
             yp = y / 64;
@@ -746,14 +783,14 @@ PIX       *pix, *pixd;
             lines = datas + yp * wpls;
             lined = datad + id * wpld;
             if (d == 8) {
-                if (yp < hm)
+                if (yp < hm) {
                     val = ((63 - yf) * GET_DATA_BYTE(lines, j) +
                            yf * GET_DATA_BYTE(lines + wpls, j) + 31) / 63;
-                else  /* yp == hm */
+                } else {  /* yp == hm */
                     val = GET_DATA_BYTE(lines, j);
+                }
                 SET_DATA_BYTE(lined, j, val);
-            }
-            else {  /* d == 32 */
+            } else {  /* d == 32 */
                 if (yp < hm) {
                     word0 = *(lines + j);
                     word1 = *(lines + wpls + j);
@@ -764,9 +801,9 @@ PIX       *pix, *pixd;
                     bval = ((63 - yf) * ((word0 >> L_BLUE_SHIFT) & 0xff) +
                            yf * ((word1 >> L_BLUE_SHIFT) & 0xff) + 31) / 63;
                     composeRGBPixel(rval, gval, bval, lined + j);
-                }
-                else  /* yp == hm */
+                } else {  /* yp == hm */
                     lined[j] = lines[j];
+                }
             }
         }
     }
@@ -794,16 +831,12 @@ l_float32  pi2;
 
        /* If angle is too close to pi/2 or -pi/2, move it */
     if (radang > pi2 - mindif) {
-        L_WARNING("angle close to pi/2; shifting away", procName);
+        L_WARNING("angle close to pi/2; shifting away\n", procName);
         radang = pi2 - mindif;
-    }
-    else if (radang < -pi2 + mindif) {
-        L_WARNING("angle close to -pi/2; shifting away", procName);
+    } else if (radang < -pi2 + mindif) {
+        L_WARNING("angle close to -pi/2; shifting away\n", procName);
         radang = -pi2 + mindif;
     }
 
     return radang;
 }
-
-
-
