@@ -241,7 +241,8 @@ pixWrite(const char  *fname,
          PIX         *pix,
          l_int32      format)
 {
-FILE  *fp;
+l_int32  ret;
+FILE    *fp;
 
     PROCNAME("pixWrite");
 
@@ -285,11 +286,10 @@ FILE  *fp;
 
 #endif  /* WRITE_AS_NAMED */
 
-    if (pixWriteStream(fp, pix, format)) {
-        fclose(fp);
-        return ERROR_INT("pix not written to stream", procName, 1);
-    }
+    ret = pixWriteStream(fp, pix, format);
     fclose(fp);
+    if (ret)
+        return ERROR_INT("pix not written to stream", procName, 1);
     return 0;
 }
 
@@ -957,7 +957,9 @@ char            fullpath[_MAX_PATH];
     } else if (var_DISPLAY_PROG == L_DISPLAY_WITH_OPEN) {
         snprintf(buffer, L_BUF_SIZE, "open %s &", tempname);
     }
+#ifndef OS_IOS /* iOS 11 does not support system() */
     ignore = system(buffer);
+#endif /* !OS_IOS */
 
 #else  /* _WIN32 */
 
@@ -1093,7 +1095,7 @@ PIX     *pix1, *pix2, *pix3, *pix4;
         pix2 = pixClone(pix1);
     } else if (scalefactor > 1.0) {
         pix2 = pixScale(pix1, scalefactor, scalefactor);
-    } else if (scalefactor < 1.0) {
+    } else {  /* scalefactor < 1.0) */
         if (pixGetDepth(pix1) == 1)
             pix2 = pixScaleToGray(pix1, scalefactor);
         else
@@ -1120,7 +1122,7 @@ PIX     *pix1, *pix2, *pix3, *pix4;
     } else if (newrow == 1) {
         top = bottom + space;
         left = 0;
-    } else if (n > 0) {
+    } else {  /* n > 0 */
         pixaGetBoxGeometry(pixa, n - 1, &bx, &by, &bw, NULL);
         top = by;
         left = bx + bw + space;
